import sys
import numpy as np
import os
import h5py
import multiprocessing as mp
from glob import glob
# from sklearn import cross_validation
from sklearn.model_selection import LeaveOneGroupOut
# import cPickle as pickle

from sklearn.preprocessing import StandardScaler
from sklearn.metrics import roc_auc_score
from sklearn.metrics import accuracy_score
# from sklearn.svm import SVR
# from sklearn.linear_model import Ridge
from sklearn.linear_model import LogisticRegression

run_on_rhino = True
# run_on_rhino = False

if run_on_rhino:
    num_mp_procs = 0
    rhino_mount = ''
else:
    num_mp_procs = 0
    rhino_mount = '/home/ctw/fusemounts/rhino'
homedir = rhino_mount+'/scratch/cweidema/LTP/ltpFR2'


pow_path_ltpFR2 = (
    rhino_mount +
    '/scratch/cweidema/LTP/ltpFR2/ltpFR2_preprocessed/pow15freq2_200hz/')


subj_folders = np.sort(glob(os.path.join(pow_path_ltpFR2, 'LTP???')))


classifierfolder_name = 'itemLR_nestedXval'
classifier_folder = (os.path.join(
    homedir, 'ltpFR2_classifiers/itemvar/',
    classifierfolder_name, 'hdf5_files'))


CS = np.array([(4.**i)/(4.**8) for i in range(0, 9)])  # = 1./ALPHAS


time_start = 0.3
time_end = 1.6
time_bins = [(time_start, time_end)]


lr_params = {'penalty':'l2', 'dual': False, 'tol': 0.0001, 'C': np.nan,
             'fit_intercept': True, 'intercept_scaling': 1,
             'class_weight': None, 'random_state': None,
             'solver': 'liblinear', 'max_iter': 100, 'multi_class': 'ovr',
             'verbose': 0, 'warm_start': False, 'n_jobs': 1}

FREQS = np.array([2., 2.77899099, 3.86139546, 5.36539159, 7.45518744,
                  10.35894936, 14.39371346, 20., 27.78990989, 38.61395458,
                  53.65391591, 74.55187441, 103.58949358, 143.9371346, 200.])
CHANS = np.array(
    ['002', '003', '004', '005', '006', '007', '009', '010', '011', '012',
     '013', '015', '016', '018', '019', '020', '022', '023', '024', '026',
     '027', '028', '029', '030', '031', '033', '034', '035', '036', '037',
     '038', '039', '040', '041', '042', '044', '045', '046', '047', '050',
     '051', '052', '053', '054', '055', '057', '058', '059', '060', '061',
     '062', '064', '065', '066', '067', '069', '070', '071', '072', '074',
     '075', '076', '077', '078', '079', '080', '082', '083', '084', '085',
     '086', '087', '089', '090', '091', '092', '093', '095', '096', '097',
     '098', '100', '101', '102', '103', '104', '105', '106', '108', '109',
     '110', '111', '112', '114', '115', '116', '117', '118', '121', '122',
     '123', '124', '129'])


def get_sessfeatures(sessdat, time_bins=time_bins): #, xtrafeatures={}):
    # sanity check that list and serial position numbers are in the right order
    assert(np.all(sessdat.dims[2]['trial'][:]==np.array(
        [i for i in range(1, 25) for j in range(24)])))
    assert(np.all(
        sessdat.dims[2]['serialpos'][:] == np.array(list(range(1, 25))*24)))
    assert(np.all(np.isclose(sessdat.dims[1]['freqs'][:], FREQS)))
    assert(np.all(np.array([c.decode()
                            for c in sessdat.dims[0]['channels']]) == CHANS))
    timebin_features = (len(sessdat.dims[0]['channels']) *
                        len(sessdat.dims[1]['freqs']))
    features = {}
    # features['data'] = np.ones((len(sessdat.dims[2]['subject']),
    #                             timebin_features * len(time_bins) +
    #                             len(xtrafeatures))) * np.nan
    features['data_unscaled'] = np.ones((len(sessdat.dims[2]['subject']),
                                timebin_features * len(time_bins))) * np.nan
    features['time_bin_labels'] = []
    features['channels'] = []
    features['freqs'] = []
    features['serialpos'] = []
    # features['xtrafeatures'] = xtrafeatures.keys()
    for t, tb in enumerate(time_bins):
        time_filter = ((sessdat.dims[3]['time'].value >= tb[0]) &
                       (sessdat.dims[3]['time'].value <= tb[1]))
        features['data_unscaled'][
            :, (t * timebin_features):((t + 1) * timebin_features)] = np.mean(
            sessdat[:, :, :, time_filter], 3).reshape(
                (len(sessdat.dims[0]['channels']) *
                 len(sessdat.dims[1]['freqs']),
                 len(sessdat.dims[2]['subject']))).T
        features['time_bin_labels'].extend(
            [str(tb[0]) + '-' + str(tb[1])] * timebin_features)
        features['serialpos'].extend(list(np.unique(sessdat.dims[2][
            'serialpos'])) * (len(sessdat.dims[0]['channels']) *
                              len(sessdat.dims[1]['freqs'])))
        for chan in sessdat.dims[0][0].value:
            features['channels'].extend(
                [chan] * (len(sessdat.dims[1]['freqs'])))
            features['freqs'].extend(sessdat.dims[1][0].value)
    # z transform
    scaler = StandardScaler()
    features['data_scaled'] = scaler.fit_transform(features['data_unscaled'])
    features['data_scalers'] = {'mean':scaler.mean_, 'std': scaler.scale_}
    return features


def classify(subjfolder, ClassifierModel=LogisticRegression, class_params=lr_params):
    subj = subjfolder.split('/')[-1]
    if int(subj[-3:]) > 330:
        return
    print(subj)
    sess_folders = np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_?')))
    sess_folders = np.r_[sess_folders, np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_??')))]
    ## get rid of Session 24 ("23" because of zero indexing):
    # get rid of Session 24 (zero indexing only for BioSemi subjects):
    if int(sess_folders[-1].split('_')[-1]) == 24:
        sess_folders = sess_folders[:-1]
    # if not (len(sess_folders) == 24 or
    #         ((len(sess_folders) == 23) and
    #          (sess_folders[-1].split('_')[-1]==23))):
    #     return
    if not (len(sess_folders) >= 20):
        return
    rfilename = os.path.join(classifier_folder,
                             'classifier_info_allfeat_'+subj+'.hdf5')
    if os.path.exists(rfilename):
        print(str(rfilename)+' already exists!')
        return
    # elif os.path.getsize(subjdatfile)<=1000000000:
    #     print(str(subjdatfile)+' too small:'+ str(os.path.getsize(subjdatfile)))
    #     return
    else:
        try:
            regressfile = h5py.File(rfilename, 'w-', libver='latest')
        except:
            print('Could not create: '+str(rfilename))
            return
    sessions = []
    lists = None
    recalled = []
    events = {}
    features = None
    # if len(sess_folders) == 24:
    #     sess_folders = sess_folders[:-1]
    for sess_folder in sess_folders:
        sess = sess_folder.split('_')[-1]
        sessdatfile = h5py.File(os.path.join(
            sess_folder, 'pow_'+subj+'_'+sess+'_allchans.hdf5'), 'r')
        if not np.all(np.isfinite(sessdatfile['data'].dims[2]['recalled'])):
            continue
        sessfeatures = get_sessfeatures(
            sessdatfile['data'], time_bins=time_bins)
        recalled.extend(sessdatfile['data'].dims[2]['recalled'])
        sessions.extend([sess] * len(
            sessdatfile['data'].dims[2]['recalled']))
        # sessdat = sessdatfile['data']
        if lists is None:
            lists = np.unique(sessdatfile['data'].dims[2]['trial'])
            for key in sessdatfile['events']:
                events[key] = sessdatfile['events'][key][:]
            # sessdatfile.copy('events', regressfile)
        else:
            assert(np.all(lists == np.unique(sessdatfile['data'].dims[2]['trial'])))
            for key in sessdatfile['events']:
                events[key] = np.r_[events[key], sessdatfile['events'][key][:]]
            # for evkey in regressfile['events']:
            #     origsize = regressfile['events'][evkey].size
            #     regressfile['events'][evkey].resize(
            #         origsize + sessdatfile['events'][evkey].size, axis=0)
            #     regressfile['events'][evkey][origsize:] =  sessdatfile[
            #         'events'][evkey][:]
        if features is None:
            features = sessfeatures
            # features.pop('data_scaler')
            features.pop('data_unscaled')
        else:
            features['data_scaled'] = np.r_[features['data_scaled'],
                                            sessfeatures['data_scaled']]
            for key in features['data_scalers']:
                features['data_scalers'][key] = np.r_[
                    np.atleast_2d(features['data_scalers'][key]),
                    np.atleast_2d(sessfeatures['data_scalers'][key])]
        sessdatfile.close()
    num_sess = len(np.unique(sessions))
    num_list = len(lists)
    num_word = len(np.unique(features['serialpos']))
    for key in events:
        regressfile.create_dataset(
        'events/'+key, data=events[key], compression="gzip",
            compression_opts=9)
    recalled = regressfile.create_dataset('recalled', data=events['recalled'],
                                          compression="gzip",
                                          compression_opts=9)
    prediction = regressfile.create_dataset(
        'prediction', (num_sess, num_list*num_word), dtype='float64',
        compression="gzip", compression_opts=9)
    prediction.attrs['subj'] = subj
    for rp in class_params:
        if class_params[rp] is None:
            prediction.attrs[rp] = 'None'
        else:
            prediction.attrs[rp] = class_params[rp]
    accuracy_score = regressfile.create_dataset(
        'accuracy_score', (num_sess,), dtype='float64',
        compression="gzip", compression_opts=9)
    auc_score = regressfile.create_dataset(
        'auc_score', (num_sess,), dtype='float64',
        compression="gzip", compression_opts=9)
    features_dat = regressfile.create_dataset(
        'features_dat', data=features['data_scaled'],
        compression="gzip", compression_opts=9)
    for c in CS:
        regressfile.create_dataset(
            'c_aucscores/'+str(c), (num_sess,), dtype='float64',
            compression="gzip", compression_opts=9)
    regressfile.create_dataset('best_cs', (num_sess,), dtype='float64',
                               compression="gzip", compression_opts=9)
    # scaler_mean = classfile.create_dataset(
    #     'scaler_mean', data=features['data_scaler']['mean'],
    #     compression="gzip", compression_opts=9)
    # scaler_std = classfile.create_dataset(
    #     'scaler_std', data=features['data_scaler']['std'],
    #     compression="gzip", compression_opts=9)
    for key in features['data_scalers']:
        regressfile.create_dataset(
            'features_scalers/'+key, data=features['data_scalers'][key],
            compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_chans', data=features['channels'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_freqs', data=features['freqs'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_serialpos', data=features['serialpos'],
        compression="gzip", compression_opts=9)
    # valevs = np.isfinite(recalled)
    all_train = []
    all_test = []
    logo = LeaveOneGroupOut()
    for session, (train, test) in enumerate(logo.split(
            # features_dat[valevs, :], recalled[valevs], np.array(sessions)[valevs])):
            features_dat, recalled, np.int16(sessions))):
        all_train.append(train)
        all_test.append(test)
        logo_nested = LeaveOneGroupOut()
        # alpha_rs = {}
        for c in CS:
            print(c)
            #nested_preds = []
            # nested_r2s = []
            # num_sess_nested = len(np.unique(sessions))
            # num_list = len(lists)
            nested_preds = np.empty((num_sess-1, num_list*num_word))
            nested_preds.fill(np.nan)
            nested_score = np.empty((num_sess-1,))
            nested_score.fill(np.nan)
            nested_recalled = np.empty((num_sess-1, num_list*num_word))
            nested_recalled.fill(np.nan)
            for session_nested, (train_nested, test_nested) in enumerate(
                    logo.split(features_dat.value[train],
                               recalled.value[train],
                               np.int16(sessions)[train])):
                params = class_params.copy()
                params['C'] = c
                reg = ClassifierModel(**params)
                # tmpreg = reg.fit(features_dat.value[train & valevs[train]][train_nested],
                #                  recalled.value[train & valevs[train]][train_nested])
                # nested_preds[session_nested] = tmpreg.predict_proba(
                #     features_dat.value[train & valevs[train]][test_nested])[:, 1]
                tmpreg = reg.fit(features_dat.value[train][train_nested],
                                 recalled.value[train][train_nested])
                nested_preds[session_nested] = tmpreg.predict_proba(
                    features_dat.value[train][test_nested])[:, 1]
                nested_score[session_nested] = tmpreg.score(
                    features_dat.value[train][test_nested],
                    recalled.value[train][test_nested])
                nested_recalled[session_nested] = recalled.value[train][test_nested]
            regressfile['c_aucscores/'+str(c)][session] = roc_auc_score(
                np.ravel(nested_recalled), np.ravel(nested_preds))
            # alpha_rs[alpha] = np.corrcoef(np.ravel(nested_preds), np.ravel(nested_prec))[0, 1]
        # best_alpha = max(alpha_rs, key=lambda key: alpha_rs[key])
        best_c = np.float(max(regressfile['c_aucscores'].keys(),
                                  key=lambda key:regressfile['c_aucscores/'+key][session]))
        regressfile['best_cs'][session] = best_c
        params = class_params.copy()
        params['C'] = best_c
        reg = ClassifierModel(**params)
        tmpreg = reg.fit(features_dat.value[train], recalled.value[train])
        prediction[session] = tmpreg.predict_proba(features_dat.value[test])[:, 1]
        accuracy_score[session] = tmpreg.score(features_dat.value[test],
                                               recalled.value[test])
        auc_score[session] = roc_auc_score(recalled.value[test],
                                           prediction[session])
        print(subj, session, auc_score[session])
    print(roc_auc_score(np.ravel(recalled), np.ravel(prediction)))
    # np.corrcoef(np.ravel(prediction), prec)
    regressfile.close()


for subjfolder in subj_folders:
    classify(subjfolder, ClassifierModel=LogisticRegression, class_params=lr_params)
