import sys
import numpy as np
import os
import h5py
import multiprocessing as mp
from glob import glob
import pandas as pd
import statsmodels.formula.api as smf

from sklearn.model_selection import LeaveOneGroupOut

from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import Ridge

run_on_rhino = True
# run_on_rhino = False

if run_on_rhino:
    num_mp_procs = 0
    rhino_mount = ''
else:
    num_mp_procs = 0
    rhino_mount = '/home/ctw/fusemounts/rhino'
homedir = rhino_mount+'/scratch/cweidema/LTP/ltpFR2'


pow_path_ltpFR2 = (
    rhino_mount +
    '/scratch/cweidema/LTP/ltpFR2/ltpFR2_preprocessed/pow15freq2_200hz/')

subj_folders = np.sort(glob(os.path.join(pow_path_ltpFR2, 'LTP???')))


regressorfolder_name = 'ridge_nestedXval_LR_pred_residuals_logit'
regressor_folder = (os.path.join(
    homedir, 'ltpFR2_regressors/listvar/',
    regressorfolder_name, 'hdf5_files'))


ALPHAS = np.array([4**i for i in np.arange(0., 9.)])

time_start = 0.3
time_end = 1.6
time_bins = [(time_start, time_end)]

ridge_params = {'alpha': np.nan, 'fit_intercept': True, 'normalize': False,
                'copy_X': True, 'max_iter': None, 'tol': 0.001,
                'solver': 'auto', 'random_state': None}


FREQS = np.array([2., 2.77899099, 3.86139546, 5.36539159, 7.45518744,
                  10.35894936, 14.39371346, 20., 27.78990989, 38.61395458,
                  53.65391591, 74.55187441, 103.58949358, 143.9371346, 200.])
CHANS = np.array(
    ['002', '003', '004', '005', '006', '007', '009', '010', '011', '012',
     '013', '015', '016', '018', '019', '020', '022', '023', '024', '026',
     '027', '028', '029', '030', '031', '033', '034', '035', '036', '037',
     '038', '039', '040', '041', '042', '044', '045', '046', '047', '050',
     '051', '052', '053', '054', '055', '057', '058', '059', '060', '061',
     '062', '064', '065', '066', '067', '069', '070', '071', '072', '074',
     '075', '076', '077', '078', '079', '080', '082', '083', '084', '085',
     '086', '087', '089', '090', '091', '092', '093', '095', '096', '097',
     '098', '100', '101', '102', '103', '104', '105', '106', '108', '109',
     '110', '111', '112', '114', '115', '116', '117', '118', '121', '122',
     '123', '124', '129'])


subjsess_info_path = (rhino_mount +
                      '/data/eeg/scalp/ltp/ltpFR2/cmldb_sess_info_ltpFR2.txt')
# subjsess_info = pd.read_table(subjsess_info_path)
subjsess_info = pd.read_csv(subjsess_info_path, sep='\t')

subjsess_info.loc[:, 'session'] += 1



# For now base recallability calculations on events stored in the item
# classifier HDF5 files. This assumes that the item classifiers are
# run first, but ensures that exactly the same events that are used
# for the regressors are the basis of the recallability
# calculation. Given that we're calculating recallability based on
# held-out data anyway, this criterion could be relaxed and we could
# get the events from the power files instead.
classifierfolder_name = 'itemLR_nestedXval'
classifier_folder = (os.path.join(
    homedir, 'ltpFR2_classifiers/itemvar/',
    classifierfolder_name, 'hdf5_files'))
classfiles = np.sort(glob(os.path.join(classifier_folder,
                                       'classifier_info_allfeat_LTP???.hdf5')))

# recallability_df = pd.DataFrame(index=np.array(np.unique(evdf['item']), dtype=np.str))
with h5py.File(classfiles[0], 'r') as classfile:
    recallability_within_df = pd.DataFrame(index=np.array(
        np.unique(classfile['events/item']), dtype=np.str))

# evdf_dict = {}
for cf in classfiles:
    with h5py.File(cf, 'r') as classfile:
        assert(len(np.unique(classfile['events/subject']))==1)
        subj = classfile['events/subject'][0]
        rec_dtypes = [(str(e), classfile['events'][e].dtype)
                      for e in classfile['events']]
        evarray = np.zeros(classfile['events/subject'].shape, rec_dtypes)
        for ev_val in classfile['events']:
            evarray[ev_val] = classfile['events'][ev_val]
        evdf = pd.DataFrame(evarray)
        try:
            recallability_within_df[subj] = np.array(
                [np.mean(evdf[np.array(evdf['item'], np.str) == word][
                    'recalled'])
                 for word in recallability_within_df.index])
        except KeyError:
            recallability_within_df[subj] = np.array(
                [np.mean(evdf[np.array(evdf['item_name'], np.str) == word][
                    'recalled'])
                 for word in recallability_within_df.index])


recallability_heldout_df = pd.DataFrame(index=recallability_within_df.index)
for subj in recallability_within_df:
    recallability_heldout_df[subj.decode()] = np.mean(
        recallability_within_df.loc[
            :, recallability_within_df.columns != subj], 1)






def get_sessfeatures(sessdat, time_bins=time_bins):
    # sanity check that list and serial position numbers are in the right order
    assert(np.all(sessdat.dims[2]['trial'][:]==np.array(
        [i for i in range(1, 25) for j in range(24)])))
    assert(np.all(
        sessdat.dims[2]['serialpos'][:] == np.array(list(range(1, 25))*24)))
    assert(np.all(np.isclose(sessdat.dims[1]['freqs'][:], FREQS)))
    assert(np.all(np.array(sessdat.dims[0]['channels'][:], np.str) == CHANS))
    timebin_features = (len(sessdat.dims[0]['channels']) *
                        len(sessdat.dims[1]['freqs']))
    features = {}
    # features['data'] = np.ones((len(sessdat.dims[2]['subject']),
    #                             timebin_features * len(time_bins) +
    #                             len(xtrafeatures))) * np.nan
    features['data_unscaled'] = np.ones((len(sessdat.dims[2]['subject']),
                                timebin_features * len(time_bins))) * np.nan
    features['time_bin_labels'] = []
    features['channels'] = []
    features['freqs'] = []
    features['serialpos'] = []
    # features['xtrafeatures'] = xtrafeatures.keys()
    for t, tb in enumerate(time_bins):
        time_filter = ((sessdat.dims[3]['time'][()] >= tb[0]) &
                       (sessdat.dims[3]['time'][()] <= tb[1]))
        features['data_unscaled'][
            :, (t * timebin_features):((t + 1) * timebin_features)] = np.mean(
            sessdat[:, :, :, time_filter], 3).reshape(
                (len(sessdat.dims[0]['channels']) *
                 len(sessdat.dims[1]['freqs']),
                 len(sessdat.dims[2]['subject']))).T
        features['time_bin_labels'].extend(
            [str(tb[0]) + '-' + str(tb[1])] * timebin_features)
        features['serialpos'].extend(list(np.unique(sessdat.dims[2][
            'serialpos'])) * (len(sessdat.dims[0]['channels']) *
                              len(sessdat.dims[1]['freqs'])))
        for chan in sessdat.dims[0][0][()]:
            features['channels'].extend(
                [chan] * (len(sessdat.dims[1]['freqs'])))
            features['freqs'].extend(sessdat.dims[1][0][()])
    # z transform
    scaler = StandardScaler()
    features['data_scaled'] = scaler.fit_transform(features['data_unscaled'])
    features['data_scalers'] = {'mean':scaler.mean_, 'std': scaler.scale_}
    return features


def logit(pred):
    # convert predictions from LR model (range between 0 and 1) to
    # logit
    pred = np.array(pred)
    # just in case any predictions are actually identical to 0 or 1:
    pred[pred==1] -= 10e-4
    pred[pred==0] += 10e-4
    return np.log(pred/(1-pred))

def logit_residuals(res):
    # convert residuals from LR model (range between -1 and 1) to
    # logit
    res = np.array(res)
    # transform to [0, 1]:
    res += 1
    res /= 2
    # just in case any transformed residuals are actually identical to
    # 0 or 1:
    res[res==1] -= 10e-4
    res[res==0] += 10e-4
    return np.log(res/(1-res))


def regress(subjfolder, RegModel=Ridge, reg_params=ridge_params):
    subj = subjfolder.split('/')[-1]
    if int(subj[-3:]) > 330:
        return
    print(subj)
    sess_folders = np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_?')))
    sess_folders = np.r_[sess_folders, np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_??')))]
    # # get rid of Session 24 ("23" because of zero indexing):
    # get rid of Session 24 (no zero indexing for EGI subjects):
    if int(sess_folders[-1].split('_')[-1]) == 24:
        sess_folders = sess_folders[:-1]
    if not (len(sess_folders) >= 20):
        return
    rfilename = os.path.join(
        regressor_folder,
        'regressor_info_allfeat_preds_residuals_'+subj+'.hdf5')
    if os.path.exists(rfilename):
        print(str(rfilename)+' already exists!')
        return
    else:
        try:
            regressfile = h5py.File(rfilename, 'w-', libver='latest')
        except:
            print('Could not create: '+str(rfilename))
            return
    sessions = []
    lists = None
    events = {}
    features = None

    subjdf = None
    for sess_folder in sess_folders:
        sess = sess_folder.split('_')[-1]
        with h5py.File(os.path.join(sess_folder, 'pow_'+subj+'_'+sess+
                                    '_allchans.hdf5'), 'r') as sessdatfile:
            tmp = []
            columns = []
            for key in sessdatfile['events']:
                tmp.append(sessdatfile['events'][key])
                columns.append(key)
            tmp = pd.DataFrame(tmp).T
            tmp.columns = columns
            if subjdf is None:
                subjdf = tmp
            else:
                subjdf = subjdf.append(
                    tmp, sort=True)
                # verify_integrity=True, sort=True)
    subjdf['recalled'] = subjdf['recalled'].astype(float)
    subjdf['itemno'] = subjdf['itemno'].astype(int)
    subjdf['serialpos'] = subjdf['serialpos'].astype(int)
    subjdf['session'] = subjdf['session'].astype(int)
    subjdf['trial'] = subjdf['trial'].astype(int)
    subjdf['block'] = ((subjdf['trial'].astype(int)-1) // 8) + 1
    subjdf['list'] = ((subjdf['trial'].astype(int)-1) % 8) + 1
    subjdf['list_squared'] = subjdf['list']**2
    subjdf['item'] = np.array([i.decode() for i in subjdf['item']])
    subjdf['subject'] = np.array([i.decode() for i in subjdf['subject']])
    tmp = recallability_heldout_df[subj]
    tmp.name = 'recallability'
    subjdf = subjdf.join(recallability_heldout_df[subj], 'item')
    subjdf = subjdf.merge(right=subjsess_info, how='left',
                          on=('subject', 'session'))
    subjdf['hrs_from_noon'] = -(12 - (subjdf.start_time % 100) /
                                60 - subjdf.start_time // 100)
    for sess_folder in sess_folders:
        sess = sess_folder.split('_')[-1]
        sessdatfile = h5py.File(os.path.join(
            sess_folder, 'pow_'+subj+'_'+sess+'_allchans.hdf5'), 'r')
        if not np.all(np.isfinite(sessdatfile['data'].dims[2]['recalled'])):
            continue
        # if len(np.unique(sessdatfile['data'].dims[2]['trial'])) != 24:
        #    continue
        sessfeatures = get_sessfeatures(
            sessdatfile['data'], time_bins=time_bins)
        sessions.extend([sess] * len(
            sessdatfile['data'].dims[2]['recalled']))
        if lists is None:
            lists = np.unique(sessdatfile['data'].dims[2]['trial'])
            for key in sessdatfile['events']:
                events[key] = sessdatfile['events'][key][:]
            # sessdatfile.copy('events', regressfile)
        else:
            assert(np.all(lists == np.unique(sessdatfile['data'].dims[2]['trial'])))
            for key in sessdatfile['events']:
                events[key] = np.r_[events[key], sessdatfile['events'][key][:]]
        if features is None:
            features = sessfeatures
            features.pop('data_unscaled')
        else:
            features['data_scaled'] = np.r_[features['data_scaled'],
                                            sessfeatures['data_scaled']]
            for key in features['data_scalers']:
                features['data_scalers'][key] = np.r_[
                    np.atleast_2d(features['data_scalers'][key]),
                    np.atleast_2d(sessfeatures['data_scalers'][key])]
        sessdatfile.close()
    sessions = np.int16(sessions)
    sessfilt = np.array([s in sessions for s in subjdf['session']])
    subjdf = subjdf[sessfilt]
    num_sess = len(np.unique(sessions))
    num_list = len(lists)
    num_word = len(np.unique(features['serialpos']))

    regresults = {
        'all': smf.logit(
            "recalled ~ recallability + C(serialpos) + C(session) + C(trial)",
            subjdf).fit(),
        'norecallability': smf.logit(
            "recalled ~ C(serialpos) + C(session) + C(trial)",
            subjdf).fit(),
        'noserialpos': smf.logit(
            "recalled ~ recallability + C(session) + C(trial)",
            subjdf).fit(),
        'nosession': smf.logit(
            "recalled ~ recallability + C(serialpos) + C(trial)",
            subjdf).fit(),
        'notrial': smf.logit(
            "recalled ~ recallability + C(serialpos) + C(session)",
            subjdf).fit()}
    for regmodel in regresults:
        subjdf['residuals_logit_'+regmodel] = logit_residuals(
            regresults[regmodel].resid_response)
        regressfile.create_dataset(
            'residuals_logit_'+regmodel, data=subjdf['residuals_logit_'+regmodel],
            compression="gzip",compression_opts=9)
        subjdf['prediction_from_responses_logit_'+regmodel] = logit(
            regresults[regmodel].predict(subjdf))
        regressfile.create_dataset(
            'prediction_from_responses_logit_'+regmodel, data=subjdf[
                'prediction_from_responses_logit_'+regmodel],
            compression="gzip",compression_opts=9)
        regressfile.create_dataset(
            'prediction_'+regmodel+'_res', data=np.ones((num_sess, num_list*num_word))*np.nan,
            dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'prediction_'+regmodel+'_pred', data=np.ones((num_sess, num_list*num_word))*np.nan,
            dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile['residuals_logit_'+regmodel].attrs['subj'] = subj
        regressfile['prediction_from_responses_logit_'+regmodel].attrs['subj'] = subj
        regressfile['prediction_'+regmodel+'_res'].attrs['subj'] = subj
        regressfile['prediction_'+regmodel+'_pred'].attrs['subj'] = subj
        regressfile.create_dataset(
            'r2score_'+regmodel+'_res', data=np.ones(num_sess)*np.nan, dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'r2score_'+regmodel+'_pred', data=np.ones(num_sess)*np.nan, dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'bestAlphas_'+regmodel+'_res', data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'bestAlphas_'+regmodel+'_pred', data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)        
    for key in events:
        if events[key].dtype.char == 'U':
            regressfile.create_dataset(
                'events/'+key, data=np.array([e.encode()
                                              for e in events[key]]),
                compression="gzip", compression_opts=9)
        else:
            regressfile.create_dataset(
                'events/'+key, data=events[key], compression="gzip",
                compression_opts=9)
    regressfile.create_dataset(
        'events/recallability', data=subjdf['recallability'],
        compression="gzip", compression_opts=9)
    
    regressfile.create_dataset(
        'recalled', data=subjdf['recalled'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'session', data=subjdf['session'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'sleep', data=subjdf['sleep'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'alertness', data=subjdf['alertness'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'hrs_from_noon', data=subjdf['hrs_from_noon'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'block', data=subjdf['block'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'list', data=subjdf['list'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'trial', data=subjdf['trial'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'serialpos', data=subjdf['serialpos'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'recallability_list', data=subjdf['recallability'],
        compression="gzip", compression_opts=9)
    for rp in reg_params:
        if reg_params[rp] is None:
            for regmodel in regresults:
                regressfile['prediction_'+regmodel+'_pred'].attrs[rp] = 'None'
                regressfile['prediction_'+regmodel+'_res'].attrs[rp] = 'None'
        else:
            for regmodel in regresults:
                regressfile['prediction_'+regmodel+'_pred'].attrs[rp] = reg_params[rp]
                regressfile['prediction_'+regmodel+'_res'].attrs[rp] = reg_params[rp]
    features_dat = regressfile.create_dataset(
        'features_dat', data=features['data_scaled'],
        compression="gzip", compression_opts=9)
    for alpha in ALPHAS:
        for regmodel in regresults:
            regressfile.create_dataset(
                'alphaRs_'+regmodel+'_res/'+str(alpha), data=np.ones(
                    num_sess)*np.nan,
                dtype='float64', compression="gzip", compression_opts=9)
            regressfile.create_dataset(
                'alphaRs_'+regmodel+'_pred/'+str(alpha), data=np.ones(
                    num_sess)*np.nan,
                dtype='float64', compression="gzip", compression_opts=9)
    for key in features['data_scalers']:
        regressfile.create_dataset(
            'features_scalers/'+key, data=features['data_scalers'][key],
            compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_chans', data=features['channels'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_freqs', data=features['freqs'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_serialpos', data=features['serialpos'],
        compression="gzip", compression_opts=9)
    dv_dict = {regmodel+'_res': regressfile['residuals_logit_'+regmodel]
               for regmodel in regresults}
    dv_dict.update({regmodel+'_pred': regressfile['prediction_from_responses_logit_'+regmodel]
                    for regmodel in regresults})
    logo = LeaveOneGroupOut()
    for dv in dv_dict:
        for session_i, (train, test) in enumerate(logo.split(
                features_dat[np.isfinite(dv_dict[dv]),:],
                dv_dict[dv][np.isfinite(dv_dict[dv])],
                sessions[np.isfinite(dv_dict[dv])])):
            session = np.unique(sessions[np.isfinite(dv_dict[dv])])[
                session_i]
            logo_nested = LeaveOneGroupOut()
            for alpha in ALPHAS:
                print(dv, alpha)
                nested_preds = np.empty((num_sess-1, num_list*num_word))
                nested_preds.fill(np.nan)
                nested_r2 = np.empty((num_sess-1,))
                nested_r2.fill(np.nan)
                nested_recalled = np.empty((num_sess-1, num_list*num_word))
                nested_recalled.fill(np.nan)
                for session_nested, (train_nested, test_nested) in enumerate(
                        logo_nested.split(
                            features_dat[np.isfinite(dv_dict[dv]),:][train],
                            dv_dict[dv][np.isfinite(dv_dict[dv])][train],
                            sessions[np.isfinite(dv_dict[dv])][
                                train])):
                    params = reg_params.copy()
                    params['alpha'] = alpha
                    reg = RegModel(**params)
                    tmpreg = reg.fit(
                        features_dat[np.isfinite(dv_dict[dv]),:][train][
                            train_nested], dv_dict[dv][np.isfinite(
                                dv_dict[dv])][train][train_nested])
                    nested_preds[session_nested] = tmpreg.predict(
                        features_dat[np.isfinite(dv_dict[dv]),:][train][
                            test_nested])
                    nested_r2[session_nested] = tmpreg.score(
                        features_dat[np.isfinite(dv_dict[dv]),:][train][
                            test_nested],
                        dv_dict[dv][np.isfinite(dv_dict[dv])][train][
                            test_nested])
                    nested_recalled[session_nested] = dv_dict[dv][
                        np.isfinite(dv_dict[dv])][train][test_nested]
                regressfile['alphaRs_'+dv+'/'+str(alpha)][
                    np.unique(regressfile['session']) == session] = np.corrcoef(
                        np.ravel(nested_preds),
                        np.ravel(nested_recalled))[0, 1]
            best_alpha = np.float(max(
                regressfile['alphaRs_'+dv].keys(),
                key=lambda key: regressfile['alphaRs_'+dv+'/'+key][
                    np.unique(regressfile['session']) == session]))
            regressfile['bestAlphas_'+dv][
                np.unique(regressfile['session']) == session] = best_alpha
            params = reg_params.copy()
            params['alpha'] = best_alpha
            reg = RegModel(**params)
            tmpreg = reg.fit(features_dat[np.isfinite(dv_dict[dv]), :][train],
                             dv_dict[dv][np.isfinite(dv_dict[dv])][train])
            sess_indx = np.where(np.unique(
                regressfile['session']) == session)[0][0]
            regressfile['prediction_'+dv][
                sess_indx] = tmpreg.predict(
                features_dat[np.isfinite(dv_dict[dv]), :][test])
            regressfile['r2score_'+dv][
                np.unique(regressfile['session'])==session] = tmpreg.score(
                features_dat[np.isfinite(dv_dict[dv]),:][test],
                dv_dict[dv][np.isfinite(dv_dict[dv])][test])
            print(subj, session)
        try:
            print(dv, np.corrcoef(
                np.ravel(regressfile['prediction_'+dv])[
                    np.isfinite(np.ravel(regressfile['prediction_'+dv]))],
                dv_dict[dv][np.isfinite(dv_dict[dv])]))
        except (IndexError, ValueError) as e:
            print(e) 
    try:
        print([(dv, np.corrcoef(
            np.ravel(regressfile['prediction_'+dv])[
                np.isfinite(np.ravel(regressfile['prediction_'+dv]))],
            dv_dict[dv][np.isfinite(dv_dict[dv])])) for dv in dv_dict])
    except (IndexError, ValueError) as e:
        print(e)
    regressfile.close()


for subjfolder in subj_folders:
    regress(subjfolder, RegModel=Ridge, reg_params=ridge_params)
