import sys
import numpy as np
import os
import h5py
import multiprocessing as mp
from glob import glob
from sklearn.model_selection import LeaveOneGroupOut

from sklearn.preprocessing import StandardScaler
from sklearn.metrics import roc_auc_score
from sklearn.metrics import accuracy_score
from sklearn.linear_model import LogisticRegression
import sys

run_on_rhino = True
# run_on_rhino = False

if run_on_rhino:
    num_mp_procs = 0
    rhino_mount = ''
    sys.path.insert(
        0, '/home1/cweidema/src/ptsa_new/build/lib.linux-x86_64-2.7/')
else:
    num_mp_procs = 0
    rhino_mount = '/home/ctw/fusemounts/rhino'
    sys.path.insert(
        0, '/home/ctw/src/ptsa_new/build/lib.linux-x86_64-2.7/')

from ptsa.data.timeseries import TimeSeries


homedir = rhino_mount+'/scratch/cweidema/LTP/ltpFR2'


pow_path_ltpFR2 = (
    rhino_mount +
    '/scratch/cweidema/LTP/ltpFR2/ltpFR2_preprocessed/pow15freq2_200hz/')


subj_folders = np.sort(glob(os.path.join(pow_path_ltpFR2, 'LTP???')))

classifierfolder_name = 'itemLR_nestedXval'
classifier_folder = (os.path.join(
    homedir, 'ltpFR2_classifiers/itemvar/',
    classifierfolder_name, 'hdf5_files'))

CS = np.array([(4.**i)/(4.**8) for i in range(0, 9)])  # = 1./ALPHAS


time_start = 0.3
time_end = 1.6
time_bins = [(time_start, time_end)]


lr_params = {'penalty':'l2', 'dual': False, 'tol': 0.0001, 'C': np.nan,
             'fit_intercept': True, 'intercept_scaling': 1,
             'class_weight': None, 'random_state': None,
             'solver': 'liblinear', 'max_iter': 100, 'multi_class': 'ovr',
             'verbose': 0, 'warm_start': False, 'n_jobs': 1}

FREQS = np.array([2., 2.77899099, 3.86139546, 5.36539159, 7.45518744,
                  10.35894936, 14.39371346, 20., 27.78990989, 38.61395458,
                  53.65391591, 74.55187441, 103.58949358, 143.9371346, 200.])

CHANS = ['A'+str(i) for i in range(1, 33)]
CHANS.extend(['B'+str(i) for i in range(1, 33)])
CHANS.extend(['C'+str(i) for i in range(1, 33)])
CHANS.extend(['D'+str(i) for i in range(1, 33)])

def get_sessfeatures(sessdat, time_bins=time_bins): #, xtrafeatures={}):
    # sanity check that list and serial position numbers are in the right order
    assert(np.all(sessdat['events'].values['trial'] == np.array(
        [i for i in range(1, 25) for j in range(24)])))
    assert(np.all(
        sessdat['events'].values['serialpos'][:] == np.array(range(1, 25)*24)))
    assert(np.all(np.isclose(sessdat['frequency'].values, FREQS)))
    assert(np.all(sessdat['channels'] == CHANS))
    timebin_features = (len(sessdat['channels']) *
                        len(sessdat['frequency']))
    features = {}
    # features['data'] = np.ones((len(sessdat.dims[2]['subject']),
    #                             timebin_features * len(time_bins) +
    #                             len(xtrafeatures))) * np.nan
    features['data_unscaled'] = np.ones(
        (len(sessdat['events'].values['subject']),
         timebin_features * len(time_bins))) * np.nan
    features['time_bin_labels'] = []
    features['channels'] = []
    features['freqs'] = []
    features['serialpos'] = []
    # features['xtrafeatures'] = xtrafeatures.keys()
    for t, tb in enumerate(time_bins):
        time_filter = ((sessdat['time'].values >= tb[0]) &
                       (sessdat['time'].values <= tb[1]))
        features_ts = sessdat.sel(time=time_filter).mean('time').stack(
            features=('channels', 'frequency'))
        features['data_unscaled'][
            :,
            (t * timebin_features):((t + 1) *timebin_features)] = features_ts
        features['time_bin_labels'].extend(
            [str(tb[0]) + '-' + str(tb[1])] * timebin_features)
        features['serialpos'].extend(list(np.unique(features_ts[
            'events'].values['serialpos'])) * (len(sessdat['channels']) *
                                               len(sessdat['frequency'])))
        # currently stacking changes the dtype of record arrays to
        # object, so need to create new record array with correct
        # dtype:
        features['channels'].extend(
            np.array(features_ts['features']['channels'], np.str))
            #                         np.array([
            # str(label) for label in np.array(features_ts['features'][
            #     'channels'], sessdat['channels'].dtype)['label']]))
        features['freqs'].extend(features_ts['features']['frequency'])
    # z transform
    scaler = StandardScaler()
    features['data_scaled'] = scaler.fit_transform(features['data_unscaled'])
    features['data_scalers'] = {'mean':scaler.mean_, 'std': scaler.scale_}
    return features


def classify(subjfolder, ClassifierModel=LogisticRegression, class_params=lr_params):
    subj = subjfolder.split('/')[-1]
    if int(subj[-3:]) < 331:
        return
    print subj
    sess_folders = np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_?')))
    sess_folders = np.r_[sess_folders, np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_??')))]
    # get rid of Session 24 ("23" because of zero indexing):
    if int(sess_folders[-1].split('_')[-1]) == 23:
        sess_folders = sess_folders[:-1]
    # if not (len(sess_folders) == 24 or
    #         ((len(sess_folders) == '23') and
    #          (sess_folders[-1].split('_')[-1]==23))):
    #     return
    if not (len(sess_folders) >= 20):
        return
    rfilename = os.path.join(classifier_folder,
                             'classifier_info_allfeat_'+subj+'.hdf5')
    if os.path.exists(rfilename):
        print(str(rfilename)+' already exists!')
        return
    # elif os.path.getsize(subjdatfile)<=1000000000:
    #     print(str(subjdatfile)+' too small:'+ str(os.path.getsize(subjdatfile)))
    #     return
    else:
        try:
            regressfile = h5py.File(rfilename, 'w-', libver='latest')
        except:
            print('Could not create: '+str(rfilename))
            return
    sessions = []
    lists = None
    recalled = []
    events = {}
    features = None
    # if len(sess_folders) == 24:
    #     sess_folders = sess_folders[:-1]
    for sess_folder in sess_folders:
        sess = sess_folder.split('_')[-1]
        sessdat = TimeSeries.from_hdf(os.path.join(
            sess_folder, 'pow_'+subj+'_'+sess+'_allchans.hdf5'))
        if not np.all(np.isfinite(sessdat['events'].values['recalled'])):
            continue
        if len(np.unique(sessdat['events'].values['trial'])) != 24:
            continue
        sessfeatures = get_sessfeatures(
            sessdat, time_bins=time_bins)
        recalled.extend(sessdat['events'].values['recalled'])
        sessions.extend([sess] * len(
            sessdat['events'].values['recalled']))
        if lists is None:
            lists = np.unique(sessdat['events'].values['trial'])
            for key in sessdat['events'].dtype.names:
                if sessdat['events'].values[key].dtype.char == 'U':
                    events[key] = np.array(sessdat['events'].values[key], np.str)
                else:
                    events[key] = sessdat['events'].values[key]
        else:
            assert(np.all(lists == np.unique(sessdat['events'].values['trial'])))
            for key in sessdat['events'].dtype.names:
                if sessdat['events'].values[key].dtype.char == 'U':
                    events[key] = np.r_[
                        events[key], np.array(sessdat['events'].values[key],
                                              np.str)]
                else:
                    events[key] = np.r_[events[key], sessdat['events'].values[key]]
        if features is None:
            features = sessfeatures
            features.pop('data_unscaled')
        else:
            features['data_scaled'] = np.r_[features['data_scaled'],
                                            sessfeatures['data_scaled']]
            for key in features['data_scalers']:
                features['data_scalers'][key] = np.r_[
                    np.atleast_2d(features['data_scalers'][key]),
                    np.atleast_2d(sessfeatures['data_scalers'][key])]
        sessdat.close()
    num_sess = len(np.unique(sessions))
    num_list = len(lists)
    num_word = len(np.unique(features['serialpos']))
    for key in events:
        regressfile.create_dataset(
        'events/'+key, data=events[key], compression="gzip",
            compression_opts=9)
    recalled = regressfile.create_dataset('recalled', data=events['recalled'],
                                          compression="gzip",
                                          compression_opts=9)
    prediction = regressfile.create_dataset(
        'prediction', (num_sess, num_list*num_word), dtype='float64',
        compression="gzip", compression_opts=9)
    prediction.attrs['subj'] = subj
    for rp in class_params:
        if class_params[rp] is None:
            prediction.attrs[rp] = 'None'
        else:
            prediction.attrs[rp] = class_params[rp]
    accuracy_score = regressfile.create_dataset(
        'accuracy_score', (num_sess,), dtype='float64',
        compression="gzip", compression_opts=9)
    auc_score = regressfile.create_dataset(
        'auc_score', (num_sess,), dtype='float64',
        compression="gzip", compression_opts=9)
    features_dat = regressfile.create_dataset(
        'features_dat', data=features['data_scaled'],
        compression="gzip", compression_opts=9)
    for c in CS:
        regressfile.create_dataset(
            'c_aucscores/'+str(c), (num_sess,), dtype='float64',
            compression="gzip", compression_opts=9)
    regressfile.create_dataset('best_cs', (num_sess,), dtype='float64',
                               compression="gzip", compression_opts=9)
    # scaler_mean = classfile.create_dataset(
    #     'scaler_mean', data=features['data_scaler']['mean'],
    #     compression="gzip", compression_opts=9)
    # scaler_std = classfile.create_dataset(
    #     'scaler_std', data=features['data_scaler']['std'],
    #     compression="gzip", compression_opts=9)
    for key in features['data_scalers']:
        regressfile.create_dataset(
            'features_scalers/'+key, data=features['data_scalers'][key],
            compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_chans', data=features['channels'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_freqs', data=features['freqs'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_serialpos', data=features['serialpos'],
        compression="gzip", compression_opts=9)
    # valevs = np.isfinite(recalled)
    all_train = []
    all_test = []
    logo = LeaveOneGroupOut()
    for session, (train, test) in enumerate(logo.split(
            # features_dat[valevs, :], recalled[valevs], np.array(sessions)[valevs])):
            features_dat, recalled, np.int16(sessions))):
        all_train.append(train)
        all_test.append(test)
        logo_nested = LeaveOneGroupOut()
        # alpha_rs = {}
        for c in CS:
            print(c)
            #nested_preds = []
            # nested_r2s = []
            # num_sess_nested = len(np.unique(sessions))
            # num_list = len(lists)
            nested_preds = np.empty((num_sess-1, num_list*num_word))
            nested_preds.fill(np.nan)
            nested_score = np.empty((num_sess-1,))
            nested_score.fill(np.nan)
            nested_recalled = np.empty((num_sess-1, num_list*num_word))
            nested_recalled.fill(np.nan)
            for session_nested, (train_nested, test_nested) in enumerate(
                    logo.split(features_dat.value[train],
                               recalled.value[train],
                               np.int16(sessions)[train])):
                params = class_params.copy()
                params['C'] = c
                reg = ClassifierModel(**params)
                tmpreg = reg.fit(features_dat.value[train][train_nested],
                                 recalled.value[train][train_nested])
                nested_preds[session_nested] = tmpreg.predict_proba(
                    features_dat.value[train][test_nested])[:, 1]
                nested_score[session_nested] = tmpreg.score(
                    features_dat.value[train][test_nested],
                    recalled.value[train][test_nested])
                nested_recalled[session_nested] = recalled.value[train][test_nested]
            regressfile['c_aucscores/'+str(c)][session] = roc_auc_score(
                np.ravel(nested_recalled), np.ravel(nested_preds))
        best_c = np.float(max(regressfile['c_aucscores'].keys(),
                                  key=lambda key:regressfile['c_aucscores/'+key][session]))
        regressfile['best_cs'][session] = best_c
        params = class_params.copy()
        params['C'] = best_c
        reg = ClassifierModel(**params)
        tmpreg = reg.fit(features_dat.value[train], recalled.value[train])
        prediction[session] = tmpreg.predict_proba(features_dat.value[test])[:, 1]
        accuracy_score[session] = tmpreg.score(features_dat.value[test],
                                               recalled.value[test])
        auc_score[session] = roc_auc_score(recalled.value[test],
                                           prediction[session])
        print subj, session, auc_score[session]
    print roc_auc_score(np.ravel(recalled), np.ravel(prediction))
    # np.corrcoef(np.ravel(prediction), prec)
    regressfile.close()


for subjfolder in subj_folders:
    classify(subjfolder, ClassifierModel=LogisticRegression, class_params=lr_params)
