import sys
import numpy as np
import scipy as sp
import os
import h5py
import numpy as np
from glob import glob
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.gridspec as gridspec
from sklearn.metrics import roc_auc_score
# from scipy.io import loadmat
#from pandas import read_table
import pandas as pd
import xarray as xr
import seaborn as sns
from scipy.stats import ttest_rel, ttest_1samp
import scipy.stats as stats
import matplotlib.lines as mlines

import matplotlib as mpl
mpl.rcParams['pdf.fonttype'] = 42
mpl.rcParams['ps.fonttype'] = 42

# run_on_rhino = True
run_on_rhino = False
# homedir = '/media/ctw/intbkup1/rhino_eeg/LTP/ltpFR2'
homedir = None
# homedir = '/home/ctw/fusemounts/swan/media/ctw/intbkup1/rhino_eeg/LTP/ltpFR2'

if run_on_rhino:
    num_mp_procs = 0
    rhino_mount = ''
else:
    num_mp_procs = 0
    rhino_mount = '/home/ctw/fusemounts/rhino'
if homedir is None:
    homedir = rhino_mount+'/scratch/cweidema/LTP/ltpFR2'


# sys.path.append('/home/ctw/src/ptsa_plot/ptsa_plot/')
# from topo import topoplot

sensors = {}

with open('/home/ctw/src/ptsa_plot/ptsa_plot/electrode_layouts/HCGSN128.dat', 'r') as sensorfile:
    angles, radii = sensorfile.readlines()
    # sensors = (np.r_[np.array([np.float(a)
    #                            for a in angles.strip().split()])*-1+90, 0],
    #            np.r_[[np.float(r) for r in radii.strip().split()], 0])
    used_sensors_indx = np.r_[
        np.arange(2, 8), np.arange(9, 14), np.arange(15, 17), np.arange(18, 21),
        np.arange(22, 25), np.arange(26, 32), np.arange(33, 43), np.arange(44, 48),
        np.arange(50, 56), np.arange(57, 63), np.arange(64, 68), np.arange(69, 73),
        np.arange(74, 81), np.arange(82, 88), np.arange(89, 94), np.arange(95, 99),
        np.arange(100, 107), np.arange(108, 113), np.arange(114, 119),
        np.arange(121, 125), [129]] - 1 
    sensors['egi'] = (
        np.r_[[np.float(a) for a in angles.strip().split()], 0][
            used_sensors_indx],
        np.r_[[np.float(r) for r in radii.strip().split()], 0][
            used_sensors_indx])

with open('/home/ctw/src/ptsa_plot/ptsa_plot/electrode_layouts/biosemi128.dat', 'r') as sensorfile:
    angles, radii = sensorfile.readlines()
    bs_chanlabels = ['A1', 'A2', 'A3', 'A4', 'A5', 'A6', 'A7', 'A8', 'A9', 'A10',
                     'A11', 'A12', 'A13', 'A14', 'A15', 'A16', 'A17', 'A18', 'A19',
                     'A20', 'A21', 'A22', 'A23', 'A24', 'A25', 'A26', 'A27', 'A28',
                     'A29', 'A30', 'A31', 'A32', 'B1', 'B2', 'B3', 'B4', 'B5', 'B6',
                     'B7', 'B8', 'B9', 'B10', 'B11', 'B12', 'B13', 'B14', 'B15',
                     'B16', 'B17', 'B18', 'B19', 'B20', 'B21', 'B22', 'B23', 'B24',
                     'B25', 'B26', 'B27', 'B28', 'B29', 'B30', 'B31', 'B32', 'C1',
                     'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9', 'C10', 'C11',
                     'C12', 'C13', 'C14', 'C15', 'C16', 'C17', 'C18', 'C19', 'C20',
                     'C21', 'C22', 'C23', 'C24', 'C25', 'C26', 'C27', 'C28', 'C29',
                     'C30', 'C31', 'C32', 'D1', 'D2', 'D3', 'D4', 'D5', 'D6', 'D7',
                     'D8', 'D9', 'D10', 'D11', 'D12', 'D13', 'D14', 'D15', 'D16',
                     'D17', 'D18', 'D19', 'D20', 'D21', 'D22', 'D23', 'D24', 'D25',
                     'D26', 'D27', 'D28', 'D29', 'D30', 'D31', 'D32']
    # the unstacking in corr_t uses the channel labels in lexiographic
    # order, so we'll order the coordinates likewise to keep
    # alignment:
    sensors['biosemi'] = (
        np.r_[[np.float(a) for a in angles.strip().split()], 0][np.argsort(bs_chanlabels)],
        np.r_[[np.float(r) for r in radii.strip().split()], 0][np.argsort(bs_chanlabels)])


regressorfolder_itemvar_name = 'ridge_nestedXval_LR_pred_residuals_logit'
regressor_folder_itemvar = (os.path.join(
    homedir, 'ltpFR2_regressors/itemvar/',
    regressorfolder_itemvar_name, 'hdf5_files'))

regfiles_itemvar = np.sort(glob(os.path.join(
    regressor_folder_itemvar,
    'regressor_info_allfeat_preds_residuals_LTP???.hdf5')))





classifierfolder_name = 'itemLR_nestedXval'
classifierfolder_listagg_name = 'itemLR_listagg_nestedXval'
classifier_folder = (os.path.join(
    homedir, 'ltpFR2_classifiers/itemvar/',
    classifierfolder_name, 'hdf5_files'))
classifier_listagg_folder = (os.path.join(
    homedir, 'ltpFR2_classifiers/itemvar/',
    classifierfolder_listagg_name, 'hdf5_files'))


classfiles = np.sort(glob(os.path.join(
    classifier_folder, 'classifier_info_allfeat_LTP???.hdf5')))
classfiles_listagg = np.sort(glob(os.path.join(
    classifier_folder, 'classifier_info_allfeat_LTP???.hdf5')))


regressorfolder_listvar_name = 'ridge_nestedXval_pred_residuals_catpredictors_logit'
regressor_folder_listvar = (os.path.join(
    homedir, 'ltpFR2_regressors/listvar/',
    regressorfolder_listvar_name, 'hdf5_files'))
regfiles_listvar = np.sort(glob(os.path.join(regressor_folder_listvar, 'regressor_info_allfeat_LTP???.hdf5')))

regmodels = {
    'item_res_all': {'dv': 'residuals_logit_all',
                     'pred': 'prediction_all_res'},
    'item_res_norecallability': {'dv': 'residuals_logit_norecallability',
                                 'pred': 'prediction_norecallability_res'},
    'item_res_noserialpos': {'dv': 'residuals_logit_noserialpos',
                             'pred': 'prediction_noserialpos_res'},
    'item_res_nosession': {'dv': 'residuals_logit_nosession',
                           'pred': 'prediction_nosession_res'},
    'item_res_notrial': {'dv': 'residuals_logit_notrial',
                         'pred': 'prediction_notrial_res'},
    'item_pred_all': {'dv': 'prediction_from_responses_logit_all',
                      'pred': 'prediction_all_pred'},
    'item_pred_norecallability': {
        'dv': 'prediction_from_responses_logit_norecallability',
        'pred': 'prediction_norecallability_pred'},
    'item_pred_noserialpos': {
        'dv': 'prediction_from_responses_logit_noserialpos',
        'pred': 'prediction_noserialpos_pred'},
    'item_pred_nosession': {
        'dv': 'prediction_from_responses_logit_nosession',
        'pred': 'prediction_nosession_pred'},
    'item_pred_notrial': {
        'dv': 'prediction_from_responses_logit_notrial',
        'pred': 'prediction_notrial_pred'},
    'raw': {'dv': 'recalled_logit', 'pred': 'prediction_raw'},
    'raw_shuffled': {'dv': 'recalled_logit',
                     'pred': 'prediction_raw_shuffled'},
    'residuals_list': {'dv': 'residuals_list',
                       'pred': 'prediction_list_res'},
    'residuals_list_shuffled': {'dv': 'residuals_list',
                                'pred': 'prediction_list_res_shuffled'},
    'residuals_sess': {'dv': 'residuals_sess',
                       'pred': 'prediction_sess_res'},
    'residuals_sess_shuffled': {'dv': 'residuals_sess',
                                'pred': 'prediction_sess_res_shuffled'},
    'residuals_sesslist': {'dv': 'residuals_sesslist',
                           'pred': 'prediction_sesslist_res'},
    'residuals_sesslist_shuffled': {
        'dv': 'residuals_sesslist',
        'pred': 'prediction_sesslist_res_shuffled'},
    'prediction_list': {'dv': 'prediction_from_responses_list',
                        'pred': 'prediction_list_pred'},
    'prediction_list_shuffled': {'dv': 'prediction_from_responses_list',
                                 'pred': 'prediction_list_pred_shuffled'},
    'prediction_sess': {'dv': 'prediction_from_responses_sess',
                        'pred': 'prediction_sess_pred'},
    'prediction_sess_shuffled': {'dv': 'prediction_from_responses_sess',
                                 'pred': 'prediction_sess_pred_shuffled'},
    'prediction_sesslist': {'dv': 'prediction_from_responses_sesslist',
                            'pred': 'prediction_sesslist_pred'},
    'prediction_sesslist_shuffled': {
        'dv': 'prediction_from_responses_sesslist',
        'pred': 'prediction_sesslist_pred_shuffled'}}

itemvars = ['item_res_all', 'item_res_norecallability', 'item_res_noserialpos',
            'item_res_nosession', 'item_res_notrial', 'item_pred_all',
            'item_pred_norecallability', 'item_pred_noserialpos',
            'item_pred_nosession', 'item_pred_notrial']

metrics = {'subject': [], 'auc_scores': {'item': [], 'item_listagg': []},
           'corrs': {'item': [], 'item_listagg': []}}
for regmodel in regmodels:
    metrics['auc_scores'][regmodel] = []
    metrics['corrs'][regmodel] = []
    if regmodel == 'raw':
        continue
    if 'shuffled' in regmodel:
        continue
    if regmodel in itemvars:
        metrics['auc_scores']['item-'+regmodel] = []
        metrics['corrs']['item-'+regmodel] = []
        metrics['auc_scores'][regmodel+'-item'] = []
        metrics['corrs'][regmodel+'-item'] = []
    else:
        metrics['auc_scores']['list-'+regmodel] = []
        metrics['corrs']['list-'+regmodel] = []
        metrics['auc_scores'][regmodel+'-list'] = []
        metrics['corrs'][regmodel+'-list'] = []
    



shuffles = 20

for rf_itemvar in regfiles_itemvar:
    rfdat_itemvar = h5py.File(rf_itemvar, 'r')
    # if not np.all(np.isfinite(rfdat[dv['intersess']])):
    #     continue
    metrics['subject'].append(rfdat_itemvar['events/subject'][0].decode())
    cf = np.sort(glob(os.path.join(
        classifier_folder, 'classifier_info_allfeat_'+
        rfdat_itemvar['events/subject'][0].decode()+'.hdf5')))
    cf_la = np.sort(glob(os.path.join(
        classifier_listagg_folder, 'classifier_info_allfeat_'+
        rfdat_itemvar['events/subject'][0].decode()+'.hdf5')))
    rf_listvar = np.sort(glob(os.path.join(
        regressor_folder_listvar, 'regressor_info_allfeat_'+
        rfdat_itemvar['events/subject'][0].decode()+'.hdf5')))
    if len(cf) != 1:
        raise ValueError('Invalid file count: '+str(cf))
    else:
        cf = cf[0]
    if len(cf_la) != 1:
        raise ValueError('Invalid file count: '+str(cf_la))
    else:
        cf_la = cf_la[0]
    if len(rf_listvar) != 1:
        raise ValueError('Invalid file count: '+str(rf_listvar))
    else:
        rf_listvar = rf_listvar[0]
    cfdat = h5py.File(cf, 'r')
    cfdat_la = h5py.File(cf_la, 'r')
    rfdat_listvar = h5py.File(rf_listvar, 'r')
    metrics['auc_scores']['item_listagg'].append(
        roc_auc_score(cfdat_la['recalled'], np.ravel(
            cfdat_la['prediction'])[cfdat_la['list_filters']]))
    metrics['corrs']['item_listagg'].append(
        np.corrcoef(cfdat_la['recalled'], np.ravel(
            cfdat_la['prediction'])[cfdat_la['list_filters']])[0,1])
    metrics['auc_scores']['item'].append(
        roc_auc_score(cfdat['recalled'], np.ravel(cfdat['prediction'])))
    metrics['corrs']['item'].append(
        np.corrcoef(cfdat['recalled'], np.ravel(cfdat['prediction']))[0,1])
    # for itemvar in itemvars:
    #     auc_scores[itemvar].append(
    #         roc_auc_score(cfdat['recalled'], np.ravel(cfdat['prediction'])))
    #     corrs['item_res_all'].append(
    #         np.corrcoef(cfdat['recalled'], np.ravel(cfdat['prediction']))[0,1])
    for regmodel in regmodels:
        if regmodel in itemvars:
            rfdat = rfdat_itemvar
        else:
            rfdat = rfdat_listvar
        if 'shuffled' in regmodel:
            dv = np.ravel(np.repeat(
                np.reshape(rfdat[regmodels[regmodel]['dv']],
                           (len(rfdat[regmodels[regmodel]['dv']])//24, 24)),
                shuffles, axis=0))
        else:
            dv = rfdat[regmodels[regmodel]['dv']]
        # if not np.all(np.isfinite(rfdat[dv[regmodel]])):
        #     continue
        valindx = np.isfinite(dv)
        binoutcome = dv[valindx] > np.percentile(
            dv[valindx], (1.0-np.mean(rfdat['events/recalled']))*100)
        metrics['auc_scores'][regmodel].append(
            roc_auc_score(
                binoutcome,
                np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx]))
        metrics['corrs'][regmodel].append(
            np.corrcoef(dv[valindx],
                        np.ravel(rfdat[regmodels[regmodel]['pred']])[
                            valindx])[0, 1])
        #
        if 'shuffled' in regmodel:
            continue
        if regmodel == 'raw':
            continue
        if regmodel in itemvars:
            regmodel_new1 = 'item-'+regmodel
            regmodel_new2 = regmodel+'-item'
            dv1_name = 'recalled'
        else:
            regmodel_new1 = 'list-'+regmodel
            regmodel_new2 = regmodel+'-list'
            dv1_name = 'recalled_logit'    
        dv2_name = regmodels[regmodel]['dv']
        valindx1 = np.isfinite(rfdat[dv1_name])
        valindx2 = np.isfinite(rfdat[dv2_name])
        slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
            rfdat[dv2_name][valindx1 & valindx2],
            rfdat[dv1_name][valindx1 & valindx2])
        dv_new = (rfdat[dv1_name][valindx1 & valindx2] -
                  (rfdat[dv2_name][valindx1 & valindx2] *
                   slope+intercept))
        binoutcome = dv_new > np.percentile(
            dv_new, (1.0-np.mean(rfdat['events/recalled']))*100)
        metrics['auc_scores'][regmodel_new1].append(
            roc_auc_score(
                binoutcome,
                np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx1 & valindx2]))
        metrics['corrs'][regmodel_new1].append(
            np.corrcoef(dv_new,
                        np.ravel(rfdat[regmodels[regmodel]['pred']])[
                            valindx1 & valindx2])[0, 1])
        slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
            rfdat[dv1_name][valindx1 & valindx2],
            rfdat[dv2_name][valindx1 & valindx2])
        dv_new = (rfdat[dv2_name][valindx1 & valindx2] -
                  (rfdat[dv1_name][valindx1 & valindx2] *
                   slope+intercept))
        binoutcome = dv_new > np.percentile(
            dv_new, (1.0-np.mean(rfdat['events/recalled']))*100)
        metrics['auc_scores'][regmodel_new2].append(
            roc_auc_score(
                binoutcome,
                np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx1 & valindx2]))
        metrics['corrs'][regmodel_new2].append(
            np.corrcoef(dv_new,
                        np.ravel(rfdat[regmodels[regmodel]['pred']])[
                            valindx1 & valindx2])[0, 1])






subj = 'LTP342'
subj = 'LTP343'
cf = os.path.join(classifier_folder, 'classifier_info_allfeat_'+subj+'.hdf5')
cfdat = h5py.File(cf, 'r')
fig, axs = plt.subplots(6, 1, figsize=((3,12)))
axs[0].boxplot(x=[np.ravel(cfdat['prediction'])[cfdat['recalled'][:]==0], np.ravel(cfdat['prediction'])[cfdat['recalled'][:]==1]], notch=True, showfliers=False)
rf_itemvar = os.path.join(regressor_folder_itemvar, 'regressor_info_allfeat_preds_residuals_'+subj+'.hdf5')
rfdat_itemvar = h5py.File(rf_itemvar, 'r')
rfdat = rfdat_itemvar
regmodel = 'item_res_all'
dv = rfdat[regmodels[regmodel]['dv']]
valindx = np.isfinite(dv)
axs[1].plot(
    dv[valindx],
    np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx], 'k.', alpha=0.1)
regmodel = 'item_res_norecallability'
dv = rfdat[regmodels[regmodel]['dv']]
valindx = np.isfinite(dv)
axs[2].plot(
    dv[valindx],
    np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx], 'k.', alpha=0.1)
regmodel = 'item_res_noserialpos'
dv = rfdat[regmodels[regmodel]['dv']]
valindx = np.isfinite(dv)
axs[3].plot(
    dv[valindx],
    np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx], 'k.', alpha=0.1)
regmodel = 'item_res_notrial'
dv = rfdat[regmodels[regmodel]['dv']]
valindx = np.isfinite(dv)
axs[4].plot(
    dv[valindx],
    np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx], 'k.', alpha=0.1)
regmodel = 'item_res_nosession'
dv = rfdat[regmodels[regmodel]['dv']]
valindx = np.isfinite(dv)
axs[5].plot(
    dv[valindx],
    np.ravel(rfdat[regmodels[regmodel]['pred']])[valindx], 'k.', alpha=0.1)

fig.subplots_adjust(left=0.2, bottom=0.03, right=0.99, top=0.99, wspace=0, hspace=0.4)
#fig.savefig(





def corr(x, y, **kwargs):
    """Adapted from https://towardsdatascience.com/visualizing-data-with-pair-plots-in-python-f228cf529166"""
    # Calculate the value
    coef = np.corrcoef(x, y)[0][1]
    # Make the label
    label = r'$r = ' + str(round(coef, 2)) + '$'
    
    # Add the label to the plot
    ax = plt.gca()
    ax.annotate(label, xy = (0.6, 0.1), size = 10, xycoords = ax.transAxes)
      
def sumstat(x, **kwargs):
    # Calculate the value
    sample_mean = str(round(np.mean(x), 2))
    sample_SE = str(round(np.std(x, ddof=1)/np.sqrt(len(x)), 3))
    # Make the label
    label = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
    # Add the label to the plot
    ax = plt.gca()
    ax.annotate(label, xy = (0.6, 0.8), size = 10, xycoords = ax.transAxes)
    #ax.annotate(label, xy = (0.5, 0.8), size = 10, xycoords = ax.transAxes)

def sumstat_corr(x, **kwargs):
    # Calculate the value
    sample_mean = str(round(np.tanh(np.mean(np.arctanh(x))), 2))
    sample_SE = str(round(np.tanh(np.std(np.arctanh(x), ddof=1))/np.sqrt(len(x)), 3))
    # Make the label
    label = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
    # Add the label to the plot
    ax = plt.gca()
    ax.annotate(label, xy = (0.6, 0.8), size = 10, xycoords = ax.transAxes)
    #ax.annotate(label, xy = (0.5, 0.8), size = 10, xycoords = ax.transAxes)



def make_gridplot(df, toplot_vars, toplot_labels, sumstat=sumstat_corr,
                  axlim=False, savefig=False):
    grid = sns.PairGrid(data=df, vars=toplot_vars, height=1.9)
    grid = grid.map_lower(plt.scatter, color='black', s=6, alpha=0.6)
    grid = grid.map_diag(plt.hist, bins=10, color='gray', 
                         edgecolor='k')
    grid = grid.map_diag(sumstat)
    grid = grid.map_upper(sns.kdeplot, shade=True, cmap='Greys',
                          shade_lowest=False)
    grid = grid.map_upper(corr)
    if axlim:
        for ax in grid.axes[0]:
            ax.set_xticks(np.arange(-1, 1.1, 0.2))
            ax.set_xlim(axlim)
    for ax in grid.axes[:, 0]:
        if axlim:
            ax.set_yticks(np.arange(-1, 1.1, 0.2))
            ax.set_ylim(axlim)
        label = ax.get_ylabel()
        if label in toplot_labels:
            ax.set_ylabel(toplot_labels[label])
    for ax in grid.axes[len(toplot_vars)-1]:
        label = ax.get_xlabel()
        if label in toplot_labels:
            ax.set_xlabel(toplot_labels[label])
    if savefig:
        grid.savefig(savefig)



itemvars_plot = ['item', 'item_res_all', 'item_res_norecallability',
                 'item_res_noserialpos', 'item_res_nosession',
                 'item_res_notrial']
itemvars_label_dict = {
    'item': 'item', 'item_res_all': 'item|all',
    'item_res_norecallability': r'item|$\neg$ recallability',
    'item_res_noserialpos': r'item|$\neg$ serial pos.',
    'item_res_nosession': r'item|$\neg$ session #',
    'item_res_notrial': r'item|$\neg$ list #'}
# make_gridplot(df=aucs_df, toplot_vars=itemvars_plot,
#               toplot_labels=itemvars_label_dict, axlim=(0.45, 0.8), savefig=False)

listvars_plot = ['raw', 'residuals_sesslist', 'residuals_list',
                 'residuals_sess']
# listvars_label_dict = {
#     'raw': r'$p($rec$)$', 'residuals_sesslist': r'$p($rec$)$|all',
#     'residuals_sess': r'$p($rec$)$|session', 'residuals_list': r'$p($rec$)$|list'}
listvars_label_dict = {
    'raw': r'list', 'residuals_sesslist': r'list|all',
    'residuals_sess': r'list|session #', 'residuals_list': r'list|list #'}
# make_gridplot(df=aucs_df, toplot_vars=listvars_plot,
#               toplot_labels=listvars_label_dict, axlim=(0.42, 1), savefig=False)

listvars_plot_shuffled = ['raw_shuffled', 'residuals_sesslist_shuffled',
                          'residuals_list_shuffled', 'residuals_sess_shuffled']
listvars_shuffled_label_dict = {
    'raw_shuffled': r'session', 'residuals_sesslist_shuffled': r'session|all',
    'residuals_sess_shuffled': r'session|session #', 'residuals_list_shuffled': r'list|list #'}
# make_gridplot(df=aucs_df, toplot_vars=listvars_plot_shuffled,
#               toplot_labels=listvars_label_dict, axlim=(0.475, 0.65), savefig=False)


itemvars_res_plot = ['item-item_res_all', 'item-item_res_norecallability',
                 'item-item_res_noserialpos', 'item-item_res_nosession',
                 'item-item_res_notrial']
itemvars_res_label_dict = {
    'item-item_res_all': 'item|all | item',
    'item-item_res_norecallability': r'item|$\neg$ recallability | item',
    'item-item_res_noserialpos': r'item|$\neg$ serial pos. | item',
    'item-item_res_nosession': r'item|$\neg$ session # | item',
    'item-item_res_notrial': r'item|$\neg$ list # | item'}


listvars_res_plot = ['list-residuals_sesslist', 'list-residuals_list',
                     'list-residuals_sess']
listvars_res_label_dict = {
    'list-residuals_sesslist': r'list|all | list',
    'list-residuals_sess': r'list|session # | list', 'list-residuals_list': r'list|list # | list'}



corrs_df = pd.DataFrame.from_dict(metrics['corrs'])
corrs_df.index=metrics['subject']




dat_all = h5py.File('../processed_dat/dat_all.hdf5', 'r')

tvals = {}
for key in dat_all:
    if isinstance(dat_all[key], h5py.Group):
        tvals[key] = {}
        for key2 in dat_all[key]:
            tvals[key][key2] = (np.mean(dat_all[key][key2], 0) /
                                (np.std(dat_all[key][key2], 0) /
                                 np.sqrt(len(dat_all[key][key2]))))
    # elif key=='subject'
    else:
        try:
            tvals[key] = (np.mean(dat_all[key], 0) /
                          (np.std(dat_all[key], 0) /
                           np.sqrt(len(dat_all[key]))))
        except TypeError:
            tvals[key] = dat_all[key]
tvals['recalled-unrecalled'] = np.mean(
    dat_all['recalled'][:]-dat_all['unrecalled'][:], 0) / (np.std(
        dat_all['recalled'][:]-dat_all['unrecalled'][:], 0) / np.sqrt(
            len(dat_all['recalled'])))


def sumstat_corr_vals(x):
    # Calculate the value
    sample_mean = str(round(np.tanh(np.mean(np.arctanh(x))), 2))
    sample_SE = str(round(np.tanh(np.std(np.arctanh(x), ddof=1))/
                          np.sqrt(len(x)), 3))
    return(sample_mean, sample_SE)





itemvars = ['item_res_all', 'item_res_norecallability',
            'item_res_noserialpos', 'item_res_nosession', 'item_res_notrial']
listvars = ['raw', 'residuals_sesslist', 'residuals_list', 'residuals_sess']
# listvars_shuffled = ['raw_shuffled', 'residuals_sesslist_shuffled',
#                      'residuals_list_shuffled', 'residuals_sess_shuffled']

# regmodels[][dv]

# regtypes = ['prec', 'intersess', 'interlist', 'sesslist']
# regtypes_itemvar = itemvars_plot + listvars_plot
corrs_dict = {'egi': {rt: [] for rt in ['item']+itemvars+listvars},#+listvars_shuffled},
              'biosemi': {rt: [] for rt in ['item']+itemvars+listvars}}#+listvars_shuffled}}
for eegsys in corrs_dict:
    for regtype in itemvars:
        corrs_dict[eegsys]['item-'+regtype] = []
        corrs_dict[eegsys][regtype+'-item'] = []
    for regtype in listvars:
        if regtype == 'raw':
            continue
        corrs_dict[eegsys]['prec-'+regtype] = []
        corrs_dict[eegsys][regtype+'-prec'] = []
    # for regtype in listvars_shuffled:
    #     if regtype == 'raw_shuffled':
    #         continue
    #     corrs_dict[eegsys]['prec-'+regtype] = []
    #     corrs_dict[eegsys][regtype+'-prec'] = []
#     corrs_dict[eegsys]['intersess-prec'] = []
#     corrs_dict[eegsys]['prec-interlist'] = []
#     corrs_dict[eegsys]['interlist-prec'] = []
#     corrs_dict[eegsys]['prec-intersess'] = []

subjects = {}
features_chans = {}
features_freqs = {}
for eegsys in corrs_dict:
    subjects[eegsys] = []
    features_chans[eegsys] = None
    features_freqs[eegsys] = None

# for regfile_itemvar, regfile_listvar in zip(regfiles_itemvar,
#                                             regfiles_listvar):
for regfile_itemvar in regfiles_itemvar:
    print(regfile_itemvar)
    with h5py.File(regfile_itemvar, 'r') as rf_item:
        subj = rf_item['events/subject'][0].decode()
        if np.int(subj[-3:]) <= 330:
            eegsys = 'egi'
        else:
            eegsys = 'biosemi'
        subjects[eegsys].append(subj)
        if features_chans[eegsys] is None:
            features_chans[eegsys] = np.array(rf_item['features_chans'][()],
                                              dtype=np.str)
        else:
            assert(np.all(features_chans[eegsys] == np.array(
                rf_item['features_chans'][()], dtype=np.str)))
        if features_freqs[eegsys] is None:
            features_freqs[eegsys] = rf_item['features_freqs'][()]
        else:
            assert(
                np.all(features_freqs[eegsys] == rf_item['features_freqs'][()]))
        
        for classfile in classfiles:
            if subj in classfile:
                break
        with h5py.File(classfile, 'r') as cf:
            assert(subj == cf['events/subject'][0].decode())
            assert(np.all(features_chans[eegsys] == np.array(
                cf['features_chans'][()], dtype=np.str)))
            assert(
                np.all(features_freqs[eegsys] == cf['features_freqs'][()]))
            dv = cf['recalled']
            feat = cf['features_dat'][np.isfinite(dv), :]
            dv = dv[np.isfinite(dv)]
            # need to zero-center for below calculation to be a
            # correlation (feat is already zero-centered):
            dv = dv-dv.mean()
            # using the above syntax rather than -=, because -= throws a
            # TypeError if dv is int.
            corrs_dict[eegsys]['item'].append(
                np.dot(dv, feat)/(np.linalg.norm(dv) *
                                  np.linalg.norm(feat, axis=0)))
            for regtype in itemvars:
                dv = rf_item[regmodels[regtype]['dv']]
                feat = rf_item['features_dat'][np.isfinite(dv), :]
                dv = dv[np.isfinite(dv)]
                # need to zero-center for below calculation to be a
                # correlation (feat is already zero-centered - for
                # residuals (but not prec), dv, should be too, but no harm
                # in making sure):
                dv -= dv.mean()
                corrs_dict[eegsys][regtype].append((np.dot(dv, feat) / (
                    np.linalg.norm(dv) * np.linalg.norm(feat, axis=0))))
                regtype_new = 'item-'+regtype
                dv1_name = 'recalled'
                dv2_name = regmodels[regtype]['dv']
                valindx1 = np.isfinite(cf[dv1_name])
                valindx2 = np.isfinite(rf_item[dv2_name])
                slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
                    rf_item[dv2_name][valindx1 & valindx2],
                    cf[dv1_name][valindx1 & valindx2])
                dv_new = (cf[dv1_name][valindx1 & valindx2] -
                          (rf_item[dv2_name][valindx1 & valindx2] *
                           slope+intercept))
                dv_new -= dv_new.mean()  # residuals should already be zero centered, but ...
                feat_new = rf_item['features_dat'][valindx1 & valindx2, :]
                corrs_dict[eegsys][regtype_new].append((np.dot(dv_new, feat_new) / (
                    np.linalg.norm(dv_new) * np.linalg.norm(feat_new, axis=0))))  
                regtype_new = regtype+'-item'
                # dv1_name = 'recalled'
                # dv2_name = dv
                # valindx1 = np.isfinite(cf[dv1_name])
                # valindx2 = np.isfinite(rf_item[dv2_name])
                slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
                    cf[dv1_name][valindx1 & valindx2],
                    rf_item[dv2_name][valindx1 & valindx2])
                dv_new = (rf_item[dv2_name][valindx1 & valindx2] -
                          (cf[dv1_name][valindx1 & valindx2] *
                           slope+intercept))
                dv_new -= dv_new.mean()  # residuals should already be zero centered, but ...
                # feat_new = rf_item['features_dat'][valindx1 & valindx2, :]
                corrs_dict[eegsys][regtype_new].append((np.dot(dv_new, feat_new) / (
                    np.linalg.norm(dv_new) * np.linalg.norm(feat_new, axis=0))))
    # regfile_listvar = regfile_itemvar.replace('itemvar', 'listvar').replace(
    #     'allfeat_preds_residuals', 'allfeat').replace(
    #         'LR_pred_residuals', 'pred_residuals_catpredictors')
    for regfile_listvar in regfiles_listvar:
        if subj in regfile_listvar:
            break
    with h5py.File(regfile_listvar, 'r') as rf_list:
        assert(subj == rf_list['events/subject'][0].decode())
        assert(np.all(features_chans[eegsys] == np.array(
                rf_list['features_chans'][()], dtype=np.str)))
        assert(
            np.all(features_freqs[eegsys] == rf_list['features_freqs'][()]))
        for regtype in listvars:
            dv = rf_list[regmodels[regtype]['dv']]
            feat = rf_list['features_dat'][np.isfinite(dv), :]
            dv = dv[np.isfinite(dv)]
            # need to zero-center for below calculation to be a
            # correlation (feat is already zero-centered - for
            # residuals (but not prec), dv, should be too, but no harm
            # in making sure):
            dv -= dv.mean()
            corrs_dict[eegsys][regtype].append((np.dot(dv, feat) / (
                np.linalg.norm(dv) * np.linalg.norm(feat, axis=0))))
            if regtype == 'raw':
                continue
            regtype_new = 'prec-'+regtype
            dv1_name = 'recalled_logit'
            dv2_name = regmodels[regtype]['dv']
            valindx1 = np.isfinite(rf_list[dv1_name])
            valindx2 = np.isfinite(rf_list[dv2_name])
            slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
                rf_list[dv2_name][valindx1 & valindx2],
                rf_list[dv1_name][valindx1 & valindx2])
            dv_new = (rf_list[dv1_name][valindx1 & valindx2] -
                      (rf_list[dv2_name][valindx1 & valindx2] *
                       slope+intercept))
            dv_new -= dv_new.mean()  # residuals should already be zero centered, but ...
            feat_new = rf_list['features_dat'][valindx1 & valindx2, :]
            corrs_dict[eegsys][regtype_new].append((np.dot(dv_new, feat_new) / (
                np.linalg.norm(dv_new) * np.linalg.norm(feat_new, axis=0))))  
            regtype_new = regtype+'-prec'
            #dv1_name = 'recalled_logit'
            #dv2_name = dv
            #valindx1 = np.isfinite(rf_list[dv1_name])
            #valindx2 = np.isfinite(rf_list[dv2_name])
            slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
                rf_list[dv1_name][valindx1 & valindx2],
                rf_list[dv2_name][valindx1 & valindx2])
            dv_new = (rf_list[dv2_name][valindx1 & valindx2] -
                      (rf_list[dv1_name][valindx1 & valindx2] *
                       slope+intercept))
            dv_new -= dv_new.mean()  # residuals should already be zero centered, but ...
            feat_new = rf_list['features_dat'][valindx1 & valindx2, :]
            corrs_dict[eegsys][regtype_new].append((np.dot(dv_new, feat_new) / (
                np.linalg.norm(dv_new) * np.linalg.norm(feat_new, axis=0))))  
        # for regtype in listvars_shuffled:
        #     dv = rf_list[regmodels[regtype]['dv']]
        #     feat = rf_list['features_dat'][np.isfinite(dv), :]
        #     dv = dv[np.isfinite(dv)]
        #     # need to zero-center for below calculation to be a
        #     # correlation (feat is already zero-centered - for
        #     # residuals (but not prec), dv, should be too, but no harm
        #     # in making sure):
        #     dv -= dv.mean()
        #     corrs_dict[eegsys][regtype].append((np.dot(dv, feat) / (
        #         np.linalg.norm(dv) * np.linalg.norm(feat, axis=0))))
        #     if regtype == 'raw_shuffled':
        #         continue
        #     regtype_new = 'prec-'+regtype
        #     dv1_name = 'recalled_logit'
        #     dv2_name = regmodels[regtype]['dv']
        #     valindx1 = np.isfinite(rf_list[dv1_name])
        #     valindx2 = np.isfinite(rf_list[dv2_name])
        #     slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
        #         rf_list[dv2_name][valindx1 & valindx2],
        #         rf_list[dv1_name][valindx1 & valindx2])
        #     dv_new = (rf_list[dv1_name][valindx1 & valindx2] -
        #               (rf_list[dv2_name][valindx1 & valindx2] *
        #                slope+intercept))
        #     dv_new -= dv_new.mean()  # residuals should already be zero centered, but ...
        #     feat_new = rf_list['features_dat'][valindx1 & valindx2, :]
        #     corrs_dict[eegsys][regtype_new].append((np.dot(dv_new, feat_new) / (
        #         np.linalg.norm(dv_new) * np.linalg.norm(feat_new, axis=0))))  
        #     regtype_new = regtype+'-prec'
        #     #dv1_name = 'recalled_logit'
        #     #dv2_name = dv
        #     #valindx1 = np.isfinite(rf_list[dv1_name])
        #     #valindx2 = np.isfinite(rf_list[dv2_name])
        #     slope, intercept, r_value, p_value, std_err = sp.stats.linregress(
        #         rf_list[dv1_name][valindx1 & valindx2],
        #         rf_list[dv2_name][valindx1 & valindx2])
        #     dv_new = (rf_list[dv2_name][valindx1 & valindx2] -
        #               (rf_list[dv1_name][valindx1 & valindx2] *
        #                slope+intercept))
        #     dv_new -= dv_new.mean()  # residuals should already be zero centered, but ...
        #     feat_new = rf_list['features_dat'][valindx1 & valindx2, :]
        #     corrs_dict[eegsys][regtype_new].append((np.dot(dv_new, feat_new) / (
        #         np.linalg.norm(dv_new) * np.linalg.norm(feat_new, axis=0))))  


    # for classfile in classfiles:
    #     if subj in classfile:
    #         break
    # with h5py.File(classfile, 'r') as cf:
    #     assert(subj == cf['events/subject'][0].decode())
    #     assert(np.all(features_chans[eegsys] == np.array(
    #             cf['features_chans'][()], dtype=np.str)))
    #     assert(
    #         np.all(features_freqs[eegsys] == cf['features_freqs'][()]))
    #     dv = cf['recalled']
    #     feat = cf['features_dat'][np.isfinite(dv), :]
    #     dv = dv[np.isfinite(dv)]
    #     # need to zero-center for below calculation to be a
    #     # correlation (feat is already zero-centered):
    #     dv = dv-dv.mean()
    #     # using the above syntax rather than -=, because -= throws a
    #     # TypeError if dv is int.
    #     corrs_dict[eegsys]['item'].append(
    #         np.dot(dv, feat)/(np.linalg.norm(dv) *
    #                           np.linalg.norm(feat, axis=0)))


corrs_dict_bkup = corrs_dict.copy()

for eegsys in corrs_dict:
    feat_inf_indx = pd.MultiIndex.from_arrays(
        [features_chans[eegsys], features_freqs[eegsys]],
        names=['channel', 'frequency'])
    corrs_dict[eegsys] = xr.DataArray(
        # data=[corrs_dict[eegsys][rt] for rt in ['item']+listvars+itemvars],
        data=[corrs_dict[eegsys][rt] for rt in corrs_dict[eegsys].keys()],
        # coords=[('dvtype', ['item']+listvars+itemvars),
        coords=[('dvtype', list(corrs_dict[eegsys].keys())),
                ('subject', subjects[eegsys]),
                ('feat_inf', feat_inf_indx)]).unstack('feat_inf')
    corrs_dict[eegsys]['channel'] = np.array(corrs_dict[eegsys]['channel'], np.str)



rois = {'egi': {'LAI': ['023', '026', '027', '033', '034', '039', '040'],
                'LAS': ['012', '013', '019', '020', '024', '028', '029'],
                'RAI': ['002', '003', '109', '115', '116', '122', '123'],
                'RAS': ['004', '005', '111', '112', '117', '118', '124'],
                'LPI': ['051', '058', '059', '064', '065', '066', '069'],
                'LPS': ['037', '042', '052', '053', '054', '060', '061'],
                'RPI': ['084', '089', '090', '091', '095', '096', '097'],
                'RPS': ['078', '079', '085', '086', '087', '092', '093']},
        'biosemi': {'LAI': ['C27', 'C28', 'C31', 'C32', 'D5', 'D6', 'D10'],
                    'LAS': ['C24', 'C25', 'D2', 'D3', 'D4', 'D12', 'D13'],
                    'RAI': ['B29', 'C5', 'C6', 'C9', 'C10', 'C14', 'C15'],
                    'RAS': ['B31', 'B32', 'C2', 'C3', 'C4', 'C11', 'C12'],
                    'LPI': ['A8', 'A9', 'A16', 'A17', 'D26', 'D29', 'D30'],
                    'LPS': ['A5', 'A6', 'A7', 'A18', 'D16', 'D17', 'D28'],
                    'RPI': ['A29', 'A30', 'B5', 'B6', 'B12', 'B13', 'B16'],
                    'RPS': ['A31','A32', 'B2', 'B3', 'B4', 'B18', 'B19']}}

corrs_roi = []
roi_indx =pd.Index(sorted(rois['egi'].keys()), name='ROI')
for eegsys in corrs_dict:
    corrs_roi.append(xr.concat(
        [np.tanh(np.arctanh(corrs_dict[eegsys]).sel(channel=np.array(
            [chan in rois[eegsys][roi]
             for chan in corrs_dict[eegsys].channel])).mean('channel'))
         for roi in roi_indx], dim=roi_indx))
corrs_roi = xr.concat(corrs_roi, dim='subject')
corrs_roi['subject'] = np.array(corrs_roi['subject'], np.str)
corrs_roi['ROI'] = np.array(corrs_roi['ROI'], np.str)


selected_rois = ['LPS', 'RPS', 'LAS', 'RAS']







itemvars_plot = ['item', 'item_res_all', 'item_res_norecallability',
                 'item_res_noserialpos', 'item_res_notrial',
                 'item_res_nosession']
listvars_plot = ['raw', 'residuals_sesslist', 'residuals_list',
                 'residuals_sess']

lineprops = {'item': {'c': 'green', 'ls': '-'},
             'item_res_all': {'c': 'green', 'ls': '-'},
             'item_res_norecallability': {'c': 'green', 'ls': '-'},
             'item_res_noserialpos': {'c': 'green', 'ls': '-'},
             'item_res_nosession': {'c': 'green', 'ls': '-'},
             'item_res_notrial': {'c': 'green', 'ls': '-'},
             'item_resres': {'c': 'green', 'ls': '--'},
             'raw': {'c': 'magenta', 'ls': '-'},
             'residuals_sesslist': {'c': 'magenta', 'ls': '-'},
             'residuals_sess': {'c': 'magenta', 'ls': '-'},
             'residuals_list': {'c': 'magenta', 'ls': '-'},
             'list_resres': {'c': 'magenta', 'ls': '--'}}


fig = plt.figure(figsize=(7, 12))
gs0 = gridspec.GridSpec(2, 1, height_ratios=[3, 2], wspace=0.6)
gs00 = gridspec.GridSpecFromSubplotSpec(6, 3, subplot_spec=gs0[0, :],
                                        # width_ratios=[1, 1, 1, 1, 20, 20],
                                        # width_ratios=[1, 2, 3],
                                        width_ratios=[2, 3, 3],
                                        # height_ratios=[6, 1],
                                        wspace=0.5, hspace=0.1)

vmax = 12
axs1 = np.empty((6, 3), object)
for r, regtype in enumerate(itemvars_plot):
    axs1[r, 1] = plt.Subplot(fig, gs00[r, 1])
    axs1[r, 1].hist(corrs_df[regtype], 10, color='green')
    axs1[r, 1].set_xlim((-0.0285, 0.42))
    axs1[r, 1].set_ylim((0, 25))
    sample_mean, sample_SE = sumstat_corr_vals(corrs_df[regtype])
    label = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
    axs1[r, 1].text(0.98, 0.81, label, horizontalalignment='right',
                   verticalalignment='center', transform=axs1[r, 1].transAxes)
    axs1[r, 2] = plt.Subplot(fig, gs00[r, 2])
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs1[r, 2].plot(
        corrs_roi['frequency'], corr_mean, label=itemvars_label_dict[regtype], **lineprops[regtype])
    axs1[r, 2].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
    axs1[r, 2].plot([2, 200], [0, 0], 'k-')
    axs1[r, 2].set_xscale('log')
    axs1[r, 2].text(0.95, 0.1, itemvars_label_dict[regtype], horizontalalignment='right',
                   verticalalignment='center', transform=axs1[r, 2].transAxes)
    # axs1[r, 2].set_ylim((-0.11, 0.05689327)) 
    axs1[r, 2].set_ylim((-0.053, 0.033)) 
    axs1[r, 2].set_xticks([2, 7, 39, 200])
    axs1[r, 2].minorticks_off()
    if r==5:
        axs1[r, 2].set_xticklabels([2, 7, 39, 200])
    else:
        axs1[r, 2].set_xticklabels([])
    if regtype == 'item':
        continue
    # # dv_new = dv+'-item'
    # regtype_new = 'item-'+regtype
    # corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
    #     [r in selected_rois for r in corrs_roi['ROI']]))).mean(
    #         'ROI').mean('subject'))
    # corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
    #     [r in selected_rois for r in corrs_roi['ROI']]))).mean(
    #         'ROI').std('subject', ddof=1)) /
    #            np.sqrt(len(corrs_roi['subject'])))
    # corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    # axs1[r, 2].plot(
    #     corrs_roi['frequency'], corr_mean,
    #     label='item-'+itemvars_label_dict[regtype], **lineprops['item_resres'])
    # axs1[r, 2].fill_between(
    #     corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
    #     alpha=0.3, color=lineprops['item_resres']['c'], edgecolor=lineprops['item_resres']['c'])
    #
    gs00b = gridspec.GridSpecFromSubplotSpec(1, 4, subplot_spec=gs00[r, 0],
                                        # height_ratios=[6, 1],
                                        wspace=0.02, hspace=0.7)
    axs1b = np.empty((1, 4), object)
    for i in range(4):
        axs1b[0, i] = plt.Subplot(fig, gs00b[0, i])
        axs1b[0, i].imshow(np.reshape(
            tvals[regtype.replace('trial', 'list')][str(i)][:, 55:].mean(1),
            (15, 1)), aspect=0.5, origin='lower', cmap='bwr',
                          vmin=-vmax, vmax=vmax)
        axs1b[0, i].set_yticks([0, 4, 9, 14])
        if r==5:
            axs1b[0, i].set_xticks([0])
            axs1b[0, i].set_xticklabels([i+1])
        else:
            axs1b[0, i].set_xticks([])
        if i==0:
            axs1b[0, i].set_yticklabels([2, 7, 39, 200])
            if r == 5:
                axs1b[0, i].set_ylabel('Frequency (Hz)')
        else:
            axs1b[0, i].set_yticklabels([])
    for ax in np.ravel(axs1b):
        if not ax is None:
            print(ax)
            fig.add_subplot(ax)

axs1[5, 0] = plt.Subplot(fig, gs00[5, 0])
axs1[5, 0].text(0.5, -0.37, 'Quartile', horizontalalignment='center',
                   verticalalignment='center', transform=axs1[5, 0].transAxes, clip_on=False)
axs1[5, 0].set_axis_off()


for ax in np.ravel(axs1):
    if not ax is None:
        print(ax)
        fig.add_subplot(ax)

axs1[5, 1].set_ylabel('Count')
axs1[5, 2].set_ylabel('Correlation ($r$)')
axs1[5, 1].set_xlabel('Correlation ($r$)')
axs1[5, 2].set_xlabel('Frequency (Hz)')

ax_quartiles = plt.Subplot(fig, gs00[0, 0])
mu = 0
variance = 1
sigma = np.sqrt(variance)
x = np.linspace(mu - 3*sigma, mu + 3*sigma, 1000)
y = stats.norm.pdf(x, mu, sigma)
ax_quartiles.plot(x, y, 'k')
ax_quartiles.fill_between(x[x<-0.67], y[x<-0.67], 0, color='0.8')
ax_quartiles.fill_between(x[(x>=-0.67) & (x<0)], y[(x>=-0.67) & (x<0)], 0, color='0.6')
ax_quartiles.fill_between(x[(x>=0) & (x<0.67)], y[(x>=0) & (x<0.67)], 0, color='0.4')
ax_quartiles.fill_between(x[x>=0.67], y[x>=0.67], 0, color='0.2')
ax_quartiles.set_xlim([-3,3])
ax_quartiles.axis('off')
fig.add_subplot(ax_quartiles)

gs01 = gridspec.GridSpecFromSubplotSpec(4, 3, subplot_spec=gs0[1, :],
                                        width_ratios=[2, 3, 3],
                                        # height_ratios=[6, 1],
                                        wspace=0.5, hspace=0.1)

vmax = 12
axs2 = np.empty((4, 3), object)
for r, regtype in enumerate(listvars_plot):
    axs2[r, 1] = plt.Subplot(fig, gs01[r, 1])
    axs2[r, 1].hist(corrs_df[regtype], 10, color='magenta')
    # axs2[r, 1].set_xlim((-0.1, 0.677))
    axs2[r, 1].set_xlim((-0.1, 0.89))
    axs2[r, 1].set_ylim((0, 25))
    sample_mean, sample_SE = sumstat_corr_vals(corrs_df[regtype])
    label = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
    axs2[r, 1].text(0.98, 0.81, label, horizontalalignment='right',
                   verticalalignment='center', transform=axs2[r, 1].transAxes)
    axs2[r, 2] = plt.Subplot(fig, gs01[r, 2])
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs2[r, 2].plot(
        corrs_roi['frequency'], corr_mean, label=listvars_label_dict[regtype], **lineprops[regtype])
    axs2[r, 2].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
    axs2[r, 2].plot([2, 200], [0, 0], 'k-')
    axs2[r, 2].set_xscale('log')
    axs2[r, 2].text(0.95, 0.1, listvars_label_dict[regtype], horizontalalignment='right',
                   verticalalignment='center', transform=axs2[r, 2].transAxes)
    
    gs01b = gridspec.GridSpecFromSubplotSpec(1, 4, subplot_spec=gs01[r, 0],
                                        # height_ratios=[6, 1],
                                        wspace=0.02, hspace=0.7)
    axs2b = np.empty((1, 4), object)
    for i in range(4):
    # i=0
        axs2b[0, i] = plt.Subplot(fig, gs01b[0, i])
        axs2b[0, i].imshow(np.reshape(
            tvals[regtype.replace(
                'raw', 'recalled_logit')][str(i)][:, 55:].mean(1), (15, 1)),
                          aspect=0.5, origin='lower', cmap='bwr',
                          vmin=-vmax, vmax=vmax)
        axs2b[0, i].set_yticks([0, 4, 9, 14])
        if r==3:
            axs2b[0, i].set_xticks([0])
            axs2b[0, i].set_xticklabels([i+1])
        else:
            axs2b[0, i].set_xticks([])
        if i==0:
            axs2b[0, i].set_yticklabels([2, 7, 39, 200])
            if r == 3:
                axs2b[0, i].set_ylabel('Frequency (Hz)')
        else:
            axs2b[0, i].set_yticklabels([])
    for ax in np.ravel(axs2b):
        if not ax is None:
            print(ax)
            fig.add_subplot(ax)
    # axs2[r, 2].set_ylim((-0.26, 0.0464)) 
    axs2[r, 2].set_ylim((-0.15, 0.05)) 
    axs2[r, 2].set_xticks([2, 7, 39, 200])
    axs2[r, 2].minorticks_off()
    if r==3:
        axs2[r, 2].set_xticklabels([2, 7, 39, 200])
    else:
        axs2[r, 2].set_xticklabels([])
    # if regtype == 'raw':
    #     continue
    # regtype_new = 'prec-'+regtype
    # corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
    #     [r in selected_rois for r in corrs_roi['ROI']]))).mean(
    #         'ROI').mean('subject'))
    # corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
    #     [r in selected_rois for r in corrs_roi['ROI']]))).mean(
    #         'ROI').std('subject', ddof=1)) /
    #            np.sqrt(len(corrs_roi['subject'])))
    # corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    # axs2[r, 2].plot(
    #     corrs_roi['frequency'], corr_mean,
    #     label='list-'+listvars_label_dict[regtype], **lineprops['list_resres'])
    # axs2[r, 2].fill_between(
    #     corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
    #     alpha=0.3, color=lineprops['list_resres']['c'], edgecolor=lineprops['list_resres']['c'])
    # axs2[r, 5].set_ylim((-0.11, 0.05689327))

    
axs2[3, 1].set_ylabel('Count')
axs2[3, 2].set_ylabel('Correlation ($r$)')
axs2[3, 1].set_xlabel('Correlation ($r$)')
axs2[3, 2].set_xlabel('Frequency (Hz)')

axs2[3, 0] = plt.Subplot(fig, gs01[3, 0])
axs2[3, 0].text(0.5, -0.37, 'Quartile', horizontalalignment='center',
                   verticalalignment='center', transform=axs2[3, 0].transAxes, clip_on=False)
axs2[3, 0].set_axis_off()

for ax in np.ravel(axs2):
    if not ax is None:
        print(ax)
        fig.add_subplot(ax)

# lines = [mlines.Line2D([], [], color='black', alpha=0.6, ls='-', label='regular'),
#          mlines.Line2D([], [], color='black', alpha=0.3, ls='--', label='semi-partial')]
# fig.legend(loc=(0.8, 0.39), handles=lines)

ax_cb2 = fig.add_axes([0.105, 0.41, 0.1, 0.01]) # [0.92, 0.753, 0.06, 0.02][0.13, 0.4, 0.06, 0.02]
norm = mpl.colors.Normalize(vmin=-vmax, vmax=vmax)
cb = mpl.colorbar.ColorbarBase(ax_cb2, cmap=plt.cm.bwr, # label='t(131)',
                               # norm=norm, ticks=range(-4, 5, 2),
                               # norm=norm, ticks=range(-5, 6, 5),
                               norm=norm, ticks=range(-9, 10, 9),
                               ticklocation='bottom',
                               orientation='horizontal')
# #ax_cb.set_title(cbtitle, size='large')
ax_cb2.set_ylabel(r'$t(96)$', rotation=0, labelpad=15, verticalalignment='center')
ax_cb2.tick_params(axis='both', which='major', pad=2)

fig.subplots_adjust(left=0.07, bottom=0.04, right=0.991, top=0.99,
                    wspace=0.06, hspace=0.2)

# plt.savefig('./figs/corrs_all_nolabels.pdf')
fig.text(0.005, 0.995, r'A',  fontdict={'weight': 'bold', 'size': 'large'},
         horizontalalignment='left', verticalalignment='top')

# fig.text(0.0, 0.38, r'B',  fontdict={'weight': 'bold', 'size': 'large'},
fig.text(0.005, 0.405, r'B',  fontdict={'weight': 'bold', 'size': 'large'},
         horizontalalignment='left', verticalalignment='top')
#plt.savefig('./figs/corrs_all_labels.pdf')







fig = plt.figure(figsize=(7, 12))
gs0 = gridspec.GridSpec(2, 1, height_ratios=[3, 2], wspace=0.6)
gs00 = gridspec.GridSpecFromSubplotSpec(6, 3, subplot_spec=gs0[0, :],
                                        # width_ratios=[1, 1, 1, 1, 20, 20],
                                        # width_ratios=[1, 2, 3],
                                        width_ratios=[2, 3, 3],
                                        # height_ratios=[6, 1],
                                        wspace=0.5, hspace=0.1)


item_dfs = [pd.DataFrame(corrs_df[regtype]) for regtype in itemvars_plot]
for idf,regtype in zip(item_dfs, itemvars_plot):
    idf['condition'] =[regtype]*len(idf)
    idf['Correlation ($r$)'] = idf[regtype]
    
item_df = pd.concat(item_dfs, axis=0, join='inner')

item_df1 = item_df.copy()
item_df2 = item_df.copy()
item_df1['column'] = [1]*len(item_df1)
item_df2['column'] = [2]*len(item_df2)

item_df_col = pd.concat([item_df1, item_df2])


# itemvars_label_dict = {
#     'item': 'item', 'item_res_all': 'item|\nall',
#     'item_res_norecallability': 'item|\n'+r'$\neg$ recallability',
#     'item_res_noserialpos': 'item|\n'+r'$\neg$ serial pos.',
#     'item_res_nosession': 'item|\n'+r'$\neg$ session #',
#     'item_res_notrial': 'item|\n'+r'$\neg$ list #'}

itemvars_label_dict = {
    'item': '   item', 'item_res_all': 'item|all',
    'item_res_norecallability': 'item|'+r'$\neg$ recallability',
    'item_res_noserialpos': 'item|'+r'$\neg$ serial pos.',
    'item_res_nosession': 'item|'+r'$\neg$ session #',
    'item_res_notrial': ' item|'+r'$\neg$ list #'}



import seaborn as sns
sns.set(style="white", rc={"axes.facecolor": (0, 0, 0, 0)})

pal = sns.cubehelix_palette(10, rot=-.25, light=.7)
# g = sns.FacetGrid(df, row="g", hue="g", aspect=15, height=.5, palette=pal)

# g = sns.FacetGrid(corrs_df, row="g", hue="g", aspect=15, height=.5, palette=pal)
# g = sns.FacetGrid(item_dfs, row="condition", hue="condition", aspect=15, height=.5, palette=pal)
g = sns.FacetGrid(item_df_col, row="condition", col="column", hue="condition",
                  aspect=6, height=.5, palette=pal, sharex='col', sharey=False,
                  # aspect=6, height=.5, palette=pal, sharex=False, sharey=False,
                  gridspec_kws=dict(width_ratios=[3, 1], wspace=0.42, left=0.02,
                                    right=0.97, bottom=0.17, top=0.996))
# g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, shade=True, alpha=1, lw=1.5, bw=.2)
# g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, color="w", lw=2, bw=.2)
g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, shade=True, alpha=1, lw=1.5, bw='scott')
g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, color="w", lw=2, bw='scott')
g.map(plt.axhline, y=0, lw=2, clip_on=False)

# Define and use a simple function to label the plot in axes coordinates
def label(x, color, label):
    ax = plt.gca()
    #ax.text(0, .2, label, fontweight="bold", color=color,
    #        ha="left", va="center", transform=ax.transAxes)
    # ax.text(0.9, .2, label, fontweight="bold", color='k', #color,
    #         ha="left", va="center", transform=ax.transAxes)
    #ax.text(1, 0.4, itemvars_label_dict[label], fontweight="bold", color=color,
    #        ha="center", va="center", transform=ax.transAxes)
    ax.text(0.85, 0.4, itemvars_label_dict[label], fontweight="bold", color=color,
            ha="left", va="center", transform=ax.transAxes)
    # ax.text(0.9, 0.4, itemvars_label_dict[label], fontweight="bold", color='k', #color,
            # ha="left", va="center", transform=ax.transAxes)


    
# def label_stats(x, color, label):
#     ax = plt.gca()
#     sample_mean, sample_SE = sumstat_corr_vals(x)
#     stat_txt = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
#     ax.text(0, 0.4, stat_txt, fontweight="bold", color='k', #color,
#             ha='left', va='center', transform=ax.transAxes)

# def mark_stats(x, color, label):
#     ax = plt.gca()
#     sample_mean, sample_SE = sumstat_corr_vals(x)
#     ax.axvline(x=sample_mean, color='k')
#     #ax.plot([sample_mean,sample_mean], [0, 1], 'k-')
# #     print(sample_mean)

#axs1[r, 1].set_xlim((-0.0285, 0.42))
#axs1[r, 1].set_ylim((0, 25))    
g.map(label, "Correlation ($r$)")
# g.map(label_stats, "Correlation ($r$)")
# g.map(mark_stats, "Correlation ($r$)")

# Set the subplots to overlap
overlap = 0.25
g.fig.subplots_adjust(hspace=-overlap)

# Remove axes details that don't play well with overlap
g.set_titles("")
g.set(yticks=[])
#g.despine(bottom=True, left=True)

for i, (regtype,ax) in enumerate(zip(itemvars_plot, g.axes)):
    ax[1].clear()
    #ax[0].set_xlim((-0.1, 0.35))
    ax[0].set_xlim((-0.05, 0.35))
    sns.utils.despine(ax=ax[0], bottom=True, left=True)
    if regtype == 'item':
        ax[1].tick_params(axis='y', reset=True, left=True, right=False)
    elif regtype == 'item_res_nosession':
        #sns.utils.despine(ax=ax[0], bottom=True, left=True)
        sns.utils.despine(ax=ax[1], bottom=True, left=True) 
        ax[1].set_yticklabels([])
        ax[1].set_yticks([])
        ax[0].tick_params(axis='x', reset=True, bottom=True, top=False)
        ax[1].tick_params(axis='x', reset=True, bottom=True, top=False)
    else:
        # sns.utils.despine(ax=ax[0], offset=5, trim=True)
        #sns.utils.despine(ax=ax[1], offset=1, trim=True)
        ax[1].set_yticklabels([])
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    ax[1].plot(
        corrs_roi['frequency'], corr_mean, label=itemvars_label_dict[regtype], c='k', ls='-')
        # corrs_roi['frequency'], corr_mean, label=itemvars_label_dict[regtype], **lineprops[regtype])
    ax[1].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=1, color=pal[i], edgecolor=pal[i])
        # alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
    ax[1].plot([2, 200], [0, 0], 'k-', lw=1)
    ax[1].set_xlim((2, 200))
    ax[1].set_xscale('log')
    ax[1].set_ylim((-0.053, 0.033)) 
    ax[1].set_xticks([2, 7, 39, 200])
    #ax[1].minorticks_off()
    ax[1].set_xticklabels([2, 7, 39, 200])
    pos = ((ax[0].get_ylim()[1] - ax[0].get_ylim()[0])/2)-overlap
    bp = ax[0].boxplot(corrs_df[regtype], vert=False, notch=True, widths=2,
                       positions=[pos], bootstrap=10000, showfliers=False)
    if regtype != 'item':
        # ax[1].set_yticklabels([])
        ax[1].yaxis.set_visible(False)
        sns.utils.despine(ax=ax[1], offset=5, trim=True, left=True, bottom=True)
    else:
        #ax[1].set_yticklabels(['-0.05', '0.00'])
        ax[1].set_yticks([-0.05, 0.0])
        sns.utils.despine(ax=ax[1], offset=5, trim=True, bottom=True)
        ax[1].set_ylabel('$r$')
    if regtype == 'item_res_nosession':
        sns.utils.despine(ax=ax[1], offset=0, trim=True, left=True)
        ax[1].set_xlabel('Frequency (Hz)')
#g.fig.savefig('./figs/corrs_item.pdf')



list_dfs = [pd.DataFrame(corrs_df[regtype]) for regtype in listvars_plot]
for ldf,regtype in zip(list_dfs, listvars_plot):
    ldf['condition'] =[regtype]*len(ldf)
    ldf['Correlation ($r$)'] = ldf[regtype]
    
list_df = pd.concat(list_dfs, axis=0, join='inner')

list_df1 = list_df.copy()
list_df2 = list_df.copy()
list_df1['column'] = [1]*len(list_df1)
list_df2['column'] = [2]*len(list_df2)

list_df_col = pd.concat([list_df1, list_df2])

# listvars_label_dict = {
#     'raw': 'list', 'residuals_sesslist': 'list|\nall',
#     'residuals_sess': 'list|\nsession #', 'residuals_list': 'list|\nlist #'}

listvars_label_dict = {
    'raw': 'list', 'residuals_sesslist': 'list|all',
    'residuals_sess': 'list|session #', 'residuals_list': 'list|list #'}


g = sns.FacetGrid(list_df_col, row="condition", col="column", hue="condition",
                  aspect=6, height=.5, palette=pal, sharex='col', sharey=False,
                  # aspect=6, height=.5, palette=pal, sharex=False, sharey=False,
                  gridspec_kws=dict(width_ratios=[3, 1], wspace=0.42, left=0.02,
                                    right=0.97, bottom=0.25, top=0.998))
# g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, shade=True, alpha=1, lw=1.5, bw=.2)
# g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, color="w", lw=2, bw=.2)
g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, shade=True, alpha=1, lw=1.5, bw='scott')
g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, color="w", lw=2, bw='scott')
g.map(plt.axhline, y=0, lw=2, clip_on=False)#, xmin=-0.1)

# Define and use a simple function to label the plot in axes coordinates
def label_list(x, color, label):
    ax = plt.gca()
    #ax.text(0, .2, label, fontweight="bold", color=color,
    #        ha="left", va="center", transform=ax.transAxes)
    # ax.text(0.9, .2, label, fontweight="bold", color='k', #color,
    #         ha="left", va="center", transform=ax.transAxes)
    ax.text(0.95, 0.4, listvars_label_dict[label], fontweight="bold", color=color,
            ha="left", va="center", transform=ax.transAxes)
    # ax.text(0.9, 0.4, itemvars_label_dict[label], fontweight="bold", color='k', #color,
            # ha="left", va="center", transform=ax.transAxes)


    
# def label_stats(x, color, label):
#     ax = plt.gca()
#     sample_mean, sample_SE = sumstat_corr_vals(x)
#     stat_txt = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
#     ax.text(0, 0.4, stat_txt, fontweight="bold", color='k', #color,
#             ha='left', va='center', transform=ax.transAxes)

# def mark_stats(x, color, label):
#     ax = plt.gca()
#     sample_mean, sample_SE = sumstat_corr_vals(x)
#     ax.axvline(x=sample_mean, color='k')
#     #ax.plot([sample_mean,sample_mean], [0, 1], 'k-')
# #     print(sample_mean)

#axs1[r, 1].set_xlim((-0.0285, 0.42))
#axs1[r, 1].set_ylim((0, 25))    
g.map(label_list, "Correlation ($r$)")
# g.map(label_stats, "Correlation ($r$)")
# g.map(mark_stats, "Correlation ($r$)")

# Set the subplots to overlap
overlap = 0.25
g.fig.subplots_adjust(hspace=-overlap)

# Remove axes details that don't play well with overlap
g.set_titles("")
g.set(yticks=[])
#g.despine(bottom=True, left=True)

for i, (regtype,ax) in enumerate(zip(listvars_plot, g.axes)):
    ax[1].clear()
    #ax[0].set_xlim((-0.1, 0.35))
    # ax[0].set_xlim((-0.05, 0.5))
    ax[0].set_xlim((-0.15, 0.7))
    sns.utils.despine(ax=ax[0], bottom=True, left=True)
    if regtype == 'raw':
        ax[1].tick_params(axis='y', reset=True, left=True, right=False)
    elif regtype == 'residuals_sess':
        #sns.utils.despine(ax=ax[0], bottom=True, left=True)
        sns.utils.despine(ax=ax[1], bottom=True, left=True) 
        ax[1].set_yticklabels([])
        ax[1].set_yticks([])
        ax[0].tick_params(axis='x', reset=True, bottom=True, top=False)
        ax[1].tick_params(axis='x', reset=True, bottom=True, top=False)
    else:
        # sns.utils.despine(ax=ax[0], offset=5, trim=True)
        #sns.utils.despine(ax=ax[1], offset=1, trim=True)
        ax[1].set_yticklabels([])
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    ax[1].plot(
        corrs_roi['frequency'], corr_mean, label=listvars_label_dict[regtype], c='k', ls='-')
    ax[1].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=1, color=pal[i], edgecolor=pal[i])
        # alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
    ax[1].plot([2, 200], [0, 0], 'k-', lw=1)
    ax[1].set_xlim((2, 200))
    ax[1].set_xscale('log')
    #ax[1].set_ylim((-0.053, 0.033)) 
    ax[1].set_ylim((-0.15, 0.033)) 
    ax[1].set_xticks([2, 7, 39, 200])
    #ax[1].minorticks_off()
    ax[1].set_xticklabels([2, 7, 39, 200])
    pos = ((ax[0].get_ylim()[1] - ax[0].get_ylim()[0])/2)-overlap
    bp = ax[0].boxplot(corrs_df[regtype], vert=False, notch=True, widths=1,
                       positions=[pos], bootstrap=10000, showfliers=False)
    if regtype != 'raw':
        # ax[1].set_yticklabels([])
        ax[1].yaxis.set_visible(False)
        sns.utils.despine(ax=ax[1], offset=5, trim=True, left=True, bottom=True)
    else:
        ax[1].set_yticks([-0.1, 0])
        #ax[1].set_yticklabels(['-0.05', '0.00'])
        sns.utils.despine(ax=ax[1], offset=5, trim=True, bottom=True)
        ax[1].set_ylabel('$r$')
    if regtype == 'residuals_sess':
        sns.utils.despine(ax=ax[1], offset=0, trim=True, left=True)
        ax[1].set_xlabel('Frequency (Hz)')
#g.fig.savefig('./figs/corrs_list.pdf')




















list_shuffled_dfs = [pd.DataFrame(corrs_df[regtype+'_shuffled']) for regtype in listvars_plot]
for ldf_shuffled,regtype in zip(list_shuffled_dfs, listvars_plot):
    regtype += '_shuffled'
    ldf_shuffled['condition'] =[regtype]*len(ldf_shuffled)
    ldf_shuffled['Correlation ($r$)'] = ldf_shuffled[regtype]
    
list_shuffled_df = pd.concat(list_shuffled_dfs, axis=0, join='inner')
listvars_label_dict = {
    'raw': 'list', 'residuals_sesslist': 'list|all',
    'residuals_sess': 'list|session #', 'residuals_list': 'list|list #'}


g = sns.FacetGrid(list_shuffled_df, row="condition", hue="condition",
                  aspect=6, height=.5, palette=pal, sharex='col', sharey=False,
                  # aspect=6, height=.5, palette=pal, sharex=False, sharey=False,
                  gridspec_kws=dict(wspace=0.42, left=0.02,
                                    right=0.97, bottom=0.25, top=0.998))
g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, shade=True, alpha=1, lw=1.5, bw='scott')
g.map(sns.kdeplot, "Correlation ($r$)", clip_on=False, color="w", lw=2, bw='scott')
g.map(plt.axhline, y=0, lw=2, clip_on=False)#, xmin=-0.1)

# Define and use a simple function to label the plot in axes coordinates
def label_list(x, color, label):
    ax = plt.gca()
    ax.text(0., 0.4, listvars_label_dict[label.split('_shuffled')[0]], fontweight="bold", color=color,
            ha="left", va="center", transform=ax.transAxes)


g.map(label_list, "Correlation ($r$)")
# Set the subplots to overlap
overlap = 0.25
g.fig.subplots_adjust(hspace=-overlap)

# Remove axes details that don't play well with overlap
g.set_titles("")
g.set(yticks=[])



for i, (regtype,ax) in enumerate(zip(listvars_plot, g.axes)):
    regtype += '_shuffled'
    ax[0].set_xlim((-0.14, 0.14))
    sns.utils.despine(ax=ax[0], bottom=True, left=True)
    if regtype == 'residuals_sess_shuffled':
        ax[0].tick_params(axis='x', reset=True, bottom=True, top=False)
    pos = ((ax[0].get_ylim()[1] - ax[0].get_ylim()[0])/2) - overlap*5
    bp = ax[0].boxplot(corrs_df[regtype], vert=False, notch=True, widths=5,
                       positions=[pos], bootstrap=10000, showfliers=False)

#g.fig.savefig('./figs/corrs_list_shuffled.pdf')


























    
bp = ax[0].boxplot(corrs_df[regtype], vert=False, notch=True, widths=2, positions=[2])

def mark_stats(x, color, label):
    ax = plt.gca()
    sample_mean, sample_SE = sumstat_corr_vals(x)
    ax.plot([sample_mean,sample_mean], [0, 1], 'k-')
    print(sample_mean)


        

    if r==5:
        axs1[r, 2].set_xticklabels([2, 7, 39, 200])
    else:
        axs1[r, 2].set_xticklabels([])



    


    
ax[1].clear()
ax[1].plot(
        corrs_roi['frequency'], corr_mean, label=itemvars_label_dict[regtype], **lineprops[regtype])
ax[1].fill_between(
    corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
    alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
ax[1].set_ylim((-0.05, 0.01))
ax[1].plot([2, 200], [0, 0], 'k-')
ax[1].set_xscale('log')
#ax[0].set_xscale('linear')
# axs2[r, 2].text(0.95, 0.1, listvars_label_dict[regtype], horizontalalignment='right',
#                 verticalalignment='center', transform=axs2[r, 2].transAxes)


for r, regtype in enumerate(listvars_plot):
    axs2[r, 1] = plt.Subplot(fig, gs01[r, 1])
    axs2[r, 1].hist(corrs_df[regtype], 10, color='magenta')
    # axs2[r, 1].set_xlim((-0.1, 0.677))
    axs2[r, 1].set_xlim((-0.1, 0.89))
    axs2[r, 1].set_ylim((0, 25))
    sample_mean, sample_SE = sumstat_corr_vals(corrs_df[regtype])
    label = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
    axs2[r, 1].text(0.98, 0.81, label, horizontalalignment='right',
                   verticalalignment='center', transform=axs2[r, 1].transAxes)
    axs2[r, 2] = plt.Subplot(fig, gs01[r, 2])
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs2[r, 2].plot(
        corrs_roi['frequency'], corr_mean, label=listvars_label_dict[regtype], **lineprops[regtype])
    axs2[r, 2].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
    axs2[r, 2].plot([2, 200], [0, 0], 'k-')
    axs2[r, 2].set_xscale('log')
    axs2[r, 2].text(0.95, 0.1, listvars_label_dict[regtype], horizontalalignment='right',
                   verticalalignment='center', transform=axs2[r, 2].transAxes)
  

















































            

# fig = plt.figure(figsize=(7, 12))
# gs0 = gridspec.GridSpec(2, 1, height_ratios=[3, 2], wspace=0.6)
# gs00 = gridspec.GridSpecFromSubplotSpec(6, 3, subplot_spec=gs0[0, :],
#                                         # width_ratios=[1, 1, 1, 1, 20, 20],
#                                         # width_ratios=[1, 2, 3],
#                                         width_ratios=[2, 3, 3],
#                                         # height_ratios=[6, 1],
#                                         wspace=0.5, hspace=0.1)

# vmax = 12
# axs1 = np.empty((6, 3), object)
# for r, regtype in enumerate(itemvars_plot):
#     axs1[r, 1] = plt.Subplot(fig, gs00[r, 1])
#     axs1[r, 1].hist(corrs_df[regtype], 10, color='green')
#     axs1[r, 1].set_xlim((-0.0285, 0.42))
#     axs1[r, 1].set_ylim((0, 25))
#     sample_mean, sample_SE = sumstat_corr_vals(corrs_df[regtype])
#     label = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
#     axs1[r, 1].text(0.98, 0.81, label, horizontalalignment='right',
#                    verticalalignment='center', transform=axs1[r, 1].transAxes)
#     axs1[r, 2] = plt.Subplot(fig, gs00[r, 2])
#     corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').mean('subject'))
#     corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').std('subject', ddof=1)) /
#                np.sqrt(len(corrs_roi['subject'])))
#     corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
#     axs1[r, 2].plot(
#         corrs_roi['frequency'], corr_mean, label=itemvars_label_dict[regtype], **lineprops[regtype])
#     axs1[r, 2].fill_between(
#         corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
#         alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
#     axs1[r, 2].plot([2, 200], [0, 0], 'k-')
#     axs1[r, 2].set_xscale('log')
#     axs1[r, 2].text(0.95, 0.1, itemvars_label_dict[regtype], horizontalalignment='right',
#                    verticalalignment='center', transform=axs1[r, 2].transAxes)
#     axs1[r, 2].set_ylim((-0.11, 0.05689327)) 
#     axs1[r, 2].set_xticks([2, 7, 39, 200])
#     axs1[r, 2].minorticks_off()
#     if r==5:
#         axs1[r, 2].set_xticklabels([2, 7, 39, 200])
#     else:
#         axs1[r, 2].set_xticklabels([])
#     if regtype == 'item':
#         continue
#     # dv_new = dv+'-item'
#     regtype_new = 'item-'+regtype
#     corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').mean('subject'))
#     corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').std('subject', ddof=1)) /
#                np.sqrt(len(corrs_roi['subject'])))
#     corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
#     axs1[r, 2].plot(
#         corrs_roi['frequency'], corr_mean,
#         label='item-'+itemvars_label_dict[regtype], **lineprops['item_resres'])
#     axs1[r, 2].fill_between(
#         corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
#         alpha=0.3, color=lineprops['item_resres']['c'], edgecolor=lineprops['item_resres']['c'])
#     #
#     gs00b = gridspec.GridSpecFromSubplotSpec(1, 4, subplot_spec=gs00[r, 0],
#                                         # height_ratios=[6, 1],
#                                         wspace=0.02, hspace=0.7)
#     axs1b = np.empty((1, 4), object)
#     for i in range(4):
#         axs1b[0, i] = plt.Subplot(fig, gs00b[0, i])
#         axs1b[0, i].imshow(np.reshape(
#             tvals[regtype.replace('trial', 'list')][str(i)][:, 55:].mean(1),
#             (15, 1)), aspect=0.5, origin='lower', cmap='bwr',
#                           vmin=-vmax, vmax=vmax)
#         axs1b[0, i].set_yticks([0, 4, 9, 14])
#         if r==5:
#             axs1b[0, i].set_xticks([0])
#             axs1b[0, i].set_xticklabels([i+1])
#         else:
#             axs1b[0, i].set_xticks([])
#         if i==0:
#             axs1b[0, i].set_yticklabels([2, 7, 39, 200])
#             if r == 5:
#                 axs1b[0, i].set_ylabel('Frequency (Hz)')
#         else:
#             axs1b[0, i].set_yticklabels([])
#     for ax in np.ravel(axs1b):
#         if not ax is None:
#             print(ax)
#             fig.add_subplot(ax)

# axs1[5, 0] = plt.Subplot(fig, gs00[5, 0])
# axs1[5, 0].text(0.5, -0.37, 'Quartile', horizontalalignment='center',
#                    verticalalignment='center', transform=axs1[5, 0].transAxes, clip_on=False)
# axs1[5, 0].set_axis_off()


# for ax in np.ravel(axs1):
#     if not ax is None:
#         print(ax)
#         fig.add_subplot(ax)

# axs1[5, 1].set_ylabel('Count')
# axs1[5, 2].set_ylabel('Correlation ($r$)')
# axs1[5, 1].set_xlabel('Correlation ($r$)')
# axs1[5, 2].set_xlabel('Frequency (Hz)')

# ax_quartiles = plt.Subplot(fig, gs00[0, 0])
# mu = 0
# variance = 1
# sigma = np.sqrt(variance)
# x = np.linspace(mu - 3*sigma, mu + 3*sigma, 1000)
# y = stats.norm.pdf(x, mu, sigma)
# ax_quartiles.plot(x, y, 'k')
# ax_quartiles.fill_between(x[x<-0.67], y[x<-0.67], 0, color='0.8')
# ax_quartiles.fill_between(x[(x>=-0.67) & (x<0)], y[(x>=-0.67) & (x<0)], 0, color='0.6')
# ax_quartiles.fill_between(x[(x>=0) & (x<0.67)], y[(x>=0) & (x<0.67)], 0, color='0.4')
# ax_quartiles.fill_between(x[x>=0.67], y[x>=0.67], 0, color='0.2')
# ax_quartiles.set_xlim([-3,3])
# ax_quartiles.axis('off')
# fig.add_subplot(ax_quartiles)



# # # # arrow = labelaxs['0'][-1].arrow(
# # # #     1.12, 0.25, 0.3, -0.3, transform=labelaxs['0'][-1].transAxes, clip_on=False,
# # # #     width=0.02, head_width=0.12, color='k')

# # # # 0.5, -0.05, 0.3, 0.3
# # arrow = labelaxs['0'][0].arrow(
# #     1.46, 2, -0.5, -0.5, transform=labelaxs['0'][-1].transAxes, clip_on=False,
# #     width=0.02, head_width=0.12, color='k')

# # # #     0.5, -0.1, 0.2, -0.3, transform=labelaxs['1'][-1].transAxes, clip_on=False,
# # arrow = labelaxs['1'][-1].arrow(
# #     0.825, 2, -0.25, -0.5, transform=labelaxs['1'][-1].transAxes, clip_on=False,
# #     width=0.02, head_width=0.12, color='k')

# # # #     0.25, -0.1, -0.2, -0.3, transform=labelaxs['2'][-1].transAxes, clip_on=False,
# # arrow = labelaxs['2'][-1].arrow(
# #     -0.25, 2, 0.25, -0.5, transform=labelaxs['2'][-1].transAxes, clip_on=False,
# #     width=0.02, head_width=0.12, color='k')

# # arrow = labelaxs['3'][-1].arrow(
# #     -0.9, 2, 0.5, -0.5, transform=labelaxs['3'][-1].transAxes, clip_on=False,
# #     width=0.02, head_width=0.12, color='k')






# gs01 = gridspec.GridSpecFromSubplotSpec(4, 3, subplot_spec=gs0[1, :],
#                                         width_ratios=[2, 3, 3],
#                                         # height_ratios=[6, 1],
#                                         wspace=0.5, hspace=0.1)

# vmax = 12
# axs2 = np.empty((4, 3), object)
# for r, regtype in enumerate(listvars_plot):
#     axs2[r, 1] = plt.Subplot(fig, gs01[r, 1])
#     axs2[r, 1].hist(corrs_df[regtype], 10, color='magenta')
#     # axs2[r, 1].set_xlim((-0.1, 0.677))
#     axs2[r, 1].set_xlim((-0.1, 0.89))
#     axs2[r, 1].set_ylim((0, 25))
#     sample_mean, sample_SE = sumstat_corr_vals(corrs_df[regtype])
#     label = r'$M = ' + sample_mean + '$\n$\mathit{SE} = '+sample_SE + '$'
#     axs2[r, 1].text(0.98, 0.81, label, horizontalalignment='right',
#                    verticalalignment='center', transform=axs2[r, 1].transAxes)
#     axs2[r, 2] = plt.Subplot(fig, gs01[r, 2])
#     corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').mean('subject'))
#     corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').std('subject', ddof=1)) /
#                np.sqrt(len(corrs_roi['subject'])))
#     corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
#     axs2[r, 2].plot(
#         corrs_roi['frequency'], corr_mean, label=listvars_label_dict[regtype], **lineprops[regtype])
#     axs2[r, 2].fill_between(
#         corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
#         alpha=0.6, color=lineprops[regtype]['c'], edgecolor=lineprops[regtype]['c'])
#     axs2[r, 2].plot([2, 200], [0, 0], 'k-')
#     axs2[r, 2].set_xscale('log')
#     axs2[r, 2].text(0.95, 0.1, listvars_label_dict[regtype], horizontalalignment='right',
#                    verticalalignment='center', transform=axs2[r, 2].transAxes)
    
#     gs01b = gridspec.GridSpecFromSubplotSpec(1, 4, subplot_spec=gs01[r, 0],
#                                         # height_ratios=[6, 1],
#                                         wspace=0.02, hspace=0.7)
#     axs2b = np.empty((1, 4), object)
#     for i in range(4):
#     # i=0
#         axs2b[0, i] = plt.Subplot(fig, gs01b[0, i])
#         axs2b[0, i].imshow(np.reshape(
#             tvals[regtype.replace(
#                 'raw', 'recalled_logit')][str(i)][:, 55:].mean(1), (15, 1)),
#                           aspect=0.5, origin='lower', cmap='bwr',
#                           vmin=-vmax, vmax=vmax)
#         axs2b[0, i].set_yticks([0, 4, 9, 14])
#         if r==3:
#             axs2b[0, i].set_xticks([0])
#             axs2b[0, i].set_xticklabels([i+1])
#         else:
#             axs2b[0, i].set_xticks([])
#         if i==0:
#             axs2b[0, i].set_yticklabels([2, 7, 39, 200])
#             if r == 3:
#                 axs2b[0, i].set_ylabel('Frequency (Hz)')
#         else:
#             axs2b[0, i].set_yticklabels([])
#     for ax in np.ravel(axs2b):
#         if not ax is None:
#             print(ax)
#             fig.add_subplot(ax)
#     axs2[r, 2].set_ylim((-0.26, 0.0464)) 
#     axs2[r, 2].set_xticks([2, 7, 39, 200])
#     axs2[r, 2].minorticks_off()
#     if r==3:
#         axs2[r, 2].set_xticklabels([2, 7, 39, 200])
#     else:
#         axs2[r, 2].set_xticklabels([])
#     if regtype == 'raw':
#         continue
#     regtype_new = 'prec-'+regtype
#     corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').mean('subject'))
#     corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=regtype_new, ROI=np.array(
#         [r in selected_rois for r in corrs_roi['ROI']]))).mean(
#             'ROI').std('subject', ddof=1)) /
#                np.sqrt(len(corrs_roi['subject'])))
#     corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
#     axs2[r, 2].plot(
#         corrs_roi['frequency'], corr_mean,
#         label='list-'+listvars_label_dict[regtype], **lineprops['list_resres'])
#     axs2[r, 2].fill_between(
#         corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
#         alpha=0.3, color=lineprops['list_resres']['c'], edgecolor=lineprops['list_resres']['c'])
#     # axs2[r, 5].set_ylim((-0.11, 0.05689327))

    
# axs2[3, 1].set_ylabel('Count')
# axs2[3, 2].set_ylabel('Correlation ($r$)')
# axs2[3, 1].set_xlabel('Correlation ($r$)')
# axs2[3, 2].set_xlabel('Frequency (Hz)')

# axs2[3, 0] = plt.Subplot(fig, gs01[3, 0])
# axs2[3, 0].text(0.5, -0.37, 'Quartile', horizontalalignment='center',
#                    verticalalignment='center', transform=axs2[3, 0].transAxes, clip_on=False)
# axs2[3, 0].set_axis_off()

# for ax in np.ravel(axs2):
#     if not ax is None:
#         print(ax)
#         fig.add_subplot(ax)

# lines = [mlines.Line2D([], [], color='black', alpha=0.6, ls='-', label='regular'),
#          mlines.Line2D([], [], color='black', alpha=0.3, ls='--', label='semi-partial')]
# fig.legend(loc=(0.8, 0.39), handles=lines)

# ax_cb2 = fig.add_axes([0.105, 0.41, 0.1, 0.01]) # [0.92, 0.753, 0.06, 0.02][0.13, 0.4, 0.06, 0.02]
# norm = mpl.colors.Normalize(vmin=-vmax, vmax=vmax)
# cb = mpl.colorbar.ColorbarBase(ax_cb2, cmap=plt.cm.bwr, # label='t(131)',
#                                # norm=norm, ticks=range(-4, 5, 2),
#                                # norm=norm, ticks=range(-5, 6, 5),
#                                norm=norm, ticks=range(-9, 10, 9),
#                                ticklocation='bottom',
#                                orientation='horizontal')
# # #ax_cb.set_title(cbtitle, size='large')
# ax_cb2.set_ylabel(r'$t(96)$', rotation=0, labelpad=15, verticalalignment='center')
# ax_cb2.tick_params(axis='both', which='major', pad=2)

# fig.subplots_adjust(left=0.07, bottom=0.04, right=0.991, top=0.99,
#                     wspace=0.06, hspace=0.2)

# # plt.savefig('./figs/corrs_all_nolabels.pdf')
# fig.text(0.005, 0.995, r'A',  fontdict={'weight': 'bold', 'size': 'large'},
#          horizontalalignment='left', verticalalignment='top')

# # fig.text(0.0, 0.38, r'B',  fontdict={'weight': 'bold', 'size': 'large'},
# fig.text(0.005, 0.405, r'B',  fontdict={'weight': 'bold', 'size': 'large'},
#          horizontalalignment='left', verticalalignment='top')
# #plt.savefig('./figs/corrs_all_labels.pdf')










corrs_df_Z = np.arctanh(corrs_df)

# # individual items:

# tres = ttest_1samp(corrs_df_Z['item'], 0)
# se = np.std(corrs_df_Z['item'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_1sampResult(statistic=22.681100191950616, pvalue=2.42339196419504e-40) 0.007224813984881699
In [587]: np.mean(corrs_df_Z['item'])/(np.std(corrs_df_Z['item'], ddof=1))
Out[587]: 2.3029168100908453

# tres = ttest_1samp(corrs_df_Z['item_res_all'], 0)
# se = np.std(corrs_df_Z['item_res_all'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_all']))
# print(tres, se)
# #  Ttest_1sampResult(statistic=19.014862738518975, pvalue=2.552245456161901e-34) 0.006483589964582912
In [588]: np.mean(corrs_df_Z['item_res_all'])/(np.std(corrs_df_Z['item_res_all'], ddof=1))
Out[588]: 1.9306667962097377

# tres = ttest_1samp(corrs_df_Z['item_res_norecallability'], 0)
# se = np.std(corrs_df_Z['item_res_norecallability'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_norecallability']))
# print(tres, se)
# # Ttest_1sampResult(statistic=19.498973421644795, pvalue=3.7337153709178155e-35) 0.006509943656759829
In [589]: np.mean(corrs_df_Z['item_res_norecallability'])/(np.std(corrs_df_Z['item_res_norecallability'], ddof=1))
Out[589]: 1.9798207887709407

# tres = ttest_1samp(corrs_df_Z['item_res_noserialpos'], 0)
# se = np.std(corrs_df_Z['item_res_noserialpos'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_noserialpos']))
# print(tres, se)
# # Ttest_1sampResult(statistic=21.811455206057133, pvalue=5.6519853478496974e-39) 0.006891073539844301
In [590]: np.mean(corrs_df_Z['item_res_noserialpos'])/(np.std(corrs_df_Z['item_res_noserialpos'], ddof=1))
Out[590]: 2.214617739945382

# tres = ttest_1samp(corrs_df_Z['item_res_nosession'], 0)
# se = np.std(corrs_df_Z['item_res_nosession'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_nosession']))
# print(tres, se)
# # Ttest_1sampResult(statistic=16.340995079095062, pvalue=1.764705809625388e-29) 0.006702352468255385
In [591]: np.mean(corrs_df_Z['item_res_nosession'])/(np.std(corrs_df_Z['item_res_nosession'], ddof=1))
Out[591]: 1.6591766688026508

# tres = ttest_1samp(corrs_df_Z['item_res_notrial'], 0)
# se = np.std(corrs_df_Z['item_res_notrial'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_notrial']))
# print(tres, se)
# # Ttest_1sampResult(statistic=22.47054270418915, pvalue=5.15546261712826e-40) 0.006496916547517073
In [592]: np.mean(corrs_df_Z['item_res_notrial'])/(np.std(corrs_df_Z['item_res_notrial'], ddof=1))
Out[592]: 2.281537936316968




# tres = ttest_rel(corrs_df_Z['item'], corrs_df_Z['item_res_all'])
# se = np.std(corrs_df_Z['item']-corrs_df_Z['item_res_all'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_relResult(statistic=9.738130302028496, pvalue=5.43244290173136e-16) 0.004167345822171959
In [595]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_all'])/(np.std(corrs_df_Z['item']-corrs_df_Z['item_res_all'], ddof=1))
Out[595]: 0.9887573257736126
In [604]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_all'])
Out[604]: 0.04058215662992462

# tres = ttest_rel(corrs_df_Z['item'], corrs_df_Z['item_res_norecallability'])
# se = np.std(corrs_df_Z['item']-corrs_df_Z['item_res_norecallability'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_relResult(statistic=8.947857822492104, pvalue=2.70669683817777e-14) 0.0041271902451236525
In [597]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_norecallability'])/(np.std(corrs_df_Z['item']-corrs_df_Z['item_res_norecallability'], ddof=1))
Out[597]: 0.908517312622821
In [605]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_norecallability'])
Out[605]: 0.03692951151974277

# tres = ttest_rel(corrs_df_Z['item'], corrs_df_Z['item_res_noserialpos'])
# se = np.std(corrs_df_Z['item']-corrs_df_Z['item_res_noserialpos'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_relResult(statistic=4.924765484311993, pvalue=3.5125476781588837e-06) 0.0027539155045152412
In [598]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_noserialpos'])/(np.std(corrs_df_Z['item']-corrs_df_Z['item_res_noserialpos'], ddof=1))
Out[598]: 0.500034174867859
In [606]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_noserialpos'])
Out[606]: 0.013562388023348299

# tres = ttest_rel(corrs_df_Z['item'], corrs_df_Z['item_res_nosession'])
# se = np.std(corrs_df_Z['item']-corrs_df_Z['item_res_nosession'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_relResult(statistic=13.438317890388532, pvalue=8.727906567684596e-24) 0.0040439303192889966
In [599]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_nosession'])/(np.std(corrs_df_Z['item']-corrs_df_Z['item_res_nosession'], ddof=1))
Out[599]: 1.3644544535852505
In [607]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_nosession'])
Out[607]: 0.054343621157185946

# tres = ttest_rel(corrs_df_Z['item'], corrs_df_Z['item_res_notrial'])
# se = np.std(corrs_df_Z['item']-corrs_df_Z['item_res_notrial'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_relResult(statistic=4.726423107550251, pvalue=7.82051389154308e-06) 0.003782456357792809
In [601]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_notrial'])/(np.std(corrs_df_Z['item']-corrs_df_Z['item_res_notrial'], ddof=1))
Out[601]: 0.47989555770500697
In [609]: np.mean(corrs_df_Z['item']-corrs_df_Z['item_res_notrial'])
Out[609]: 0.01787748913277229


# tres = ttest_rel(corrs_df_Z['item_res_all'], corrs_df_Z['item_res_norecallability'])
# se = np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_norecallability'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_all']))
# print(tres, se)
# # Ttest_relResult(statistic=-5.938811406520847, pvalue=4.580136662697216e-08) 0.0006150464899712454
In [8]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_norecallability'])/(np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_norecallability'], ddof=1))
Out[8]: -0.6029949387062736
In [9]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_norecallability'])
Out[9]: -0.0036526451101818415

# tres = ttest_rel(corrs_df_Z['item_res_all'], corrs_df_Z['item_res_noserialpos'])
# se = np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_noserialpos'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_all']))
# print(tres, se)
# # Ttest_relResult(statistic=-9.017344091282139, pvalue=1.9204767160234102e-14) 0.0029964220432376217
In [10]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_noserialpos'])/(np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_noserialpos'], ddof=1))
Out[10]: -0.9155725742773618
In [11]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_noserialpos'])
Out[11]: -0.02701976860657632

# tres = ttest_rel(corrs_df_Z['item_res_all'], corrs_df_Z['item_res_nosession'])
# se = np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_nosession'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_all']))
# print(tres, se)
# # Ttest_relResult(statistic=9.904722185168723, pvalue=2.3824709122734525e-16) 0.0013893842017969648
In [12]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_nosession'])/(np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_nosession'], ddof=1))
Out[12]: 1.0056721687424939
In [13]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_nosession'])
Out[13]: 0.013761464527261331

# tres = ttest_rel(corrs_df_Z['item_res_all'], corrs_df_Z['item_res_notrial'])
# se = np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_notrial'], ddof=1)/np.sqrt(len(corrs_df_Z['item_res_all']))
# print(tres, se)
# # Ttest_relResult(statistic=-10.790392436584465, pvalue=3.0135016869673926e-18) 0.0021041558618547476
In [14]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_notrial'])/(np.std(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_notrial'], ddof=1))
Out[14]: -1.0955983580772843
In [15]: np.mean(corrs_df_Z['item_res_all']-corrs_df_Z['item_res_notrial'])
Out[15]: -0.02270466749715233




# # lists:

tres = ttest_1samp(corrs_df_Z['raw'], 0)
se = np.std(corrs_df_Z['raw'], ddof=1)/np.sqrt(len(corrs_df_Z['raw']))
print(tres, se)
# Ttest_1sampResult(statistic=18.213008219939592, pvalue=6.56544163262557e-33) 0.014627637269204595
In [611]: np.mean(corrs_df_Z['raw'])/np.std(corrs_df_Z['raw'], ddof=1)
Out[611]: 1.8492508051662742

tres = ttest_1samp(corrs_df_Z['residuals_sesslist'], 0)
se = np.std(corrs_df_Z['residuals_sesslist'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_sesslist']))
print(tres, se)
# Ttest_1sampResult(statistic=14.331656763574264, pvalue=1.384495958212865e-25) 0.015332787042611342
In [612]: np.mean(corrs_df_Z['residuals_sesslist'])/np.std(corrs_df_Z['residuals_sesslist'], ddof=1)
Out[612]: 1.4551592734906431

tres = ttest_1samp(corrs_df_Z['residuals_list'], 0)
se = np.std(corrs_df_Z['residuals_list'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_list']))
print(tres, se)
# Ttest_1sampResult(statistic=12.668456583796093, pvalue=3.330638144708095e-22) 0.01312726612135498
In [613]: np.mean(corrs_df_Z['residuals_list'])/np.std(corrs_df_Z['residuals_list'], ddof=1)
Out[613]: 1.2862868810519115

tres = ttest_1samp(corrs_df_Z['residuals_sess'], 0)
se = np.std(corrs_df_Z['residuals_sess'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_sess']))
print(tres, se)
# Ttest_1sampResult(statistic=20.132256828920195, pvalue=3.1535242821220115e-36) 0.016495333597984444
In [614]: np.mean(corrs_df_Z['residuals_sess'])/np.std(corrs_df_Z['residuals_sess'], ddof=1)
Out[614]: 2.044120976672924



# tres = ttest_rel(corrs_df_Z['raw'], corrs_df_Z['residuals_sesslist'])
# se = np.std(corrs_df_Z['raw']-corrs_df_Z['residuals_sesslist'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_relResult(statistic=5.547676032247753, pvalue=2.554600513117004e-07) 0.008412357972483185
In [617]: np.mean(corrs_df_Z['raw']-corrs_df_Z['residuals_sesslist'])
Out[617]: 0.046669036698633294
In [618]: np.mean(corrs_df_Z['raw']-corrs_df_Z['residuals_sesslist'])/np.std(corrs_df_Z['raw']-corrs_df_Z['residuals_sesslist'], ddof=1)
Out[618]: 0.5632811584746451

# tres =ttest_rel(corrs_df_Z['raw'], corrs_df_Z['residuals_list'])
# se = np.std(corrs_df_Z['raw']-corrs_df_Z['residuals_list'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
#  #Ttest_relResult(statistic=12.466662885973792, pvalue=8.738653826709282e-22) 0.008030302721398666
In [619]: np.mean(corrs_df_Z['raw']-corrs_df_Z['residuals_list'])
Out[619]: 0.10011107689999509
In [620]: np.mean(corrs_df_Z['raw']-corrs_df_Z['residuals_list'])/np.std(corrs_df_Z['raw']-corrs_df_Z['residuals_list'], ddof=1)
Out[620]: 1.2657978353287107


tres =ttest_rel(corrs_df_Z['residuals_sesslist'], corrs_df_Z['residuals_list'])
se = np.std(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_list'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
print(tres, se)
# Ttest_relResult(statistic=11.606044959329683, pvalue=5.573378524013586e-20) 0.004604672857001271
In [19]: np.mean(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_list'])
Out[19]: 0.05344204020136179
In [20]: np.mean(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_list'])/np.std(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_list'], ddof=1)
Out[20]: 1.178415324182376




# tres = ttest_rel(corrs_df_Z['raw'], corrs_df_Z['residuals_sess'])
# se = np.std(corrs_df_Z['raw']-corrs_df_Z['residuals_sess'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
# print(tres, se)
# # Ttest_relResult(statistic=-13.950269412789632, pvalue=8.02967043740521e-25) 0.004707795434460214
In [621]: np.mean(corrs_df_Z['raw']-corrs_df_Z['residuals_sess'])
Out[621]: -0.06567501465102099
In [622]: np.mean(corrs_df_Z['raw']-corrs_df_Z['residuals_sess'])/np.std(corrs_df_Z['raw']-corrs_df_Z['residuals_sess'], ddof=1)
Out[622]: -1.4164352550856774

tres =ttest_rel(corrs_df_Z['residuals_sesslist'], corrs_df_Z['residuals_sess'])
se = np.std(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sess'], ddof=1)/np.sqrt(len(corrs_df_Z['item']))
print(tres, se)
# Ttest_relResult(statistic=-13.133856444894924, pvalue=3.6572367075044084e-23) 0.008553774881049653
In [21]: np.mean(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sess'])
Out[21]: -0.11234405134965425
In [22]: np.mean(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sess'])/np.std(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sess'], ddof=1)
Out[22]: -1.3335410774739525

# List shuffled

tres = ttest_1samp(corrs_df_Z['raw_shuffled'], 0)
se = np.std(corrs_df_Z['raw_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['raw_shuffled']))
print(tres, se)
# Ttest_1sampResult(statistic=-3.6358046014931706, pvalue=0.0004478710369844768) 0.0020952904478706317
# In [637]: np.tanh(np.mean(corrs_df_Z['raw_shuffled']))
# Out[637]: -0.007617919283907943

tres = ttest_1samp(corrs_df_Z['residuals_sesslist_shuffled'], 0)
se = np.std(corrs_df_Z['residuals_sesslist_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_sesslist_shuffled']))
print(tres, se)
# Ttest_1sampResult(statistic=-1.7336319020356756, pvalue=0.08619388414540445) 0.0015650234024759363
# In [638]: np.tanh(np.mean(corrs_df_Z['residuals_sesslist_shuffled']))
# Out[638]: -0.00271316784047282

tres = ttest_1samp(corrs_df_Z['residuals_list_shuffled'], 0)
se = np.std(corrs_df_Z['residuals_list_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_list_shuffled']))
print(tres, se)
# Ttest_1sampResult(statistic=-14.588920258188017, pvalue=4.271464409506547e-26) 0.0018072340751227552
# In [639]: np.tanh(np.mean(corrs_df_Z['residuals_list_shuffled']))
# Out[639]: -0.026359486208606613

tres = ttest_1samp(corrs_df_Z['residuals_sess_shuffled'], 0)
se = np.std(corrs_df_Z['residuals_sess_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_sess_shuffled']))
print(tres, se)
# Ttest_1sampResult(statistic=7.692368200369217, pvalue=1.2647600021505255e-11) 0.0033281925096128226
# In [640]: np.tanh(np.mean(corrs_df_Z['residuals_sess_shuffled']))
# Out[640]: 0.0255960901838918


tres =ttest_rel(corrs_df_Z['raw'], corrs_df_Z['raw_shuffled'])
se = np.std(corrs_df_Z['raw']-corrs_df_Z['raw_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['raw']))
print(tres, se)
# Ttest_relResult(statistic=19.75106633537931, pvalue=1.3878900245265288e-35) 0.013874255689339116
In [29]: np.mean(corrs_df_Z['raw']-corrs_df_Z['raw_shuffled'])
Out[29]: 0.2740313444741507
In [30]: np.mean(corrs_df_Z['raw']-corrs_df_Z['raw_shuffled'])/np.std(corrs_df_Z['raw']-corrs_df_Z['raw_shuffled'], ddof=10)
Out[30]: 1.9091000555120745

tres =ttest_rel(corrs_df_Z['residuals_sesslist'], corrs_df_Z['residuals_sesslist_shuffled'])
se = np.std(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sesslist_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_sesslist']))
print(tres, se)
# Ttest_relResult(statistic=14.285753398741248, pvalue=1.7091485660405256e-25) 0.015571976458816006
In [31]: np.mean(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sesslist_shuffled'])
Out[31]: 0.2224574156216494
In [32]: np.mean(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sesslist_shuffled'])/np.std(corrs_df_Z['residuals_sesslist']-corrs_df_Z['residuals_sesslist_shuffled'], ddof=1)
Out[32]: 1.4504984929456488

tres =ttest_rel(corrs_df_Z['residuals_list'], corrs_df_Z['residuals_list_shuffled'])
se = np.std(corrs_df_Z['residuals_list']-corrs_df_Z['residuals_list_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_list']))
print(tres, se)
# Ttest_relResult(statistic=14.646629885099484, pvalue=3.2846437648945825e-26) 0.013154411372692397
In [33]: np.mean(corrs_df_Z['residuals_list']-corrs_df_Z['residuals_list_shuffled'])
Out[33]: 0.192667794732169
In [34]: np.mean(corrs_df_Z['residuals_list']-corrs_df_Z['residuals_list_shuffled'])/np.std(corrs_df_Z['residuals_list']-corrs_df_Z['residuals_list_shuffled'], ddof=1)
Out[34]: 1.4871399485967223

tres =ttest_rel(corrs_df_Z['residuals_sess'], corrs_df_Z['residuals_sess_shuffled'])
se = np.std(corrs_df_Z['residuals_sess']-corrs_df_Z['residuals_sess_shuffled'], ddof=1)/np.sqrt(len(corrs_df_Z['residuals_sess']))
print(tres, se)
# Ttest_relResult(statistic=20.36061713081251, pvalue=1.3087958650194675e-36) 0.015052913586979065
In [35]: np.mean(corrs_df_Z['residuals_sess']-corrs_df_Z['residuals_sess_shuffled'])
Out[35]: 0.3064866102476862
In [36]: np.mean(corrs_df_Z['residuals_sess']-corrs_df_Z['residuals_sess_shuffled'])/np.std(corrs_df_Z['residuals_sess']-corrs_df_Z['residuals_sess_shuffled'], ddof=1)
Out[36]: 2.067307452352434


# corrs_df = pd.DataFrame.from_dict(corrs)
# corrs_df.index=subjs
# corrs_df_Z = np.arctanh(corrs_df)
# corrs_means = np.tanh(corrs_df_Z.mean())
# corrs_CI95_Z = corrs_df_Z.std(ddof=1)/np.sqrt(
#     len(corrs_df_Z)) * sp.stats.t.ppf(0.975, len(corrs_df_Z)-1)
# corrs_CI95_u = np.tanh(corrs_df_Z.mean()+corrs_CI95_Z)
# corrs_CI95_l = np.tanh(corrs_df_Z.mean()-corrs_CI95_Z)






# itemvars_plot = ['item', 'item_res_all', 'item_res_norecallability',
#                  'item_res_noserialpos', 'item_res_nosession',
#                  'item_res_notrial']
# itemvars_label_dict = {
#     'item': 'item', 'item_res_all': 'item|all',
#     'item_res_norecallability': r'item|$\neg$ recallability',
#     'item_res_noserialpos': r'item|$\neg$ serial pos.',
#     'item_res_nosession': r'item|$\neg$ session #',
#     'item_res_notrial': r'item|$\neg$ list #'}
# # make_gridplot(df=aucs_df, toplot_vars=itemvars_plot,
# #               toplot_labels=itemvars_label_dict, axlim=(0.45, 0.8), savefig=False)

# listvars_plot = ['raw', 'residuals_sesslist', 'residuals_list',
#                  'residuals_sess']



# itemvars_key_dict = {
#     'item': 'recalled', 'item_res_all': 'residuals_logit_all',
#     'item_res_norecallability': 'residuals_logit_norecallability',
#     'item_res_noserialpos': 'residuals_logit_noserialpos',
#     'item_res_nosession': 'residuals_logit_nosession',
#     'item_res_notrial': 'residuals_logit_notrial'}



itemvars_label_dict = {
    'item': 'item', 'item_res_all': 'item|all',
    'item_res_norecallability': r'item|$\neg$ recallability',
    'item_res_noserialpos': r'item|$\neg$ serial pos.',
    'item_res_nosession': r'item|$\neg$ session #',
    'item_res_notrial': r'item|$\neg$ list #',
}
# for itemvar in itemvars_label_dict:
#     if itemvar == item:
#         continue
#     itemvars_label_dict['item-'+itemvar] = 'item-'+itemvars_label_dict['itemvar']
# listvars_label_dict = {
#     'raw': r'list', 'residuals_sesslist': r'list|all',
#     'residuals_sess': r'list|session #', 'residuals_list': r'list|list #'}

lineprops = {'item': {'c': 'k', 'ls': '-'},
             'item_res_all': {'c': 'k', 'ls': '--'},
             'item_res_norecallability': {'c': 'k', 'ls': '--'},
             'item_res_noserialpos': {'c': 'k', 'ls': '--'},
             'item_res_nosession': {'c': 'k', 'ls': '--'},
             'item_res_notrial': {'c': 'k', 'ls': '--'},
             'item_resres': {'c': 'g', 'ls': ':'},
             'raw': {'c': 'b', 'ls': '-'},
             'residuals_sesslist': {'c': 'b', 'ls': '--'},
             'residuals_sess': {'c': 'b', 'ls': '--'},
             'residuals_list': {'c': 'b', 'ls': '--'},
             'list_resres': {'c': 'c', 'ls': ':'}}



fig, axs = plt.subplots(2, 6, sharex=True, sharey='row', figsize=(10, 3))
for i, dv in enumerate(['item']+itemvars):
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[0, i].plot(
        corrs_roi['frequency'], corr_mean, label=itemvars_label_dict[dv], **lineprops[dv])
    axs[0, i].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[0, i].plot([2, 200], [0, 0], 'k-')
    axs[0, i].set_xscale('log')
    axs[0, i].text(0.95, 0.1, itemvars_label_dict[dv], horizontalalignment='right',
                   verticalalignment='center', transform=axs[0, i].transAxes)
    if dv == 'item':
        continue
    dv_new = dv+'-item'
    dv_new = 'item-'+dv
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[0, i].plot(
        corrs_roi['frequency'], corr_mean,
        label=itemvars_label_dict[dv]+'-item', **lineprops['item_resres'])
    axs[0, i].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops['item_resres']['c'], edgecolor=lineprops['item_resres']['c'])
axs[0, i].set_ylim((-0.11, 0.05689327)) 
for i, dv in enumerate(listvars):
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[1, i].plot(
        corrs_roi['frequency'], corr_mean, label=listvars_label_dict[dv], **lineprops[dv])
    axs[1, i].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[1, i].plot([2, 200], [0, 0], 'k-')
    axs[1, i].set_xscale('log')
    axs[1, i].text(0.95, 0.1, listvars_label_dict[dv], horizontalalignment='right',
                   verticalalignment='center', transform=axs[1, i].transAxes)
    if dv == 'raw':
        continue
    # dv_new = dv+'-prec'
    dv_new = 'prec-'+dv
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[1, i].plot(
        corrs_roi['frequency'], corr_mean,
        label=listvars_label_dict[dv]+'-list', **lineprops['list_resres'])
    axs[1, i].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops['list_resres']['c'], edgecolor=lineprops['list_resres']['c'])
        
axs[1, 0].set_xlabel('Frequency')
axs[0, 0].set_ylabel('Correlation ($r$)')
axs[1, 0].set_ylabel('Correlation ($r$)')
axs[1, 4].set_visible(False)
axs[1, 5].set_visible(False)
fig.subplots_adjust(left=0.075, bottom=0.16, right=0.995, top=0.99,
                    wspace=0.05, hspace=0.04)
fig.savefig('./figs/corrs_all.pdf')









fig, axs = plt.subplots(6, 2, sharex=True, sharey=True, figsize=(3,9))
for i, dv in enumerate(['item']+itemvars):
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[i, 0].plot(
        corrs_roi['frequency'], corr_mean, label=itemvars_label_dict[dv], **lineprops[dv])
    axs[i, 0].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[i, 0].plot([2, 200], [0, 0], 'k-')
    axs[i, 0].set_xscale('log')
    if dv == 'item':
        continue
    dv_new = dv+'-item'
    dv_new = 'item-'+dv
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[i, 0].plot(
        corrs_roi['frequency'], corr_mean,
        label=itemvars_label_dict[dv]+'-item', **lineprops['item_resres'])
    axs[i, 0].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops['item_resres']['c'], edgecolor=lineprops['item_resres']['c'])
    
axs[i, 0].set_xlabel('Frequency')
axs[i, 0].set_ylabel('Correlation ($r$)')
for i, dv in enumerate(listvars):
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[i, 1].plot(
        corrs_roi['frequency'], corr_mean, label=listvars_label_dict[dv], **lineprops[dv])
    axs[i, 1].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[i, 1].plot([2, 200], [0, 0], 'k-')
    axs[i, 1].set_xscale('log')
    if dv == 'raw':
        continue
    # dv_new = dv+'-prec'
    dv_new = 'prec-'+dv
    corr_mean = np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').mean('subject'))
    corr_SE = (np.tanh(np.arctanh(corrs_roi.sel(dvtype=dv_new, ROI=np.array(
        [r in selected_rois for r in corrs_roi['ROI']]))).mean(
            'ROI').std('subject', ddof=1)) /
               np.sqrt(len(corrs_roi['subject'])))
    corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
    axs[i, 1].plot(
        corrs_roi['frequency'], corr_mean,
        label=listvars_label_dict[dv]+'-list', **lineprops['list_resres'])
    axs[i, 1].fill_between(
        corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
        alpha=0.5, color=lineprops['list_resres']['c'], edgecolor=lineprops['list_resres']['c'])
        
fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
                    wspace=0.05, hspace=0.04)




lineprops = {'interlist': {'c': 'k', 'ls': '-'}, 'intersess': {'c': 'b', 'ls': '--'}}
linelabels = {'interlist': 'interlist', 'intersess': 'intersession'}
fig, axs = plt.subplots(4, 2, sharex=True, sharey=True, figsize=(6,10))
for roi in corrs_roi['ROI']:
    for dv in ['interlist', 'intersess']:
        corr_mean = np.tanh(np.arctanh(
            corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
        corr_SE = (
            np.tanh(np.arctanh(
                corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', ddof=1)) /
            np.sqrt(len(corrs_roi['subject'])))
        corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
        axs[roi_coords[roi.values.tolist()]].plot(
            corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
        axs[roi_coords[roi.values.tolist()]].fill_between(
            corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
            alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
    axs[roi_coords[roi.values.tolist()]].set_xscale('log')
    if roi_coords[roi.values.tolist()][0] == 3:
        axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
    if roi_coords[roi.values.tolist()][1] == 0:
        axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
    if roi_coords[roi.values.tolist()][0] % 2 == 0:
        y = 0.05
    else:
        y = 0.95
    if roi_coords[roi.values.tolist()][1] == 0:
        x = 0.92
    else:
        x = 0.08
    axs[roi_coords[roi.values.tolist()]].text(
        x, y, roi.values.tolist(), horizontalalignment='center',
        fontweight='bold', verticalalignment='center',
        transform=axs[roi_coords[roi.values.tolist()]].transAxes)
fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
                    wspace=0.05, hspace=0.04)
axs[3, 1].legend(loc='lower right')
# In [50]: axs[3, 1].get_ylim()
# Out[50]: (-0.14718259680331391, 0.05682513429183883)
axs[3, 1].set_ylim((-0.145, 0.055))
axs[3, 1].set_yticks([0.05, 0, -0.05, -0.1])

# ax = fig.add_subplot(111)
# ax = fig.add_axes([0.4, 0.49, 0.32, 0.15], aspect='equal')
# ax = fig.add_axes([0.4, 0.5, 0.32, 0.15], aspect='equal')
ax = fig.add_axes([0.4, 0.48, 0.32, 0.15], aspect='equal')
# topoplot(center=(0.5,0.5), radius=0.4, axes=ax)
roi_sensors = tuple(s[np.array([c in np.ravel(list(rois['biosemi'].values()))
                           for c in corrs_dict['biosemi']['channel']])]
               for s in sensors['biosemi'])
topoplot(sensors=sensors['biosemi'],
         head_props=dict(head_linewidth=1, nose_linewidth=1, ear_linewidth=1),
         values=np.zeros(len(sensors['biosemi'][0])), cmap='bwr',
         axes=ax, sensor_props=dict(marker=''))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='green'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='green'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='purple'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='purple'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
fig.savefig('./figs/corrs_roi_listsess.pdf')




lineprops = {'item': {'c': 'k', 'ls': '-'}, 'prec': {'c': 'b', 'ls': '--'}}
linelabels = {'item': 'item', 'prec': r'$p($rec$)$'}
fig, axs = plt.subplots(4, 2, sharex=True, sharey=True, figsize=(6,10))
for roi in corrs_roi['ROI']:
    for dv in ['item', 'prec']:
        corr_mean = np.tanh(np.arctanh(
            corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
        corr_SE = (
            np.tanh(np.arctanh(
                corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', ddof=1)) /
            np.sqrt(len(corrs_roi['subject'])))
        corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
        axs[roi_coords[roi.values.tolist()]].plot(
            corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
        axs[roi_coords[roi.values.tolist()]].fill_between(
            corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
            alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
    axs[roi_coords[roi.values.tolist()]].set_xscale('log')
    if roi_coords[roi.values.tolist()][0] == 3:
        axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
    if roi_coords[roi.values.tolist()][1] == 0:
        axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
    if roi_coords[roi.values.tolist()][0] % 2 == 0:
        y = 0.05
    else:
        y = 0.95
    if roi_coords[roi.values.tolist()][1] == 0:
        x = 0.92
    else:
        x = 0.08
    axs[roi_coords[roi.values.tolist()]].text(
        x, y, roi.values.tolist(), horizontalalignment='center',
        fontweight='bold', verticalalignment='center',
        transform=axs[roi_coords[roi.values.tolist()]].transAxes)
fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
                    wspace=0.05, hspace=0.04)
axs[3, 1].legend(loc='lower right')
axs[3, 1].set_ylim((-0.145, 0.055))
axs[3, 1].set_yticks([0.05, 0, -0.05, -0.1])

# ax = fig.add_subplot(111)    
# ax = fig.add_axes([0.4, 0.49, 0.32, 0.15], aspect='equal')
# ax = fig.add_axes([0.4, 0.5, 0.32, 0.15], aspect='equal')
ax = fig.add_axes([0.4, 0.48, 0.32, 0.15], aspect='equal')
# topoplot(center=(0.5,0.5), radius=0.4, axes=ax)
roi_sensors = tuple(s[np.array([c in np.ravel(list(rois['biosemi'].values()))
                           for c in corrs_dict['biosemi']['channel']])]
               for s in sensors['biosemi'])
#roi_sensors = tuple(s[np.array([c in np.ravel(list(rois['biosemi']['RPI']))
#                             for c in corrs_dict['biosemi']['channel']])]
#               for s in sensors['biosemi'])
topoplot(sensors=sensors['biosemi'],
         head_props=dict(head_linewidth=1, nose_linewidth=1, ear_linewidth=1),
         values=np.zeros(len(sensors['biosemi'][0])), cmap='bwr',
         axes=ax, sensor_props=dict(marker=''))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='green'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='green'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='purple'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='purple'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
fig.savefig('./figs/corrs_roi_LRraw.pdf')










lineprops = {'item': {'c': 'k', 'ls': '-'}, 'prec': {'c': 'b', 'ls': '--'},
             'interlist': {'c': 'r', 'ls': '-.'},
             'intersess': {'c': 'm', 'ls': ':'},
             'prec-interlist': {'c': 'b', 'ls': '--'},
             'prec-intersess': {'c': 'b', 'ls': '--'},
             'interlist-prec': {'c': 'r', 'ls': '-.'},
             'intersess-prec': {'c': 'm', 'ls': ':'}}

linelabels = {'item': 'item', 'prec': r'$p($rec$)$',
              'interlist': 'interlist', 'intersess': 'intersession'}
fig, axs = plt.subplots(4, 5, sharex=True, sharey=True, figsize=(10, 7),
                        gridspec_kw={'width_ratios': [15, 15, 1, 15, 15]})
for roi in corrs_roi['ROI']:
    for dv in ['item', 'prec']:
        corr_mean = np.tanh(np.arctanh(
            corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
        corr_SE = (
            np.tanh(np.arctanh(
                corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', ddof=1)) /
            np.sqrt(len(corrs_roi['subject'])))
        corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
        axs[roi_coords[roi.values.tolist()]].plot(
            corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
        axs[roi_coords[roi.values.tolist()]].fill_between(
            corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
            alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
    axs[roi_coords[roi.values.tolist()]].set_xscale('log')
    if roi_coords[roi.values.tolist()][0] == 3:
        axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
    if roi_coords[roi.values.tolist()][1] == 0:
        axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
    if roi_coords[roi.values.tolist()][0] % 2 == 0:
        y = 0.1  # 0.05
    else:
        y = 0.9  # 0.95
    if roi_coords[roi.values.tolist()][1] == 0:
        x = 0.9  # 0.92
    else:
        x = 0.1  # 0.08
    axs[roi_coords[roi.values.tolist()]].text(
        x, y, roi.values.tolist(), horizontalalignment='center',
        fontweight='bold', verticalalignment='center',
        transform=axs[roi_coords[roi.values.tolist()]].transAxes)
fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
                    wspace=0.05, hspace=0.04)
axs[3, 1].legend(loc='lower right', frameon=False, borderpad=0)
axs[3, 1].set_ylim((-0.145, 0.055))
axs[3, 1].set_yticks([0.05, 0, -0.05, -0.1])

# [left, bottom, width, height]
# ax = fig.add_axes([0.4, 0.49, 0.32, 0.15], aspect='equal')
# ax = fig.add_axes([0.4, 0.5, 0.32, 0.15], aspect='equal')
# ax = fig.add_axes([0.375, 0.48, 0.32, 0.15], aspect='equal')
# ax = fig.add_axes([0.325, 0.482, 0.42, 0.17], aspect='equal')
# ax = fig.add_axes([0.3265, 0.482, 0.42, 0.165], aspect='equal')
# ax = fig.add_axes([0.3265, 0.485, 0.42, 0.16], aspect='equal')
# ax = fig.add_axes([0.3265, 0.485, 0.42, 0.17], aspect='equal')
# ax = fig.add_axes([0.3265, 0.484, 0.42, 0.1725], aspect='equal')
# ax = fig.add_axes([0.3265, 0.484, 0.42, 0.173], aspect='equal')
ax = fig.add_axes([0.3265, 0.484, 0.42, 0.173], aspect='equal')
# ax = fig.add_axes([0.4, 0.48, 0.32, 0.15], aspect='equal')
# topoplot(center=(0.5,0.5), radius=0.4, axes=ax)
roi_sensors = tuple(s[np.array([c in np.ravel(list(rois['biosemi'].values()))
                           for c in corrs_dict['biosemi']['channel']])]
               for s in sensors['biosemi'])
#roi_sensors = tuple(s[np.array([c in np.ravel(list(rois['biosemi']['RPI']))
#                             for c in corrs_dict['biosemi']['channel']])]
#               for s in sensors['biosemi'])
topoplot(sensors=sensors['biosemi'],
         head_props=dict(head_linewidth=1, nose_linewidth=1, ear_linewidth=1),
         values=np.zeros(len(sensors['biosemi'][0])), cmap='bwr',
         axes=ax, sensor_props=dict(marker=''))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='full',
         sensor_props=dict(marker='<', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='full',
         sensor_props=dict(marker='>', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='none',
         sensor_props=dict(marker='<', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='none',
         sensor_props=dict(marker='>', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='none',
         sensor_props=dict(marker='<', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='none',
         sensor_props=dict(marker='>', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='full',
         sensor_props=dict(marker='<', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, fillstyle='full',
         sensor_props=dict(marker='>', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))


for roi in corrs_roi['ROI']:
    panel_indx = tuple(np.array(roi_coords[roi.values.tolist()]) + [0, 3])
    for dv in ['interlist', 'intersess']:
        corr_mean = np.tanh(np.arctanh(
            corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
        corr_SE = (
            np.tanh(np.arctanh(
                corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', ddof=1)) /
            np.sqrt(len(corrs_roi['subject'])))
        corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
        axs[panel_indx].plot(
            corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
        axs[panel_indx].fill_between(
            corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
            alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[panel_indx].plot([2, 200], [0, 0], 'k-')
    axs[panel_indx].set_xscale('log')
    if panel_indx[0] == 3:
        axs[panel_indx].set_xlabel('Frequency')
    if panel_indx[1] == 0:
        axs[panel_indx].set_ylabel('Correlation ($r$)')
    if panel_indx[0] % 2 == 0:
        y = 0.1 # 0.05
    else:
        y = 0.9 # 0.95
    if panel_indx[1] == 3:
        x = 0.9 #0.92
    else:
        x = 0.1 #0.08
    axs[panel_indx].text(
        x, y, roi.values.tolist(), horizontalalignment='center',
        fontweight='bold', verticalalignment='center',
        transform=axs[panel_indx].transAxes)
for i in range(len(axs)):
    axs[i, 2].set_visible(False)
# fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
#                     wspace=0.05, hspace=0.04)
fig.subplots_adjust(left=0.075, bottom=0.065, right=0.998, top=0.995,
                    wspace=0.06, hspace=0.06)
axs[3, 4].legend(loc='lower right', frameon=False, borderpad=0)
# In [50]: axs[3, 1].get_ylim()
# Out[50]: (-0.14718259680331391, 0.05682513429183883)
axs[3, 4].set_ylim((-0.145, 0.055))
axs[3, 4].set_yticks([0.05, 0, -0.05, -0.1])
fig.savefig('./figs/corrs_roi_all.pdf')






# lineprops = {'item': {'c': 'k', 'ls': '-'}, 'prec': {'c': 'b', 'ls': '--'},
#              'interlist': {'c': 'r', 'ls': '-.'},
#              'intersess': {'c': 'm', 'ls': ':'}}
# linelabels = {'item': 'LR', 'prec': r'$p($rec$)$',
#               'interlist': 'interlist', 'intersess': 'intersession'}
lineprops = {'intersess-prec': {'c': 'c', 'ls': '-'},
             'prec-interlist': {'c': 'g', 'ls': '--'}}
linelabels = {'intersess-prec': 'intersessession | $p($rec$)$',
              'prec-interlist': '$p($rec$)$ | interlist'}
fig, axs = plt.subplots(4, 2, sharex=True, sharey=True, figsize=(5, 7))  #,
#                        gridspec_kw={'width_ratios': [15, 15, 1, 15, 15]})
for roi in corrs_roi['ROI']:
    for dv in ['intersess-prec', 'prec-interlist']:
        corr_mean = np.tanh(np.arctanh(
            corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
        corr_SE = (
            np.tanh(np.arctanh(
                corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', skipna=False, ddof=1)) /
            np.sqrt(len(corrs_roi['subject'])))
        corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
        axs[roi_coords[roi.values.tolist()]].plot(
            corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
        axs[roi_coords[roi.values.tolist()]].fill_between(
            corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
            alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
    axs[roi_coords[roi.values.tolist()]].set_xscale('log')
    if roi_coords[roi.values.tolist()][0] == 3:
        axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
    if roi_coords[roi.values.tolist()][1] == 0:
        axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
    if roi_coords[roi.values.tolist()][0] % 2 == 0:
        y = 0.1  # 0.05
    else:
        y = 0.9  # 0.95
    if roi_coords[roi.values.tolist()][1] == 0:
        x = 0.9  # 0.92
    else:
        x = 0.1  # 0.08
    axs[roi_coords[roi.values.tolist()]].text(
        x, y, roi.values.tolist(), horizontalalignment='center',
        fontweight='bold', verticalalignment='center',
        transform=axs[roi_coords[roi.values.tolist()]].transAxes)
# fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
#                     wspace=0.05, hspace=0.04)
fig.subplots_adjust(left=0.135, bottom=0.07, right=0.995, top=0.995,
                    wspace=0.05, hspace=0.04)
axs[3, 1].legend(loc='lower center', frameon=False, borderpad=0)
axs[3, 1].set_ylim((-0.36, 0.07))
axs[3, 1].set_yticks(np.arange(0, -0.31, -0.1))
fig.savefig('./figs/corrs_roi_partial.pdf')





lineprops = {'intersess-prec': {'c': 'c', 'ls': '-'},
             'interlist-prec': {'c': 'g', 'ls': '--'}}
linelabels = {'intersess-prec': 'intersessession | $p($rec$)$',
              'interlist-prec': 'interlist | $p($rec$)$'}
fig, axs = plt.subplots(4, 2, sharex=True, sharey=True, figsize=(5, 7))  #,
#                        gridspec_kw={'width_ratios': [15, 15, 1, 15, 15]})
for roi in corrs_roi['ROI']:
    for dv in ['intersess-prec', 'interlist-prec']:
        corr_mean = np.tanh(np.arctanh(
            corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
        corr_SE = (
            np.tanh(np.arctanh(
                corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', skipna=False, ddof=1)) /
            np.sqrt(len(corrs_roi['subject'])))
        corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
        axs[roi_coords[roi.values.tolist()]].plot(
            corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
        axs[roi_coords[roi.values.tolist()]].fill_between(
            corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
            alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
    axs[roi_coords[roi.values.tolist()]].set_xscale('log')
    if roi_coords[roi.values.tolist()][0] == 3:
        axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
    if roi_coords[roi.values.tolist()][1] == 0:
        axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
    if roi_coords[roi.values.tolist()][0] % 2 == 0:
        y = 0.1  # 0.05
    else:
        y = 0.9  # 0.95
    if roi_coords[roi.values.tolist()][1] == 0:
        x = 0.9  # 0.92
    else:
        x = 0.1  # 0.08
    axs[roi_coords[roi.values.tolist()]].text(
        x, y, roi.values.tolist(), horizontalalignment='center',
        fontweight='bold', verticalalignment='center',
        transform=axs[roi_coords[roi.values.tolist()]].transAxes)
# fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
#                     wspace=0.05, hspace=0.04)
fig.subplots_adjust(left=0.135, bottom=0.07, right=0.995, top=0.995,
                    wspace=0.05, hspace=0.04)
axs[3, 1].legend(loc='lower center', frameon=False, borderpad=0)
axs[3, 1].set_ylim((-0.16, 0.24))
#axs[3, 1].set_yticks(np.arange(0, -0.31, -0.1))
fig.savefig('./figs/corrs_roi_partial_intersessinterlist.pdf')


# lineprops = {'intersess-prec': {'c': 'c', 'ls': '-'},
#              'interlist-prec': {'c': 'g', 'ls': '--'},
#              'prec-interlist': {'c': 'b', 'ls': ':'}}
# linelabels = {'intersess-prec': 'intersessession | $p($rec$)$',
#               'interlist-prec': 'interlist | $p($rec$)$',
#               'prec-interlist': '$p($rec$)$ | interlist'}
# fig, axs = plt.subplots(4, 2, sharex=True, sharey=True, figsize=(5, 7))  #,
# #                        gridspec_kw={'width_ratios': [15, 15, 1, 15, 15]})
# for roi in corrs_roi['ROI']:
#     for dv in ['intersess-prec', 'interlist-prec', 'prec-interlist']:
#         corr_mean = np.tanh(np.arctanh(
#             corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
#         corr_SE = (
#             np.tanh(np.arctanh(
#                 corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', skipna=False, ddof=1)) /
#             np.sqrt(len(corrs_roi['subject'])))
#         corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
#         axs[roi_coords[roi.values.tolist()]].plot(
#             corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
#         axs[roi_coords[roi.values.tolist()]].fill_between(
#             corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
#             alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
#     axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
#     axs[roi_coords[roi.values.tolist()]].set_xscale('log')
#     if roi_coords[roi.values.tolist()][0] == 3:
#         axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
#     if roi_coords[roi.values.tolist()][1] == 0:
#         axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
#     if roi_coords[roi.values.tolist()][0] % 2 == 0:
#         y = 0.1  # 0.05
#     else:
#         y = 0.9  # 0.95
#     if roi_coords[roi.values.tolist()][1] == 0:
#         x = 0.9  # 0.92
#     else:
#         x = 0.1  # 0.08
#     axs[roi_coords[roi.values.tolist()]].text(
#         x, y, roi.values.tolist(), horizontalalignment='center',
#         fontweight='bold', verticalalignment='center',
#         transform=axs[roi_coords[roi.values.tolist()]].transAxes)
# # fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
# #                     wspace=0.05, hspace=0.04)
# fig.subplots_adjust(left=0.135, bottom=0.07, right=0.995, top=0.995,
#                     wspace=0.05, hspace=0.04)
# axs[3, 1].legend(loc='lower center', frameon=False, borderpad=0)
# #axs[3, 1].set_ylim((-0.16, 0.24))
# #axs[3, 1].set_yticks(np.arange(0, -0.31, -0.1))
# fig.savefig('./figs/corrs_roi_partial_all.pdf')




# lineprops = {'prec-intersess': {'c': 'c', 'ls': '-'},
#              'prec-interlist': {'c': 'g', 'ls': '--'}}
# linelabels = {'prec-intersess': '$p($rec$)$ | intersession',
#               'prec-interlist': '$p($rec$)$ | interlist'}
# fig, axs = plt.subplots(4, 2, sharex=True, sharey=True, figsize=(5, 7))  #,
# #                        gridspec_kw={'width_ratios': [15, 15, 1, 15, 15]})
# for roi in corrs_roi['ROI']:
#     for dv in ['prec-intersess']: #, 'prec-interlist']:
#     # for dv in ['prec-intersess', 'prec-interlist']:
#         corr_mean = np.tanh(np.arctanh(
#             corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
#         corr_SE = (
#             np.tanh(np.arctanh(
#                 corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', skipna=False, ddof=1)) /
#             np.sqrt(len(corrs_roi['subject'])))
#         corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
#         axs[roi_coords[roi.values.tolist()]].plot(
#             corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
#         axs[roi_coords[roi.values.tolist()]].fill_between(
#             corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
#             alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
#     #axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
#     axs[roi_coords[roi.values.tolist()]].set_xscale('log')
#     if roi_coords[roi.values.tolist()][0] == 3:
#         axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
#     if roi_coords[roi.values.tolist()][1] == 0:
#         axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
#     if roi_coords[roi.values.tolist()][0] % 2 == 0:
#         y = 0.1  # 0.05
#     else:
#         y = 0.9  # 0.95
#     if roi_coords[roi.values.tolist()][1] == 0:
#         x = 0.9  # 0.92
#     else:
#         x = 0.1  # 0.08
#     axs[roi_coords[roi.values.tolist()]].text(
#         x, y, roi.values.tolist(), horizontalalignment='center',
#         fontweight='bold', verticalalignment='center',
#         transform=axs[roi_coords[roi.values.tolist()]].transAxes)
# # fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
# #                     wspace=0.05, hspace=0.04)
# fig.subplots_adjust(left=0.135, bottom=0.07, right=0.995, top=0.995,
#                     wspace=0.05, hspace=0.04)
# axs[3, 1].legend(loc='lower center', frameon=False, borderpad=0)


# corrs_roi.sel(dvtype='prec-intersess').min()
# corrs_roi.sel(dvtype='prec-intersess').max();p



# for roi in corrs_roi['ROI']:
#     panel_indx = tuple(np.array(roi_coords[roi.values.tolist()]) + [0, 3])
#     for dv in ['interlist', 'intersess']:
#         corr_mean = np.tanh(np.arctanh(
#             corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
#         corr_SE = (
#             np.tanh(np.arctanh(
#                 corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', ddof=1)) /
#             np.sqrt(len(corrs_roi['subject'])))
#         corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
#         axs[panel_indx].plot(
#             corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
#         axs[panel_indx].fill_between(
#             corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
#             alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
#     axs[panel_indx].plot([2, 200], [0, 0], 'k-')
#     axs[panel_indx].set_xscale('log')
#     if panel_indx[0] == 3:
#         axs[panel_indx].set_xlabel('Frequency')
#     if panel_indx[1] == 0:
#         axs[panel_indx].set_ylabel('Correlation ($r$)')
#     if panel_indx[0] % 2 == 0:
#         y = 0.1 # 0.05
#     else:
#         y = 0.9 # 0.95
#     if panel_indx[1] == 3:
#         x = 0.9 #0.92
#     else:
#         x = 0.1 #0.08
#     axs[panel_indx].text(
#         x, y, roi.values.tolist(), horizontalalignment='center',
#         fontweight='bold', verticalalignment='center',
#         transform=axs[panel_indx].transAxes)
# for i in range(len(axs)):
#     axs[i, 2].set_visible(False)
# # fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
# #                     wspace=0.05, hspace=0.04)
# fig.subplots_adjust(left=0.075, bottom=0.065, right=0.998, top=0.995,
#                     wspace=0.06, hspace=0.06)
# axs[3, 4].legend(loc='lower right', frameon=False, borderpad=0)
# # In [50]: axs[3, 1].get_ylim()
# # Out[50]: (-0.14718259680331391, 0.05682513429183883)
# axs[3, 4].set_ylim((-0.145, 0.055))
# axs[3, 4].set_yticks([0.05, 0, -0.05, -0.1])
# fig.savefig('./figs/corrs_roi_partial.pdf')
















lineprops = {'item': {'c': 'k', 'ls': '-'}, 'prec': {'c': 'b', 'ls': '--'}}
linelabels = {'item': 'LR', 'prec': r'$p($rec$)$'}
fig, axs = plt.subplots(4, 2, sharex=True, sharey=True, figsize=(6,10))
for roi in corrs_roi['ROI']:
    for dv in ['item', 'prec']:
        corr_mean = np.tanh(np.arctanh(
            corrs_roi.sel(dvtype=dv, ROI=roi)).mean('subject'))
        corr_SE = (
            np.tanh(np.arctanh(
                corrs_roi.sel(dvtype=dv, ROI=roi)).std('subject', ddof=1)) /
            np.sqrt(len(corrs_roi['subject'])))
        corr_CI95 = corr_SE * sp.stats.t.ppf(0.975, len(corrs_roi['subject'])-1)
        axs[roi_coords[roi.values.tolist()]].plot(
            corrs_roi['frequency'], corr_mean, label=linelabels[dv], **lineprops[dv])
        axs[roi_coords[roi.values.tolist()]].fill_between(
            corrs_roi['frequency'], corr_mean+corr_CI95, corr_mean-corr_CI95,
            alpha=0.5, color=lineprops[dv]['c'], edgecolor=lineprops[dv]['c'])
    axs[roi_coords[roi.values.tolist()]].plot([2, 200], [0, 0], 'k-')
    axs[roi_coords[roi.values.tolist()]].set_xscale('log')
    if roi_coords[roi.values.tolist()][0] == 3:
        axs[roi_coords[roi.values.tolist()]].set_xlabel('Frequency')
    if roi_coords[roi.values.tolist()][1] == 0:
        axs[roi_coords[roi.values.tolist()]].set_ylabel('Correlation ($r$)')
    if roi_coords[roi.values.tolist()][0] % 2 == 0:
        y = 0.05
    else:
        y = 0.95
    if roi_coords[roi.values.tolist()][1] == 0:
        x = 0.92
    else:
        x = 0.08
    axs[roi_coords[roi.values.tolist()]].text(
        x, y, roi.values.tolist(), horizontalalignment='center',
        fontweight='bold', verticalalignment='center',
        transform=axs[roi_coords[roi.values.tolist()]].transAxes)
fig.subplots_adjust(left=0.125, bottom=0.05, right=0.995, top=0.995,
                    wspace=0.05, hspace=0.04)
axs[3, 1].legend(loc='lower right')
axs[3, 1].set_ylim((-0.145, 0.055))
axs[3, 1].set_yticks([0.05, 0, -0.05, -0.1])

# ax = fig.add_subplot(111)    
# ax = fig.add_axes([0.4, 0.49, 0.32, 0.15], aspect='equal')
# ax = fig.add_axes([0.4, 0.5, 0.32, 0.15], aspect='equal')
ax = fig.add_axes([0.4, 0.48, 0.32, 0.15], aspect='equal')
# topoplot(center=(0.5,0.5), radius=0.4, axes=ax)
roi_sensors = tuple(s[np.array([c in np.ravel(list(rois['biosemi'].values()))
                           for c in corrs_dict['biosemi']['channel']])]
               for s in sensors['biosemi'])
#roi_sensors = tuple(s[np.array([c in np.ravel(list(rois['biosemi']['RPI']))
#                             for c in corrs_dict['biosemi']['channel']])]
#               for s in sensors['biosemi'])
topoplot(sensors=sensors['biosemi'],
         head_props=dict(head_linewidth=1, nose_linewidth=1, ear_linewidth=1),
         values=np.zeros(len(sensors['biosemi'][0])), cmap='bwr',
         axes=ax, sensor_props=dict(marker=''))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='green'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='green'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RAS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='black'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPS']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='gray'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))

roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['LPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='<', c='purple'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
roi_sensors = tuple([s[np.array([c in np.ravel(list(rois['biosemi']['RPI']))
                                for c in corrs_dict['biosemi']['channel']])]
                     for s in sensors['biosemi']])
topoplot(sensors=roi_sensors, sensor_props=dict(marker='>', c='purple'), axes=ax,
         head_props=dict(head_linewidth=0, nose_linewidth=0, ear_linewidth=0))
fig.savefig('./figs/corrs_roi_LRraw.pdf')
































































regtypes = ['prec', 'intersess', 'interlist', 'sesslist']
correlations = {'egi': {rt: [] for rt in regtypes},
                'biosemi': {rt: [] for rt in regtypes}}

for regfile in regfiles:
    with h5py.File(regfile) as rf:
        subj = rf['events/subject'][0].decode()
        if np.int(subj[-3:]) <= 330:
            eegsys = 'egi'
        else:
            eegsys = 'biosemi'
        if 'features_chans' not in correlations[eegsys]:
            correlations[eegsys]['features_chans'] = np.array(
                rf['features_chans'].value, dtype=np.str)
            correlations[eegsys]['features_freqs'] = rf['features_freqs'].value
        else:
            assert(np.all(
                correlations[eegsys]['features_chans'] == np.array(
                    rf['features_chans'].value, dtype=np.str)))
            assert(np.all(
                correlations[eegsys]['features_freqs'] == rf[
                    'features_freqs'].value))
        for regtype in regtypes:
            if regtype == 'prec':
                dv = rf['recalled_logit']
            else:
                dv = rf['residuals_'+regtype]
            feat = rf['features_dat'][np.isfinite(dv), :]
            dv = dv[np.isfinite(dv)]
            # need to zero-center for below calculation to be a
            # correlation (feat is already zero-centered - for
            # residuals (but not prec), dv, should be too, but no harm
            # in making sure):
            dv -= dv.mean()
            correlations[eegsys][regtype].append(
                np.dot(dv, feat)/(np.linalg.norm(dv) *
                                  np.linalg.norm(feat, axis=0)))

for eegsys in correlations:
    correlations[eegsys]['item'] = []
for classfile in classfiles:
    with h5py.File(classfile) as cf:
        subj = cf['events/subject'][0].decode()
        if np.int(subj[-3:]) <= 330:
            eegsys = 'egi'
        else:
            eegsys = 'biosemi'
        if 'features_chans' not in correlations[eegsys]:
            correlations[eegsys]['features_chans'] = np.array(
                cf['features_chans'].value, dtype=np.str)
            correlations[eegsys]['features_freqs'] = cf['features_freqs'].value
        else:
            assert(np.all(
                correlations[eegsys]['features_chans'] == np.array(
                    cf['features_chans'].value, dtype=np.str)))
            assert(np.all(
                correlations[eegsys]['features_freqs'] == cf[
                    'features_freqs'].value))
        dv = cf['recalled']
        feat = cf['features_dat'][np.isfinite(dv), :]
        dv = dv[np.isfinite(dv)]
        # need to zero-center for below calculation to be a
        # correlation (feat is already zero-centered):
        dv = dv-dv.mean()
        # using the above syntax rather than -=, because -= throws a
        # TypeError if dv is int.
        correlations[eegsys]['item'].append(
            np.dot(dv, feat)/(np.linalg.norm(dv) *
                              np.linalg.norm(feat, axis=0)))


feature_info = ['features_chans', 'features_freqs']
corr_t = {}
for eegsys in correlations:
    feat_inf_indx = pd.MultiIndex.from_arrays(
        [correlations[eegsys]['features_chans'],
         correlations[eegsys]['features_freqs']],
        names=['channel', 'frequency'])
    corr_tvals = {}
    for regtype in ['item'] + regtypes:
        corr_z = np.arctanh(correlations[eegsys][regtype])
        corr_SE = corr_z.std(0, ddof=1) / np.sqrt(len(corr_z))
        corr_tvals[regtype] = corr_z.mean(0)/corr_SE
    corr_t[eegsys] = xr.DataArray(
        data=[corr_tvals[rt] for rt in ['item']+regtypes],
        coords=[('regtype', ['item']+regtypes),
                ('feat_inf', feat_inf_indx)]).unstack('feat_inf')
    # xarray bug (?) makes strings objects, so convert back to string:
    corr_t[eegsys]['channel'] = np.array(corr_t[eegsys]['channel'], np.str)

maxval = 7
fig, axs = plt.subplots(15, 5, sharex=True, sharey=True, figsize=(5, 11), subplot_kw=dict(aspect='equal'))
plt.subplots_adjust(left=0.0001, bottom=0.0001, right=0.9999, top=0.9999,
                    wspace=0.01, hspace=0.01)
for r, regtype in enumerate(['item']+regtypes):
    # flip frequencies to plot low frequencies at the bottom:
    for f, freq in enumerate(corr_t['egi'].frequency.values[::-1]):
        topoplot(values=corr_t['egi'].sel(regtype=regtype, frequency=freq),
                 sensors=sensors['egi'], axes=axs[f, r], sensor_props=dict(marker=''),
                 head_props=dict(head_linewidth=1,nose_linwidth=1,ear_linewidth=1),
                 vmin=-maxval, vmax=maxval, cmap='bwr')
fig.savefig('./figs/corrs_all_egi.pdf')

maxval = 7
fig, axs = plt.subplots(15, 5, sharex=True, sharey=True, figsize=(5, 11), subplot_kw=dict(aspect='equal'))
plt.subplots_adjust(left=0.0001, bottom=0.0001, right=0.9999, top=0.9999,
                    wspace=0.01, hspace=0.01)
for r, regtype in enumerate(['item']+regtypes):
    # flip frequencies to plot low frequencies at the bottom:
    for f, freq in enumerate(corr_t['biosemi'].frequency.values[::-1]):
        topoplot(values=corr_t['biosemi'].sel(regtype=regtype, frequency=freq),
                 sensors=sensors['biosemi'], axes=axs[f, r], sensor_props=dict(marker=''),
                 head_props=dict(head_linewidth=1,nose_linwidth=1,ear_linewidth=1),
                 vmin=-maxval, vmax=maxval, cmap='bwr')
fig.savefig('./figs/corrs_all_biosemi.pdf')
