import sys
import numpy as np
import os
import h5py
import multiprocessing as mp
from glob import glob
import pandas as pd
import statsmodels.formula.api as smf

# from sklearn import cross_validation
from sklearn.model_selection import LeaveOneGroupOut
# import cPickle as pickle

from sklearn.preprocessing import StandardScaler
# from sklearn.metrics import roc_auc_score
# from sklearn.svm import SVR
from sklearn.linear_model import Ridge

run_on_rhino = True
# run_on_rhino = False

if run_on_rhino:
    num_mp_procs = 0
    rhino_mount = ''
else:
    num_mp_procs = 0
    rhino_mount = '/home/ctw/fusemounts/rhino'
homedir = rhino_mount+'/scratch/cweidema/LTP/ltpFR2'


pow_path_ltpFR2 = (
    rhino_mount +
    '/scratch/cweidema/LTP/ltpFR2/ltpFR2_preprocessed/pow15freq2_200hz/')


subj_folders = np.sort(glob(os.path.join(pow_path_ltpFR2, 'LTP???')))


regressorfolder_name = 'ridge_nestedXval_pred_residuals_catpredictors_logit'
regressor_folder = (os.path.join(
    homedir, 'ltpFR2_regressors/listvar/',
    regressorfolder_name, 'hdf5_files'))

SHUFFLES = 20

ALPHAS = np.array([4**i for i in np.arange(0., 9.)])

time_start = 0.3
time_end = 1.6

ridge_params = {'alpha': np.nan, 'fit_intercept': True, 'normalize': False,
                'copy_X': True, 'max_iter': None, 'tol': 0.001,
                'solver': 'auto', 'random_state': None}


FREQS = np.array([2., 2.77899099, 3.86139546, 5.36539159, 7.45518744,
                  10.35894936, 14.39371346, 20., 27.78990989, 38.61395458,
                  53.65391591, 74.55187441, 103.58949358, 143.9371346, 200.])
CHANS = ['A'+str(i) for i in range(1, 33)]
CHANS.extend(['B'+str(i) for i in range(1, 33)])
CHANS.extend(['C'+str(i) for i in range(1, 33)])
CHANS.extend(['D'+str(i) for i in range(1, 33)])

subjsess_info_path = (rhino_mount +
                      '/data/eeg/scalp/ltp/ltpFR2/cmldb_sess_info_ltpFR2.txt')
subjsess_info = pd.read_table(subjsess_info_path)
subjsess_info.loc[:, 'session'] += 1



# For now base recallability calculations on events stored in the item
# classifier HDF5 files. This assumes that the item classifiers are
# run first, but ensures that exactly the same events that are used
# for the regressors are the basis of the recallability
# calculation. Given that we're calculating recallability based on
# held-out data anyway, this criterion could be relaxed and we could
# get the events from the power files instead.
classifierfolder_name = 'itemLR_nestedXval'
classifier_folder = (os.path.join(
    homedir, 'ltpFR2_classifiers/itemvar/',
    classifierfolder_name, 'hdf5_files'))
classfiles = np.sort(glob(os.path.join(classifier_folder,
                                       'classifier_info_allfeat_LTP???.hdf5')))

# recallability_df = pd.DataFrame(index=np.array(np.unique(evdf['item']), dtype=np.str))
with h5py.File(classfiles[0], 'r') as classfile:
    recallability_within_df = pd.DataFrame(index=np.array(
        np.unique(classfile['events/item']), dtype=np.str))

# evdf_dict = {}
for cf in classfiles:
    with h5py.File(cf, 'r') as classfile:
        assert(len(np.unique(classfile['events/subject']))==1)
        subj = classfile['events/subject'][0]
        rec_dtypes = [(str(e), classfile['events'][e].dtype)
                      for e in classfile['events']]
        evarray = np.zeros(classfile['events/subject'].shape, rec_dtypes)
        for ev_val in classfile['events']:
            evarray[ev_val] = classfile['events'][ev_val]
        evdf = pd.DataFrame(evarray)
        try:
            recallability_within_df[subj] = np.array(
                [np.mean(evdf[np.array(evdf['item'], np.str) == word][
                    'recalled'])
                 for word in recallability_within_df.index])
        except KeyError:
            recallability_within_df[subj] = np.array(
                [np.mean(evdf[np.array(evdf['item_name'], np.str) == word][
                    'recalled'])
                 for word in recallability_within_df.index])


recallability_heldout_df = pd.DataFrame(index=recallability_within_df.index)
for subj in recallability_within_df:
    recallability_heldout_df[subj.decode()] = np.mean(
        recallability_within_df.loc[
            :, recallability_within_df.columns != subj], 1)






def get_sessfeatures(sessdat, encevsfunc=None, shuffles=SHUFFLES):
    # sanity check that list and serial position numbers are in the right order
    assert(np.all(sessdat['coords']['events']['trial']==np.array(
        [i for i in range(1, 25) for j in range(24)])))
    assert(np.all(
        sessdat['coords']['events']['serialpos'] == np.array(
            list(range(1, 25))*24)))
    assert(np.all(np.isclose(sessdat['coords']['frequency'][()], FREQS)))
    assert(np.all(np.array(sessdat['coords']['channels'][()], np.str) == CHANS))
    data = np.mean(sessdat['data'][
        ..., (sessdat['coords']['time'][:] >= time_start) &
        (sessdat['coords']['time'][:] <= time_end)], 3)
    recalled_dat = data[..., sessdat['coords']['events']['recalled']==1]
    unrecalled_dat = data[..., sessdat['coords']['events']['recalled']==0]
    shuffled_data = []
    for i in range(shuffles):
        np.random.shuffle(recalled_dat.T)
        np.random.shuffle(unrecalled_dat.T)
        shuffled_data.append(np.ones(data.shape)*np.nan)
        shuffled_data[-1][..., sessdat['coords']['events']['recalled']==1] = recalled_dat
        shuffled_data[-1][..., sessdat['coords']['events']['recalled']==0] = unrecalled_dat
    shuffled_data = np.concatenate(shuffled_data, 2)
    features = {}

    # reshape is only valid if above assertions are true:
    data = data.reshape(
        len(sessdat['coords']['channels']),
        len(sessdat['coords']['frequency']),
        len(np.unique(sessdat['coords']['events']['trial'])),
        len(np.unique(sessdat['coords']['events']['serialpos'])))
    orig_dims = len(data.shape)
    shuffled_data = shuffled_data.reshape(
        len(sessdat['coords']['channels']),
        len(sessdat['coords']['frequency']),
        len(np.unique(sessdat['coords']['events']['trial']))*shuffles,
        len(np.unique(sessdat['coords']['events']['serialpos'])))
    orig_dims_shuffled = len(shuffled_data.shape)
    if encevsfunc is None:
        serialpos = np.unique(sessdat['coords']['events']['serialpos'])
    else:
        data = encevsfunc(data, axis=-1)
        shuffled_data = encevsfunc(shuffled_data, axis=-1)
        serialpos = np.atleast_1d(encevsfunc(np.unique(
            sessdat['coords']['events']['serialpos'])))
    if len(data.shape) == orig_dims:
        data = np.moveaxis(data, -2, -1)
    if len(shuffled_data.shape) == orig_dims_shuffled:
        shuffled_data = np.moveaxis(shuffled_data, -2, -1)
    # reshape to feature array:
    features['data_unscaled'] = data.reshape(
        (len(sessdat['coords']['channels']) *
         len(sessdat['coords']['frequency']) * len(serialpos), -1)).T
    features['shuffled_data_unscaled'] = shuffled_data.reshape(
        (len(sessdat['coords']['channels']) *
         len(sessdat['coords']['frequency']) * len(serialpos), -1)).T
    # z transform
    scaler = StandardScaler()
    features['data_scaled'] = scaler.fit_transform(features['data_unscaled'])
    features['data_scalers'] = {'mean':scaler.mean_, 'std': scaler.scale_}
    scaler_shuffled = StandardScaler()
    features['shuffled_data_scaled'] = scaler_shuffled.fit_transform(
        features['shuffled_data_unscaled'])
    features['shuffled_data_scalers'] = {
        'mean': scaler_shuffled.mean_, 'std': scaler_shuffled.scale_}
    if encevsfunc is None:
        features['serialpos'] = list(np.unique(sessdat['coords']['events'][
            'serialpos'])) * (len(sessdat['coords']['channels']) *
                              len(sessdat['coords']['frequency']))
    else:
        features['serialpos'] = list(
            encevsfunc(np.unique(
                sessdat['coords']['events']['serialpos']))) * (
                    len(sessdat['coords']['channels']) *
                    len(sessdat['coords']['frequency']))
    features['channels'] = []
    features['freqs'] = []
    for chan in sessdat['coords']['channels']:
        features['channels'].extend(
            [chan]*(len(sessdat['coords']['frequency']) * len(serialpos)))
        for freq in sessdat['coords']['frequency']:
            features['freqs'].extend([freq] * len(serialpos))
    return features


def get_serialposbins(data, minval=0, maxval=24, step=4,
                      statistic=np.mean, **kwargs):
    dat = np.array(
        [statistic(data[..., i:i+step], **kwargs)
         for i in range(minval, maxval, step)])
    return np.moveaxis(dat, 0, -1)


def logit(prec):
    prec = np.array(prec)
    prec[prec==1] -= 10e-4
    prec[prec==0] += 10e-4
    return np.log(prec/(1-prec))


def regress(subjfolder, RegModel=Ridge, reg_params=ridge_params):
    subj = subjfolder.split('/')[-1]
    if int(subj[-3:]) <= 330:
        return
    print(subj)
    sess_folders = np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_?')))
    sess_folders = np.r_[sess_folders, np.sort(
        glob(os.path.join(pow_path_ltpFR2, subj, 'session_??')))]
    # get rid of Session 24 ("23" because of zero indexing):
    if int(sess_folders[-1].split('_')[-1]) == 23:
        sess_folders = sess_folders[:-1]
    # if not (len(sess_folders) == 24 or
    #         ((len(sess_folders) == 23) and
    #          (sess_folders[-1].split('_')[-1]=='23'))):
    #     return
    if not (len(sess_folders) >= 20):
        return    
    rfilename = os.path.join(regressor_folder,
                             'regressor_info_allfeat_'+subj+'.hdf5')
    if os.path.exists(rfilename):
        print(str(rfilename)+' already exists!')
        return
    # elif os.path.getsize(subjdatfile)<=1000000000:
    #     print(str(subjdatfile)+' too small:'+ str(os.path.getsize(subjdatfile)))
    #     return
    else:
        try:
            regressfile = h5py.File(rfilename, 'w-', libver='latest')
        except:
            print('Could not create: '+str(rfilename))
            return
    sessions = []
    lists = None
    # serialpos = None
    prec = []
    events = {}
    features = None
    if len(sess_folders) == 24:
        sess_folders = sess_folders[:-1]
    for sess_folder in sess_folders:
        sess = sess_folder.split('_')[-1]
        sessdatfile = h5py.File(os.path.join(
            sess_folder, 'pow_'+subj+'_'+sess+'_allchans.hdf5'), 'r')
        if not np.all(np.isfinite(sessdatfile['coords']['events'][
                'recalled'])):
            continue
        if len(np.unique(sessdatfile['coords']['events']['trial'])) != 24:
            continue
        sessfeatures = get_sessfeatures(
            sessdatfile,
            encevsfunc=lambda x, **kwargs: get_serialposbins(
                # x, step=4, **kwargs))
                x, step=24, **kwargs), shuffles=SHUFFLES)
                # x, step=12, **kwargs))
                # x, step=1, **kwargs))        
                # x, step=8, **kwargs))
                # x, step=6, **kwargs))
        prec_tmp = [np.mean(
            sessdatfile['coords']['events']['recalled'][
                sessdatfile['coords']['events']['trial']==lst])
                    for lst in np.unique(
                            sessdatfile['coords']['events']['trial'])]
        if not np.all(np.isfinite(prec_tmp)):
            continue
        prec.extend(prec_tmp)
        sessions.extend([sess] * len(
            np.unique(sessdatfile['coords']['events']['trial'])))
        # sessdat = sessdatfile['data']
        if lists is None:
            lists = np.unique(sessdatfile['coords']['events']['trial'])
            # serialpos = np.unique(sessdatfile['coords']['events']['serialpos'])
            for key in sessdatfile['coords']['events'].dtype.names:
                if sessdatfile['coords']['events'][key].dtype.char == 'S':
                    events[key] = sessdatfile['coords']['events'][key].astype(
                        np.str)
                else:
                    events[key] = sessdatfile['coords']['events'][key]
            # sessdatfile.copy('events', regressfile)
        else:
            assert(np.all(lists == np.unique(
                sessdatfile['coords']['events']['trial'])))
            for key in sessdatfile['coords']['events'].dtype.names:
                if sessdatfile['coords']['events'][key].dtype.char == 'S':
                    events[key] = np.r_[
                        events[key],
                        sessdatfile['coords']['events'][key].astype(np.str)]
                else:
                    events[key] = np.r_[events[key],
                                        sessdatfile['coords']['events'][key]]
            # for evkey in regressfile['events']:
            #     origsize = regressfile['events'][evkey].size
            #     regressfile['events'][evkey].resize(
            #         origsize + sessdatfile['events'][evkey].size, axis=0)
            #     regressfile['events'][evkey][origsize:] =  sessdatfile[
            #         'events'][evkey][:]
        if features is None:
            features = sessfeatures
            # features.pop('data_scaler')
            features.pop('data_unscaled')
            features.pop('shuffled_data_unscaled')
        else:
            features['data_scaled'] = np.r_[features['data_scaled'],
                                            sessfeatures['data_scaled']]
            features['shuffled_data_scaled'] = np.r_[features['shuffled_data_scaled'],
                                                     sessfeatures['shuffled_data_scaled']]
            for key in features['data_scalers']:
                features['data_scalers'][key] = np.r_[
                    np.atleast_2d(features['data_scalers'][key]),
                    np.atleast_2d(sessfeatures['data_scalers'][key])]
                features['shuffled_data_scalers'][key] = np.r_[
                    np.atleast_2d(features['shuffled_data_scalers'][key]),
                    np.atleast_2d(sessfeatures['shuffled_data_scalers'][key])]
        sessdatfile.close()
    sessions = np.int16(sessions)
    num_sess = len(np.unique(sessions))
    # max_sess = 23
    # if max_sess<num_sess:
    #     raise ValueError('max_sess < num_sess: '+ str(max_sess) + ' vs. ' +
    #                      str(num_sess)) 
    num_list = len(lists)
    # num_serialpos = len(serialpos)
    evdf = pd.DataFrame(events)
    # evdf.astype(dtype={
    #     'protocol': 'unicode', 'montage': 'unicode', 'subject': 'unicode',
    #     'item_name': 'unicode', 'type': 'unicode', 'experiment': 'unicode',
    #     'eegfile': 'unicode'})
    # evdf[['protocol', 'montage', 'subject', 'item_name', 'type', 'experiment',
    #       'eegfile']] = evdf[['protocol', 'montage', 'subject', 'item_name',
    #                           'type', 'experiment', 'eegfile']].astype('U256')
    if 'item_name' in events:
        item_key = 'item_name'
    else:
        item_key = 'item'
    evdf['recallability'] = recallability_heldout_df.loc[
        evdf[item_key], subj].reset_index().loc[:, subj]
    evdf_list = evdf.loc[:, ['subject', 'session', 'trial', 'recalled',
                             'recallability']].groupby(
                                 ['subject', 'session', 'trial']).agg(
                                     np.mean).reset_index()
    
    evsessinfo = pd.merge(left=evdf_list, right=subjsess_info, how='left',
                          left_on=('subject', 'session'),
                          right_on=('subject', 'session'))
    evsessinfo['recalled_logit'] = logit(evsessinfo.recalled)
    evsessinfo['hrs_from_noon'] = -(12 - (evsessinfo.start_time % 100) /
                                    60 - evsessinfo.start_time//100)
    evsessinfo['list'] = ((evsessinfo['trial']-1) % 8) + 1
    evsessinfo['list_squared'] = evsessinfo['list'] ** 2
    evsessinfo['block'] = ((evsessinfo['trial']-1) // 8) + 1
    regresults = {
        'sesslist': smf.ols("recalled_logit ~ C(session) + C(trial) + recallability",
                            evsessinfo).fit(),
        'sess': smf.ols("recalled_logit ~ C(session)",
                        evsessinfo).fit(),
        'list': smf.ols("recalled_logit ~ C(trial) + recallability",
                        evsessinfo).fit()}
    # regresults['intersess'] = smf.ols(
    #     'recalled_logit ~ 1 + session + sleep + alertness + hrs_from_noon',
    #     data=evsessinfo).fit()
    # regresults['interlist'] = smf.ols(
    #     'recalled_logit ~ 1 + session + block + list + list_squared + ' +
    #     'recallability', data=evsessinfo).fit()
    # regresults['sesslist'] = smf.ols(
    #     'recalled_logit ~ 1 + session + block + list + list_squared + ' +
    #     'recallability + sleep + alertness + hrs_from_noon',
    #     data=evsessinfo).fit()
    for regmodel in regresults:
        evsessinfo['residuals_'+regmodel] = regresults[regmodel].resid
        regressfile.create_dataset(
            'residuals_'+regmodel, data=evsessinfo['residuals_'+regmodel],
            compression="gzip",compression_opts=9)
        evsessinfo['prediction_from_responses_'+regmodel] = regresults[
            regmodel].predict(evsessinfo)
        regressfile.create_dataset(
            'prediction_from_responses_'+regmodel,
            data=evsessinfo['prediction_from_responses_'+regmodel],
            dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'prediction_'+regmodel+'_res', data=np.ones((num_sess, num_list))*np.nan,
            dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'prediction_'+regmodel+'_pred', data=np.ones((num_sess, num_list))*np.nan,
            dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'prediction_'+regmodel+'_res_shuffled',
            data=np.ones((num_sess, num_list*SHUFFLES))*np.nan,
            dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'prediction_'+regmodel+'_pred_shuffled',
            data=np.ones((num_sess, num_list*SHUFFLES))*np.nan,
            dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile['residuals_'+regmodel].attrs['subj'] = subj
        regressfile['prediction_from_responses_'+regmodel].attrs['subj'] = subj
        regressfile['prediction_'+regmodel+'_res'].attrs['subj'] = subj
        regressfile['prediction_'+regmodel+'_pred'].attrs['subj'] = subj
        regressfile['prediction_'+regmodel+'_res_shuffled'].attrs['subj'] = subj
        regressfile['prediction_'+regmodel+'_pred_shuffled'].attrs['subj'] = subj
        regressfile.create_dataset(
            'r2score_'+regmodel+'_res', data=np.ones(num_sess)*np.nan, dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'r2score_'+regmodel+'_pred', data=np.ones(num_sess)*np.nan, dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'bestAlphas_'+regmodel+'_res', data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'bestAlphas_'+regmodel+'_pred', data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'r2score_'+regmodel+'_res_shuffled', data=np.ones(num_sess)*np.nan, dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'r2score_'+regmodel+'_pred_shuffled', data=np.ones(num_sess)*np.nan, dtype='float64',
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'bestAlphas_'+regmodel+'_res_shuffled', data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'bestAlphas_'+regmodel+'_pred_shuffled', data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'prediction_raw', data=np.ones((num_sess, num_list))*np.nan, dtype='float64',
        compression="gzip", compression_opts=9)
    regressfile['prediction_raw'].attrs['subj'] = subj
    regressfile.create_dataset(
        'r2score_raw', data=np.ones(num_sess)*np.nan, dtype='float64',
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'bestAlphas_raw', data=np.ones(num_sess)*np.nan,
        dtype='float64', compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'prediction_raw_shuffled',
        data=np.ones((num_sess, num_list*SHUFFLES))*np.nan, dtype='float64',
        compression="gzip", compression_opts=9)
    regressfile['prediction_raw_shuffled'].attrs['subj'] = subj
    regressfile.create_dataset(
        'r2score_raw_shuffled', data=np.ones(num_sess)*np.nan, dtype='float64',
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'bestAlphas_raw_shuffled', data=np.ones(num_sess)*np.nan,
        dtype='float64', compression="gzip", compression_opts=9)

    for key in events:
        if events[key].dtype.char == 'U':
            regressfile.create_dataset(
                'events/'+key, data=np.array([e.encode()
                                              for e in events[key]]),
                compression="gzip", compression_opts=9)
        else:
            regressfile.create_dataset(
                'events/'+key, data=events[key], compression="gzip",
                compression_opts=9)
    regressfile.create_dataset(
        'events/recallability', data=evdf['recallability'],
        compression="gzip", compression_opts=9)
    
    regressfile.create_dataset(
        'recalled_logit', data=evsessinfo['recalled_logit'],
        compression="gzip", compression_opts=9)
    # regressfile.create_dataset(
    #     'recalled_logit', data=np.ones(max_sess*num_list*num_serialpos)*np.nan,
    #     compression="gzip", compression_opts=9)
    # sess_labels = []
    # for sess in np.arange(max_sess):
    #     sess_labels += [sess] * num_list*num_serialpos
    # sess_labels = np.array(sess_labels)
    # for sess in np.arange(max_sess):
    #     if np.sum(evsessinfo['session'] == sess) == np.sum(sess_labels == sess):
    #         regressfile['recalled_logit'][sess_labels == sess] = evsessinfo[
    #             'recalled_logit'][evsessinfo['session'] == sess]
    #     elif np.sum(evsessinfo['session'] == sess) != 0:
    #         raise ValueError('Sums mismatch: ' + str(np.sum(
    #             evsessinfo['session'] == sess)) + ' vs. ' +
    #                          str(np.sum(sess_labels == sess)))
    regressfile.create_dataset(
        'session', data=evsessinfo['session'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'sleep', data=evsessinfo['sleep'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'alertness', data=evsessinfo['alertness'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'hrs_from_noon', data=evsessinfo['hrs_from_noon'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'block', data=evsessinfo['block'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'list', data=evsessinfo['list'], compression="gzip",
        compression_opts=9)
    regressfile.create_dataset(
        'recallability_list', data=evsessinfo['recallability'],
        compression="gzip", compression_opts=9)
    for rp in reg_params:
        if reg_params[rp] is None:
            for regmodel in regresults:
                regressfile['prediction_'+regmodel+'_res'].attrs[rp] = 'None'
                regressfile['prediction_'+regmodel+'_pred'].attrs[rp] = 'None'
                regressfile['prediction_'+regmodel+'_res_shuffled'].attrs[rp] = 'None'
                regressfile['prediction_'+regmodel+'_pred_shuffled'].attrs[rp] = 'None'
            regressfile['prediction_raw'].attrs[rp] = 'None'
            regressfile['prediction_raw_shuffled'].attrs[rp] = 'None'
        else:
            for regmodel in regresults:
                regressfile['prediction_'+regmodel+'_res'].attrs[rp] = reg_params[rp]
                regressfile['prediction_'+regmodel+'_pred'].attrs[rp] = reg_params[rp]
                regressfile['prediction_'+regmodel+'_res_shuffled'].attrs[rp] = reg_params[rp]
                regressfile['prediction_'+regmodel+'_pred_shuffled'].attrs[rp] = reg_params[rp]
            regressfile['prediction_raw'].attrs[rp] = reg_params[rp]
            regressfile['prediction_raw_shuffled'].attrs[rp] = reg_params[rp]
    features_dat = regressfile.create_dataset(
        'features_dat', data=features['data_scaled'],
        compression="gzip", compression_opts=9)
    features_shuffled_dat = regressfile.create_dataset(
        'features_shuffled_dat', data=features['shuffled_data_scaled'],
        compression="gzip", compression_opts=9)
    for alpha in ALPHAS:
        for regmodel in regresults:
            regressfile.create_dataset(
                'alphaRs_'+regmodel+'_res'+'/'+str(alpha), data=np.ones(
                    num_sess)*np.nan,
                dtype='float64', compression="gzip", compression_opts=9)
            regressfile.create_dataset(
                'alphaRs_'+regmodel+'_pred'+'/'+str(alpha), data=np.ones(
                    num_sess)*np.nan,
                dtype='float64', compression="gzip", compression_opts=9)
            regressfile.create_dataset(
                'alphaRs_'+regmodel+'_res_shuffled'+'/'+str(alpha), data=np.ones(
                    num_sess)*np.nan,
                dtype='float64', compression="gzip", compression_opts=9)
            regressfile.create_dataset(
                'alphaRs_'+regmodel+'_pred_shuffled'+'/'+str(alpha), data=np.ones(
                    num_sess)*np.nan,
                dtype='float64', compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'alphaRs_raw/'+str(alpha), data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'alphaRs_raw_shuffled/'+str(alpha), data=np.ones(num_sess)*np.nan,
            dtype='float64', compression="gzip", compression_opts=9)
    for key in features['data_scalers']:
        regressfile.create_dataset(
            'features_scalers/'+key, data=features['data_scalers'][key],
            compression="gzip", compression_opts=9)
        regressfile.create_dataset(
            'features_shuffled_scalers/'+key, data=features['shuffled_data_scalers'][key],
            compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_chans', data=features['channels'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_freqs', data=features['freqs'],
        compression="gzip", compression_opts=9)
    regressfile.create_dataset(
        'features_serialpos', data=features['serialpos'],
        compression="gzip", compression_opts=9)
    dv_dict = {regmodel+'_res': regressfile['residuals_'+regmodel]
               for regmodel in regresults}
    dv_dict.update(
        {regmodel+'_pred': regressfile['prediction_from_responses_'+regmodel]
         for regmodel in regresults})
    dv_dict['raw'] = regressfile['recalled_logit']
    # dv_dict = {'logit': recalled_logit,
    #            'residuals_logit': recalled_residuals_logit}
    # r2_dict = {'logit': r2_score_logit,
    #            'residuals_logit': r2_score_residuals_logit}
    # prediction_dict = {'logit': prediction_logit,
    #                    'residuals_logit': prediction_residuals_logit}
    logo = LeaveOneGroupOut()
    for shuffled in [False, True]:
        if shuffled:
            feats = features_shuffled_dat
            multiplier = SHUFFLES
            shuffsuffix = '_shuffled'
            print('shuffled')
        else:
            feats = features_dat
            multiplier = 1
            shuffsuffix = ''
            print('unshuffled')
        for dv in dv_dict:
            for session_i, (train, test) in enumerate(logo.split(
                    feats[np.tile(np.isfinite(dv_dict[dv]), multiplier),:],
                    np.tile(dv_dict[dv][np.isfinite(dv_dict[dv])], multiplier),
                    np.tile(sessions[np.isfinite(dv_dict[dv])], multiplier))):
                session = np.unique(sessions[np.isfinite(dv_dict[dv])])[
                    session_i]
                logo_nested = LeaveOneGroupOut()
                for alpha in ALPHAS:
                    print(dv, alpha)
                    nested_preds = np.empty((num_sess-1, num_list*multiplier))
                    nested_preds.fill(np.nan)
                    nested_r2 = np.empty((num_sess-1,))
                    nested_r2.fill(np.nan)
                    nested_recalled = np.empty((num_sess-1, num_list*multiplier))
                    nested_recalled.fill(np.nan)
                    for session_nested, (train_nested, test_nested) in enumerate(
                            logo_nested.split(
                                feats[np.tile(np.isfinite(dv_dict[dv]), multiplier),:][train],
                                np.tile(dv_dict[dv][np.isfinite(dv_dict[dv])], multiplier)[train],
                                np.tile(sessions[np.isfinite(dv_dict[dv])], multiplier)[train])):
                        params = reg_params.copy()
                        params['alpha'] = alpha
                        reg = RegModel(**params)
                        tmpreg = reg.fit(
                            feats[np.tile(np.isfinite(dv_dict[dv]), multiplier),:][train][
                                train_nested], np.tile(dv_dict[dv][np.isfinite(
                                    dv_dict[dv])],multiplier)[train][train_nested])
                        nested_preds[session_nested] = tmpreg.predict(
                            feats[np.tile(np.isfinite(dv_dict[dv]), multiplier),:][train][
                                test_nested])
                        nested_r2[session_nested] = tmpreg.score(
                            feats[np.tile(np.isfinite(dv_dict[dv]), multiplier),:][train][
                                test_nested],
                            np.tile(dv_dict[dv][np.isfinite(dv_dict[dv])], multiplier)[train][
                                test_nested])
                        nested_recalled[session_nested] = np.tile(dv_dict[dv][
                            np.isfinite(dv_dict[dv])], multiplier)[train][test_nested]
                    regressfile['alphaRs_'+dv+shuffsuffix+'/'+str(alpha)][
                        np.unique(regressfile['session'])==session] = np.corrcoef(
                            np.ravel(nested_preds),
                            np.ravel(nested_recalled))[0, 1]
                    # alpha_rs[alpha] = np.corrcoef(np.ravel(nested_preds), np.ravel(nested_logitprec))[0, 1]
                # best_alpha = max(alpha_rs, key=lambda key: alpha_rs[key])
                best_alpha = np.float(max(
                    regressfile['alphaRs_'+dv+shuffsuffix].keys(),
                    key=lambda key: regressfile['alphaRs_'+dv+shuffsuffix+'/'+key][
                        np.unique(regressfile['session'])==session]))
                regressfile['bestAlphas_'+dv+shuffsuffix][
                    np.unique(regressfile['session'])==session] = best_alpha
                params = reg_params.copy()
                params['alpha'] = best_alpha
                reg = RegModel(**params)
                tmpreg = reg.fit(feats[np.tile(np.isfinite(dv_dict[dv]), multiplier),:][train],
                                 np.tile(dv_dict[dv][np.isfinite(dv_dict[dv])], multiplier)[train])
                sess_indx = np.where(np.unique(
                    regressfile['session'])==session)[0][0] 
                regressfile['prediction_'+dv+shuffsuffix][
                    sess_indx] = tmpreg.predict(
                    feats[np.tile(np.isfinite(dv_dict[dv]), multiplier), :][test])
                regressfile['r2score_'+dv+shuffsuffix][
                    np.unique(regressfile['session'])==session] = tmpreg.score(
                    feats[np.tile(np.isfinite(dv_dict[dv]), multiplier),:][test],
                    np.tile(dv_dict[dv][np.isfinite(dv_dict[dv])], multiplier)[test])
                print(subj, session)
            try:
                print(dv, np.corrcoef(
                    np.ravel(regressfile['prediction_'+dv+shuffsuffix])[
                        np.isfinite(np.ravel(regressfile['prediction_'+dv+shuffsuffix]))],
                    np.tile(dv_dict[dv][np.isfinite(dv_dict[dv])], multiplier)))
            except (IndexError, ValueError) as e:
                print(e) 
    try:
        print([(dv, np.corrcoef(
            np.ravel(regressfile['prediction_'+dv])[
                np.isfinite(np.ravel(regressfile['prediction_'+dv]))],
            dv_dict[dv][np.isfinite(dv_dict[dv])])) for dv in dv_dict])
    except (IndexError, ValueError) as e:
        print(e)
    regressfile.close()


for subjfolder in subj_folders:
    regress(subjfolder, RegModel=Ridge, reg_params=ridge_params)
